# Functions for open source XEN

sub type_xen_desc
{
return $text{'xen_typedesc'};
}

# type_xen_parent(&server, [&all-servers])
# Returns the parent system that hosts this xen
sub type_xen_parent
{
my ($server, $servers) = @_;
if ($servers) {
	my ($vhost) = grep { $_->{'id'} == $server->{'xen_on'} } @$servers;
	return $vhost;
	}
else {
	return &get_managed_server($server->{'xen_on'});
	}
}

sub type_xen_must_image
{
return 1;			# Xen instances always come from an image
}

sub type_xen_can_manual_image
{
return 1;			# An image can be created from a CD or raw disk
}

sub type_xen_is_virtual
{
return 1;
}

sub type_xen_can_transfer
{
my ($server) = @_;
return $server->{'xen_mount'} && !$server->{'nohost'} ||	# Via host
       $server->{'status'} eq 'nowebmin' ||	# Can transfer if SSH is up
       $server->{'status'} eq 'downwebmin' ||
       $server->{'status'} eq 'nowebminlogin' ||
       $server->{'status'} eq 'novirt' ||
       $server->{'status'} eq 'virt';
}

# Can change the password if totally down, or can SSH in
sub type_xen_can_password
{
my ($server) = @_;
return $server->{'status'} ne 'nohost' &&
       $server->{'status'} ne 'nossh' &&
       $server->{'status'} ne 'nosshlogin';
}

# Xen instances can have a hostname separate from the DNS name
sub type_xen_can_custom_hostname
{
return 1;
}

# Can reset, if the host is up
sub type_xen_can_reset
{
my ($server) = @_;
return $server->{'status'} ne 'nohost';
}

# Empty Xen instances can be created
sub type_xen_can_create_empty
{
return 1;
}

# Returns links for creating and adding xens (if this system supports
# them, or remote systems do)
sub type_xen_create_links
{
my @vhosts = grep { $_->{'xen_host'} } &list_managed_servers();
my @rv;
if (@vhosts) {
	push(@rv, { 'create' => 0,
		    'desc' => $text{'xen_addlink'} });
	push(@rv, { 'create' => 1,
		    'desc' => $text{'xen_createlink'} });
	}
return @rv;
}

# Returns a link to the page list xen-capable systems
sub type_xen_global_links
{
if ($module_info{'onehost'} eq 'xen') {
	# Page for just this host
	return ( { 'link' => 'edit_xen.cgi?id=0',
		   'icon' => 'images/xens.gif',
		   'title' => $text{'xen_editone'},
		   'cat' => 'hosts' } );
	}
else {
	# List of Xen hosts
	return ( { 'link' => 'list_xens.cgi',
		   'icon' => 'images/xens.gif',
		   'title' => $text{'xens_title'},
		   'cat' => 'hosts' } );
	}
}

# type_xen_add_form()
# Returns inputs for adding an existing VServer. This is just a matter of
# selecting the host and the server.
sub type_xen_add_form
{
my $rv;

# Hosting system
my @vhosts = grep { $_->{'xen_host'} } &list_managed_servers();
$rv .= &ui_table_row($text{'add_xenhost'},
		&ui_select("xen_on", $vhosts[0]->{'id'},
			[ map { [ $_->{'id'}, $_->{'host'} ] } @vhosts ]));

if (@vhosts == 1) {
	# Can just show a drop-down of existing xen instances, except those we
	# are already hosting)
	my $instances = &list_xen_instances($vhosts[0]);
	my %already = map { $_->{'xen'}, $_ }
			     &list_managed_servers();
	$rv .= &ui_table_row($text{'add_xeninstance'},
	    &ui_select("xen_instance", undef,
		[ map { [ $_->{'instance'} ] }
		      grep { !$already{$_->{'instance'}} } @$instances] ));
	}
else {
	# Need to show xen name field, with popup
	$rv .= &ui_table_row($text{'add_xeninstance'},
	     &ui_textbox("xen_instance", undef, 20)." ".
	     &xen_instance_chooser_button("xen_instance",
					      "xen_on"));
	}

# Configuration file on host
$rv .= &ui_table_row($text{'add_xenconf'},
		&ui_textbox("xen_conf", undef, 60));

return $rv;
}

# type_xen_add_parse(&server, &in)
# Parse inputs from type_xen_add_form
sub type_xen_add_parse
{
my ($server, $in) = @_;

# Save Xen host
$server->{'xen_on'} = $in->{'xen_on'};
my $vhost = &type_xen_parent($server);

# Check and save Xen instance
my $instances = &list_xen_instances($vhost);
$in->{'xen_instance'} || &error($text{'add_exeninstance'});
my ($got) = grep { $_->{'instance'} eq $in->{'xen_instance'} } @$instances;
$got || &error(&text('add_exeninstance2', $vhost->{'host'}));
$server->{'xen'} = $in->{'xen_instance'};

# Check and save Xen config file
$in->{'xen_conf'} || &error($text{'add_exenconf'});
my $temp = &transname();
my $err = &get_file_from_server($vhost, $in->{'xen_conf'}, $temp);
$err && &error($text{'add_exenconf2'});
my $conf = &parse_xen_config($temp);
my $kernel = &xen_find_value("kernel", $conf);
my $bootloader = &xen_find_value("bootloader", $conf);
my $builder = &xen_find_value("builder", $conf);
if (!$bootloader && !$kernel && !$builder) {
	&error($text{'add_exenconf4'});
	}
if ($builder eq "hvm") {
	$server->{'xen_hvm'} = 1;
	}
elsif ($bootloader =~ /pygrub/) {
	$server->{'xen_pygrub'} = $bootloader;
	$server->{'xen_bootloader'} = $bootloader;
	}
elsif ($kernel =~ /pv-grub/) {
	$server->{'xen_pygrub'} = 1;
	$server->{'xen_bootloader'} = $kernel;
	}
my $cname = &xen_find_value("name", $conf);
!$cname || $cname eq $server->{'xen'} || &error($text{'add_exenconf3'});
$server->{'xen_conf'} = $in->{'xen_conf'};
$server->{'ip_addresses'} = &xen_find_value("address", $conf);

# Get root disk from .cfg file
&set_xen_filesystem($server, $conf);

# Work out vif name from config
my $vifs = &xen_find_value("vif", $conf);
if (@$vifs) {
	my %info = map { split(/=/, $_) } split(/\s*,\s*/, $vifs->[0]);
	if ($info{'vifname'}) {
		$server->{'xen_vif'} = $info{'vifname'};
		}
	}
}

# set_xen_filesystem(&server, &conf)
# Updates the xen_filesystem field in a server object based on disks, if
# possible
sub set_xen_filesystem
{
my ($server, $conf) = @_;
my $disks = &xen_find_value("disk", $conf);
my $root = &xen_find_value("root", $conf);
if ($root =~ /^\/dev\/(\S+)/) {
	$root = $1;
	}
else {
	$root = undef;
	}
if ($root &&
    $disks->[0] =~ /^(file|phy|tap:aio):(\S+),(\Q$root\E),(\S+)$/) {
	# Can get root disk from root directive
	$server->{'xen_filesystem'} = $2;
	$server->{'xen_wholedisk'} = $3 =~ /\d+$/ ? 0 : 1;
	}
elsif ($disks->[0] =~ /^(file|phy|tap:aio):(\S+),(\S+),(\S+)$/) {
	# Just assume first
	$server->{'xen_filesystem'} = $2;
	$server->{'xen_wholedisk'} = $3 =~ /\d+$/ ? 0 : 1;
	}
if ($server->{'xen_filesystem'} =~ /^\/dev\/(\S+)\/(\S+)$/) {
	# Looks like an LVM root
	$server->{'xen_filesystem_vg'} = $1;
	$server->{'xen_filesystem_lv'} = $2;
	}
elsif ($server->{'xen_filesystem'} =~ /^\/dev\/disk\/by-path\/ip-(\S+):(\d+)-iscsi-(\S+):(\S+)-lun-(\d+)$/) {
	# On an ISCSI server
        my ($ip, $port, $name, $target, $lun) = ($1, $2, $3, $4, $5);
        my $iscsiserver = &get_managed_server_by("ip", $ip);
        if ($iscsiserver) {
                $server->{'xen_filesystem_iscsi'} = $iscsiserver->{'id'};
                }
        $server->{'xen_filesystem_name'} = $name;
        $server->{'xen_filesystem_target'} = $target;
        $server->{'xen_filesystem_ip'} = $ip;
        $server->{'xen_filesystem_port'} = $port;
        }
if ($server->{'xen_hvm'}) {
	$server->{'xen_wholedisk'} = 1;
	}
return $server->{'xen_filesystem'};
}

# type_xen_reboot(&server)
# Use the xm reboot command
sub type_xen_reboot
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
delete($list_xen_instances_cache{$vhost->{'id'}});
my @st = &stat_file_on_server($vhost, $server->{'xen_conf'});
if (@st && $st[9] > $server->{'xen_lastboot'} ||
    !&can_ssh_status($server)) {
	# The config file was modified since the last boot. We need to do
	# a full stop/start to apply it
	my $err = &type_xen_shutdown($server);
	return $err if ($err);
	my ($ok, $err) = &type_xen_startup($server);
	return $ok ? undef : $err;
	}
else {
	# Just do a reboot
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm reboot $server->{'xen'} -w");
	&compute_ip_addresses($server);
	return $err ? $err : $ex ? $out : undef;
	}
}

# type_xen_shutdown(&server, [force])
# Use the xm shutdown command to shut down
sub type_xen_shutdown
{
my ($server, $force) = @_;
my $vhost = &type_xen_parent($server);
if (&can_ssh_status($server)) {
	# Sync filesystems first, to avoid corruption
	&execute_command_on_server($server, "sync ; sync ; sync");
	}
delete($list_xen_instances_cache{$vhost->{'id'}});

my ($out, $ex, $err, $status);
if (&can_ssh_status($server) && !$force && $server->{'xen_hvm'}) {
	# A xen shutdown doesn't work for HVM, so try to shutdown on the host
	($out, $ex, $err) = &execute_command_on_server($server,
				  "poweroff >/dev/null 2>&1 </dev/null &");
	return $err if ($err);
	return $out if ($ex);

	# Wait 30 seconds for shutdown
	my $start = time();
	while(time() - $start < 30) {
		last if (!&ping_server_host($server->{'ip'}));
		sleep(1);
		}
	}
elsif (!$force) {
	# Try a clean shutdown
	($out, $ex, $err, $status) = &run_xen_api_command($vhost,
		"xm shutdown $server->{'xen'} -w", 60);
	}

# Check if still alive
my $insts = &list_xen_instances($vhost, 1);
my $alive;
if (ref($insts)) {
	($alive) = grep { $_->{'instance'} eq $server->{'xen'} } @$insts;
	}
if ($status == 4 || $force || $alive) {
	# shutdown timed out or failed .. escalate to destruction
	($out, $ex, $err, $status) = &run_xen_api_command($vhost,
		$alive->{'zombie'} ?
			"xm destroy Zombie-$server->{'xen'}" :
			"xm destroy $server->{'xen'}");
	}

# Wait 5 seconds for real shutdown
my $tries = 0;
while($tries < 5) {
	my $insts = &list_xen_instances($vhost, 1);
	my $alive;
	if (ref($insts)) {
		($alive) = grep { $_->{'instance'} eq $server->{'xen'} }
				@$insts;
		last if (!$alive);
		}
	$tries++;
	sleep(1);
	}
return $err ? $err : $ex ? $out : undef;
}

# type_xen_startup(&server, [no-ping])
# Use the xm create command to start up, and wait till it looks like init
# started OK. This has to be done in a pty, as the Xen console won't work
# if it thinks it is writing to a pipe or file.
sub type_xen_startup
{
my ($server, $noping) = @_;
if ($server->{'ssh_nologin'} && $server->{'ssh_noping'}) {
        $noping = 1;
        }

# If on iSCSI, re-connect any disconnected disks
if ($server->{'xen_filesystem_iscsi'} ne '') {
        my $disks = &type_xen_list_disks($server, 0);
        if (ref($disks)) {
                foreach my $d (@$disks) {
                        if ($d->{'iscsi_server'} ne '' &&
                            $d->{'iscsi_connect'} == 0) {
                                &connect_iscsi_disk($server, $d);
                                }
                        }
                }
        }

my $vhost = &type_xen_parent($server);
delete($list_xen_instances_cache{$vhost->{'id'}});
my $conf = &get_xen_config($server, 1);
my $bootloader = &xen_find_value("bootloader", $conf);
my ($pinged, $sshed);
my $xenver = &get_xen_software_version($vhost);
my $cmd = $vhost->{'xen_api_type'} eq 'xl' ?
		"xl create $server->{'xen_conf'}" :
	  $xenver >= 4.2 ?
		"xm start -c $server->{'xen'}" :
		"xm create -c $server->{'xen_conf'}";
if ($xenver >= 4.2 && $vhost->{'xen_api_type'} ne 'xl') {
	# In case this VM hasn't been registered before, do it now
	&run_xen_api_command($vhost, "xm new $server->{'xen_conf'}");
	}
if ($server->{'xen_vcpus'}) {
	my @vcpus = map { { 'num' => $_ } } (0..($server->{'xen_vcpus'}-1));
	&type_xen_save_vcpus($server, \@vcpus);
	}
if (!$bootloader &&
    !$server->{'xen_hvm'} &&
    &has_command_on_server($vhost, "perl") &&
    !&put_file_to_server($vhost, "$module_root_directory/pty.pl",
				 "/usr/bin/pty.pl")) {
	if (!$server->{'xen_console'}) {
		$server->{'xen_console'} = $server->{'xen_conf'};
		$server->{'xen_console'} =~ s/\.cfg$//;
		$server->{'xen_console'} .= ".console";
		}
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"(perl /usr/bin/pty.pl $cmd ; echo SERVER-MANAGER-COMPLETE) >$server->{'xen_console'} 2>&1 </dev/null &");
	return (0, $err) if ($err);
	return (0, $out) if ($ex);
	my $tries = 0;
	my $pid;
	while($tries++ < 60) {
		sleep(1);
		my $data = &get_file_data_from_server($vhost,
					$server->{'xen_console'});
		if ($data =~ /PTY\s+PID:\s+(\d+)/) {
			$pid = $1;	# got PID of xm create command
			}
		if ($data =~ /SERVER-MANAGER-COMPLETE/) {
			# xm command exited, which indicates a possible
			# boot failure. check if the VM actually got launched.
			$data =~ s/SERVER-MANAGER-COMPLETE\n//g;
			my $instances = &list_xen_instances($vhost, 1);
			next if (!ref($instances));
			my ($got) = grep { $_->{'instance'} eq
					   $server->{'xen'} } @$instances;
			if ($got) {
				# Seems to have started OK
				last;
				}
			return (0, "<pre>".&html_escape($data)."</pre>");
			}
		elsif ($data =~ /INIT:/) {
			# init started, so it looks like we are mostly done.
			last;
			}
		}
	&execute_command_on_server($vhost, "kill $pid") if ($pid);
	}
else {
	# Since the Xen host doesn't have Perl or the instance isn't going to
	# output boot messages, we can only call xm create
	$cmd =~ s/ -c//;
	my ($out, $ex, $err) = &run_xen_api_command($vhost, $cmd);
	return (0, $err) if ($err);
	return (0, $out) if ($ex);

	# Wait for xm list to find it, for up to 30 seconds
	my $tries = 0;
	my $found = 0;
	while($tries++ < 30) {
		my $instances = &list_xen_instances($vhost, 1);
		if (ref($instances)) {
			my ($got) = grep { $_->{'instance'} eq
					   $server->{'xen'} } @$instances;
			if ($got) {
				$found = 1;
				last;
				}
			}
		}
	return (0, &text('xen_estartxm', 30)) if (!$found);
	}

# Wait up to 60 seconds for ping
if (!$noping) {
	$pinged = &wait_for_ping($server);
	}

# Wait for SSH to come up
if (defined($pinged) && $pinged ne '0' && !$server->{'ssh_nologin'}) {
	$sshed = &wait_for_ssh($server);
	}

# If a CPU credit is set, apply it now
if ($server->{'xen_credit_cpu'}) {
	&run_xen_api_command($vhost,
		"xm sched-credit -d $server->{'xen'} ".
		"-c $server->{'xen_credit_cpu'}");
	}
if ($server->{'xen_credit_weight'}) {
	&run_xen_api_command($vhost,
		"xm sched-credit -d $server->{'xen'} ".
		"-w $server->{'xen_credit_weight'}");
	}

# Record the boot time. This can be later used by the reboot function
# to check if a shutdown / startup is needed
$server->{'xen_lastboot'} = time();

# Re-check which IPs are really active
if ($sshed) {
	&compute_ip_addresses($server);
	}

# Return an error if something went wrong
if (defined($pinged) && !$pinged) {
	return (2, &text('xen_estartping', $config{'ping_wait_timeout'}));
	}
if (defined($sshed) && !$sshed) {
	return (3, &text('xen_estartssh', $config{'ping_wait_timeout'}));
	}
return (1, undef);
}

# type_xen_depends(&parent, &vserver)
# Returns 1 if the Xen instance hosted by the given parent
sub type_xen_depends
{
my ($parent, $server) = @_;
return $server->{'xen_on'} eq $parent->{'id'};
}

# type_xen_check_login(&server)
# Check if the Xen instance is up, and if we can SSH in. Returns a status
# code and error message
sub type_xen_check_login
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
if (!$vhost) {
	return (5, "Xen host does not exist!");
	}
if ($vhost->{'status'} eq 'down' ||
    $vhost->{'status'} eq 'nossh' ||
    $vhost->{'status'} eq 'nohost' ||
    $vhost->{'status'} eq 'nosshlogin') {
	return (5, "Xen host $vhost->{'host'} is down");
	}
my $instances = &list_xen_instances($vhost);
if (!ref($instances)) {
	# Host may be down .. check now, so that the status goes to "host down"
	# immediately, and failovers work
	my $vping = &ping_server_host($vhost->{'ip'});
	if ($vping) {
		return (1, "Could not fetch Xen instances : $instances");
		}
	else {
		return (5, "Could not fetch Xen instances, host is probably ".
			   "down : $instances");
		}
	}
my ($got) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
if (!$got) {
	return (6, "Xen instance is down");
	}
elsif ($got->{'state'} eq 'p') {
	return (8, "Xen instance is paused");
	}
#elsif ($got->{'state'} ne 'r' && $got->{'state'} ne 'b') {
#	return (1, "Xen instance is hung (state $got->{'state'})");
#	}

# Tell check_server_ssh_login that we need to SSH
return ( );
}

# type_xen_post_refresh(&server, regenerate)
# Called after a refresh to compute total disk size, if needed
sub type_xen_post_refresh
{
my ($server, $regen) = @_;

# If running, cannot be mounted
if ($server->{'status'} ne 'down' &&
    $server->{'status'} ne 'nohost') {
	delete($server->{'xen_mount'});
	}

if ($server->{'xen_disktotal'} <= 0 || $regen) {
	&type_xen_compute_disksize($server);
	}

if ($server->{'status'} ne 'nohost') {
	if ($server->{'xen_network_rate'} eq '' || $regen) {
		$server->{'xen_network_rate'} =
			&type_xen_get_network_rate($server);
		}
	if ($server->{'xen_memory'} eq '' || $regen) {
		$server->{'xen_memory'} =
			&type_xen_get_memory($server) / 1024 / 1024;
		}
	if ($server->{'xen_credit_cpu'} eq '' || $regen) {
		($server->{'xen_credit_cpu'},
		 $server->{'xen_credit_weight'}) = &type_xen_get_cpu($server);
		}
	if ($server->{'xen_vcpus'} eq '' || $regen) {
		my $vcpus = &type_xen_get_vcpus($server);
		if (ref($vcpus)) {
			$server->{'xen_vcpus'} = scalar(@$vcpus);
			}
		}
	}

# Cache label->partition map, for use when the system is down
if ($server->{'xen_wholedisk'} &&
    &can_ssh_status($server) && (!defined($server->{'label_map'}) || $regen)) {
	&generate_label_map($server);
	}

if ((!defined($server->{'xen_ip6'}) || $regen) &&
    ($server->{'status'} eq 'down' || &can_ssh_status($server))) {
	# Fetch IPv6 addresses from interfaces
	my $newip6 = join(" ", &list_interface_ip6_addresses($server));
	$server->{'xen_ip6'} = $newip6 if ($newip6);
	}
}

# type_xen_post_host_refresh(&server, re-generate, &info)
# Called after a Xen host system is refreshed, to re-start xenconsoled and
# maybe get LVM info
sub type_xen_post_host_refresh
{
my ($server, $regen, $info) = @_;
return if (!&can_ssh_status($server));

# Work out which Xen API is in use
if (&has_command_on_server($server, 'xm')) {
	my ($out, $ex, $err) = &execute_command_on_server($server, "xm info");
	if ($out =~ /different\s+toolstack\s+\(xl\)/) { 
		$server->{'xen_api_type'} = 'xl';
		}
	else {
		$server->{'xen_api_type'} = 'xm';
		}
	}
elsif (&has_command_on_server($server, 'xl')) {
	$server->{'xen_api_type'} = 'xl';
	}

# Restart xenconsoled if down
my $procs = &list_processes_on_server($server);
return if (!ref($procs));	# Couldn't list
my ($c) = grep { $_->{'args'} =~ /(^|\s|\/)xenconsoled($|\s)/ } @$procs;
if (!$c) {
	# Not running .. need to restart
	my $cmd = $server->{'xen_consoled_cmd'} ||
		     "xenconsoled --log none --timestamp none";
	&execute_command_on_server($server,
		"$cmd >/dev/null 2>&1 </dev/null &");
	}
else {
	# Running .. save the command line, so we can restart it later
	$server->{'xen_consoled_cmd'} = $c->{'args'};
	}

# No longer needed
delete($server->{'xen_vg_total'});
delete($server->{'xen_vg_used'});
delete($server->{'xen_vg_free'});

# Cache Xen version
$server->{'xen_version'} = &get_xen_software_version($server);

# Use xm info to see how much RAM is really used and free for Xen. If the host
# has been configured with a dom0 RAM limit via grub, the Linux "free" command
# will report that total RAM is less than actually exists.
if (&can_ssh_status($server)) {
	my ($out, $ex, $err) = &run_xen_api_command($server, "xm info");
	if (!$ex && !$err) {
		my ($out2) = &run_xen_api_command($server, "xm list");
		$out .= $out2;
		}
	my $grub = &get_file_data_from_server($server, "/boot/grub/menu.lst");
	if (!$ex && !$err) {
		my ($total, $hostfree, $hostmem, $xenmem);
		my $info = &get_server_collected($server);
		foreach my $l (split(/\r?\n/, $out)) {
			if ($l =~ /^total_memory\s+:\s+(\d+)/) {
				$total = $1;
				}
			elsif ($l =~ /^free_memory\s+:\s+(\d+)/) {
				$hostfree = $1;
				}
			elsif ($l =~ /^Domain-0\s+\d+\s+(\d+)/) {
				$hostmem = $1;
				}
			elsif ($l =~ /^\S+\s+\d+\s+(\d+)\s+\d+\s+/) {
				$xenmem += $1;
				}
			}
		if ($total) {
			# Memory used is usage on dom0 (as shown by free),
			# plus usage by all Xen instances.
			$server->{'xen_mem_total'} = $total*1024*1024;
			$server->{'xen_mem_used'} = $xenmem*1024*1024;
			if ($grub =~ /dom0_mem=(\d+)(k|M|G)/i) {
				# A dom0 memory limit is set in the grub config.
				# Consider that memory used, as it isn't
				# available to Xen instances
				my $units = $2 eq "G" ? 1024*1024*1024 :
					    lc($2) eq "k" ? 1024 :
							1024*1024;
				$server->{'xen_mem_used'} += $1*$units;
				}
			else {
				# Dom0 memory total is that of whole system,
				# so only add to used memory that is actually
				# in use by the host
				$server->{'xen_mem_used'} +=
					($info->{'mem'}->[0] -
					 $info->{'mem'}->[1])*1024;
				}
			$server->{'xen_mem_free'} =
				$server->{'xen_mem_total'} -
				$server->{'xen_mem_used'};
			}
		else {
			delete($server->{'xen_mem_total'});
			delete($server->{'xen_mem_used'});
			delete($server->{'xen_mem_free'});
			}
		}
	}
}

# type_xen_edit_form(&server)
# Show information about some Xen system
sub type_xen_edit_form
{
my ($server) = @_;
my $rv;

# Xen instance name
$rv .= &ui_table_row($text{'edit_xeninstance'},
		     $server->{'xen'});

# Actual Xen memory
if ($config{'xen_mem_mode'}) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($v) = grep { $_->{'instance'} eq $server->{'xen'} }
				  @$instances;
		if ($v) {
			$rv .= &ui_table_row($text{'edit_xenmem'},
					     &nice_size($v->{'mem'}));
			}
		}
	}

# Xen config file
$rv .= &ui_table_row($text{'edit_xenconf'},
		     "<tt>$server->{'xen_conf'}</tt>");

# Virtualization type
$rv .= &ui_table_row($text{'edit_xenvtype'},
	$server->{'xen_hvm'} ? $text{'edit_citrix_hvm'}
			     : $text{'edit_citrix_pv'});

return $rv;
}

# type_xen_server_info(&server, [show-in-bytes])
# Returns an array of information for list-systems.pl to display
sub type_xen_server_info
{
my ($server, $bytes) = @_;
my @rv;
my $vhost = &type_xen_parent($server);
push(@rv, [ "Hosted on", $vhost->{'host'} ]);
push(@rv, [ "Xen instance", $server->{'xen'} ]);
if ($config{'xen_mem_mode'}) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($v) = grep { $_->{'instance'} eq $server->{'xen'} }
				  @$instances;
		if ($v) {
			push(@rv, [ "Xen memory now",
			    $bytes ? $v->{'mem'} : &nice_size($v->{'mem'}) ]);
			}
		}
	}
push(@rv, [ "Xen config file", $server->{'xen_conf'} ]);
if ($server->{'xen_disktotal'}) {
	push(@rv, [ "Xen total disk size",
		    $bytes ? $server->{'xen_disktotal'}
			   : &nice_size($server->{'xen_disktotal'}) ]);
	}
push(@rv, [ "Xen virtualization type",
	    $server->{'xen_hvm'} ? "hvm" : "pv" ]);
if (!$server->{'xen_pygrub'}) {
	push(@rv, [ "Xen kernel", "From host system" ]);
	}
else {
	push(@rv, [ "Xen kernel", "From VM using bootloader ".
				  $server->{'xen_bootloader'} ]);
	}
return @rv;
}

# type_xen_server_host_info(&server, [show-in-bytes])
# Returns an array of information for list-systems.pl to display, for a Xen
# host system
sub type_xen_server_host_info
{
my ($server, $bytes) = @_;
my @rv;      
if (&can_ssh_status($server)) {
	my @bridges = &list_host_bridges($server);
	if (@bridges) {
		push(@rv, [ "Xen network bridges", join(" ", @bridges) ]);
		}
	}
push(@rv, [ "Xen directory", $server->{'xen_dir'} ]);
if ($server->{'xen_vg'}) {
	push(@rv, [ "Xen LVM VG", $server->{'xen_vg'} ]);
	}
if ($server->{'xen_iscsi'} ne '') {
	my $iscsiserver = &get_managed_server($server->{'xen_iscsi'});
	push(@rv, [ "Xen iSCSI server", $iscsiserver->{'host'} ]);
	}
if ($server->{'xen_version'}) {
	push(@rv, [ "Xen version", $server->{'xen_version'} ]);
	}
push(@rv, [ "Xen toolstack", $server->{'xen_api_type'} || 'xm' ]);
return @rv;
}

# type_xen_create_form([empty-flag])
# Returns HTML for Xen-specific inputs
sub type_xen_create_form
{
my ($empty) = @_;
my $rv;

# Check if any vhosts have more than one bridge
my @vhosts = grep { $_->{'xen_host'} && &can_host_server($_) }
		     &list_managed_servers();
my @all_ifaces;
foreach my $vhost (@vhosts) {
	my @hostifaces = grep { &can_use_bridge($_) }
			      split(/\s+/, $vhost->{'xen_iface'});
	push(@all_ifaces, @hostifaces);
	}
@all_ifaces = &unique(@all_ifaces);
my @v6vhosts = grep { $_->{'xen_pool6'} } @vhosts;

# Xen host system
my ($hostf, $hostjs);
if (@vhosts == 1 && $module_info{'onehost'} eq 'xen') {
	# Can only be hosted locally
	$rv .= &ui_hidden("xen_on", $vhosts[0]->{'id'});
	}
else {
	($hostf, $hostjs) = &show_host_server_field($text{'add_xenhost'},
						    "xen", "xen");
	$rv .= $hostf;
	}

# IP addresses
if (&can_action(undef, "ips")) {
	my $nm = "255.255.255.0";
	$rv .= &ui_hidden("xen_all_ifaces", join(" ", @all_ifaces));
	if (@all_ifaces <= 1 && 0) {
		# No host system has more than one bridge
		$rv .= &ui_table_row($text{'create_xenip'},
			&ui_radio("xen_ip_def", 1,
			  [ [ 1, $text{'create_xenipauto'} ],
			    [ 0, &text('create_xenipman',
				       &ui_textbox("xen_ip_0", undef, 20),
				       &ui_textbox("xen_netmask_0", $nm, 15))
			    ] ]));

		# Default gateway
		$rv .= &ui_table_row($text{'create_xengw'},
			&ui_opt_textbox("xen_gw_0", undef, 20,
					$text{'create_xengwdef'},
					$text{'create_xengwip'}));
		}
	else {
		# Some hosts have more than one, so need to show an option
		# for all of them
		my $bsel;
		foreach my $b (@all_ifaces) {
			$bsel .= &ui_checkbox("xen_bridge", $b, $b, 1)."&nbsp;";
			}
		$bsel .= "<br>\n";
		$rv .= &ui_table_row($text{'create_xenip'},
			&ui_radio("xen_ip_def", 1,
			  [ [ 1, $text{'create_xenipauto'}."<br>" ],
			    [ 2, $text{'create_xenipbridges'}." ".$bsel ],
			    [ 0, $text{'create_xenipman2'} ] ]));

		my $sfx = $empty ? "empty" : "";
		my $vtable = &ui_columns_start([
			$text{'create_xenifacecol'},
			$text{'create_xenipcol'},
			$text{'create_xennetmaskcol'},
			$text{'create_xengwcol'}]);
		for(my $i=0; $i<@all_ifaces; $i++) {
			$vtable .= &ui_columns_row([
			  $all_ifaces[$i],
			  &ui_textbox("xen_ip_$i", undef, 15),
			  &ui_textbox("xen_netmask_$i", $nm, 15),
			  &ui_opt_textbox("xen_gw_$i", undef, 15,
					  $text{'create_xengwdef'}),
			  ]);
			$vtable .= &ui_hidden("xen_iface_$i", $all_ifaces[$i]);
			}
		$vtable .= &ui_columns_end();
		$rv .= &ui_table_row("", $vtable);
		}
	}

if (&can_action(undef, "ips") && @v6vhosts && !$empty) {
	# IPv6 address
	$rv .= &ui_table_row($text{'create_xenip6'},
		&ui_radio_table("xen_ip6_mode", $config{'alloc_ipv6'} || 0,
				[ [ 0, $text{'create_kvmip6no'} ],
				  [ 1, $text{'create_kvmip6alloc'} ],
				  [ 2, $text{'create_kvmip6manual'},
				       &ui_textbox("xen_ip6", undef, 30)." ".
				       $text{'create_kvmip6netmask'}." ".
				       &ui_textbox("xen_netmask6", 64, 5) ] ],
				1));

        # Default IPv6 gateway
        $rv .= &ui_table_row($text{'create_xengw6'},
                &ui_opt_textbox("xen_gw6", undef, 20,
                                $text{'create_xengwdef'},
                                $text{'create_xengwip6'}));
	}

# Warn the user if we have no DNS zone
if (@vhosts == 1 && $module_info{'onehost'} eq 'xen' &&
    !$vhosts[0]->{'xen_zone'}) {
	$rv .= &ui_table_row(" ",
		"<font color=#ff000>".
		&text('create_xendnswarn', "edit_xen.cgi?id=0", "../bind8/").
		"</font>");
	}

return $rv;
}

# type_xen_limits_create_form([empty-flag])
# Returns HTML for Xen-specific inputs related to CPU, RAM and disk limits
sub type_xen_limits_create_form
{
my ($empty) = @_;
my $rv;

# Physical memory
my $limits = defined(&get_owner_limits) ? &get_owner_limits() : undef;
my $defmem = $config{'xen_mem'}*1024*1024;
my $mem = $defmem;
if ($limits && $limits->{'max_ram'}) {
	$mem = $limits->{'max_ram'} - $limits->{'ram'};
	$mem = $defmem if ($mem > $defmem);
	$mem = 128*1024*1024 if ($ram < 128*1024*1024);
	}
$rv .= &ui_table_row($text{'create_xenmemory'},
	&ui_bytesbox("xen_memory", $mem, 8));

# Filesystem size
$rv .= &ui_table_row($text{'create_xendisksize'},
	$empty ? &ui_bytesbox("xen_disk", 2048*1024*1024, 8) :
		 &vui_opt_bytesbox("xen_disk", undef, 8,
				   $text{'create_xendiskdef'}));

# Swap space
if (!$empty && &can_action(undef, 'swap')) {
	$rv .= &ui_table_row($text{'create_xenswapsize'},
		&vui_opt_bytesbox("xen_swap",
				  $config{'xen_swap'}*1024*1024 || undef, 8,
				  $text{'create_xennoswap'}));
	}

# CPU allocation
my $cpu;
if ($limits && $limits->{'max_cpu'}) {
	$cpu = $limits->{'max_cpu'} - $limits->{'cpu'};
	$cpu = 100 if ($cpu > 100);
	$cpu = 1 if ($cpu < 1);
	}
$rv .= &ui_table_row($text{'create_xencpu'},
	&ui_opt_textbox("xen_cpu", $cpu, 4, $text{'domains_unlimited'})."%");

# Network rate limit
$rv .= &ui_table_row($text{'create_xenrate'},
	&ui_radio("xen_rate_def", 1,
		  [ [ 1, $text{'edit_lunlimit'} ],
                    [ 0, &ui_ratebox("xen_rate", undef) ] ]));

return $rv;
}

# type_xen_advanced_create_form([empty-mode])
# Returns HTML for Xen-specific inputs
sub type_xen_advanced_create_form
{
my ($empty) = @_;
my $rv;

# Network interface MAC address
$rv .= &ui_table_row($text{'create_xenmac'},
	&ui_opt_textbox("xen_mac", undef, 30, $text{'create_xenmacauto'},
			$text{'create_xenmacether'}));

# Interface name
$rv .= &ui_table_row($text{'create_xenvif'},
	&ui_opt_textbox("xen_vif", undef, 10, $text{'create_xenvifauto'},
			$text{'create_xenvifset'}));

# Instance UUID
$rv .= &ui_table_row($text{'create_xenuuid'},
	&ui_opt_textbox("xen_uuid", undef, 50, $text{'create_xenuuidauto'},
			$text{'create_xenuuidset'}));

# Kernel boot type
if (!$empty) {
	$rv .= &ui_table_row($text{'xen_pygrub'},
		&ui_radio("xen_pygrub", $config{'xen_pygrub'},
			  [ [ 0, $text{'xen_pygrub0'} ],
			    [ 1, $text{'xen_pygrub1'} ] ]));
	}

# Storage location
my @vhosts = grep { $_->{'xen_host'} } &list_managed_servers();
my @storages;
my %done;
foreach my $vhost (@vhosts) {
        my $storages = &type_xen_list_storages($vhost, 1);
        push(@storages, grep { !$done{$_->{'id'}}++ } @$storages);
        }
$rv .= &ui_table_row($text{'xen_onlvm'},
        &ui_select("xen_storage", "",
                   [ [ "", "&lt;$text{'xen_onlvmdef'}&gt;" ],
                     map { [ $_->{'id'}, $_->{'desc'} ] } @storages ]));

# On crash/etc settings
my @grid;
foreach my $o (&list_xen_on_actions()) {
	my @opts = &list_xen_on_options($o);
	push(@grid, $text{'create_xenon'.$o});
	push(@grid, &ui_select("xen_on".$o, $config{'xen_on_'.$o},
		   [ [ undef, "&lt;$text{'default'}&gt;" ],
		     map { [ $_, $text{'create_xenon'.$_} ] } @opts ]));
	}
$rv .= &ui_table_row($text{'create_xenonacts'},
	&ui_grid_table(\@grid, 2));

# Network interface type
my $models = &type_xen_list_models();
$rv .= &ui_table_row($text{'create_kvmmodel'},
	&ui_select("xen_model", $config{'xen_model'},
		   [ [ undef, $text{'default'} ],
		     @$models ]));

# Other .cfg file entries
$rv .= &ui_table_row($text{'create_xenmanual'},
	&ui_textarea("xen_manual",
		     join("\n", split(/\t+/, $config{'xen_manual'})), 3, 60));

# Start after creation?
$rv .= &ui_table_row($text{'create_xenstart'},
	&ui_yesno_radio("xen_start", 1));

return $rv;
}

# type_xen_create_parse(&server, &in, [empty], [&limits-plan])
# Parse inputs from type_xen_create_form, and update the server object
sub type_xen_create_parse
{
my ($server, $in, $empty, $plan) = @_;

# Validate memory parameters
if ($plan && $plan->{'max_ram'}) {
	$server->{'xen_memory'} = $plan->{'max_ram'} / (1024*1024);
	}
else {
	$in{'xen_memory'} =~ /^\d+$/ || &error($text{'create_exenmemory'});
	$server->{'xen_memory'} = ($in{'xen_memory'}*$in{'xen_memory_units'}) /
				  (1024*1024);
	}
$server->{'xen_memory'} >= 32 ||
        &error(&text('create_exenmemory2', '32M'));

if ($config{'xen_mem_max'} &&
    $server->{'xen_memory'} > $config{'xen_mem_max'}) {
	&error(&text('create_exenmemory3', $config{'xen_mem_max'}));
	}

# Validate swap file
$server->{'xen_disktotal'} = 0;
if ($plan && $plan->{'max_disk'}) {
	# If disk limit comes from plan, it all goes towards actual disk
	}
elsif (!$in{'xen_swap_def'} && !$empty && &can_action(undef, 'swap')) {
	$in{'xen_swap'} =~ /^\d+$/ || &error($text{'create_exenswapsize'});
	$server->{'xen_swap'} = ($in{'xen_swap'} * $in{'xen_swap_units'}) /
				(1024*1024);
	$server->{'xen_disktotal'} += $server->{'xen_swap'}*1024*1024;
	}

# Validate disk size
my $img = $server->{'image'} ? &get_server_image($server->{'image'})
			     : undef;
if ($plan && $plan->{'max_disk'}) {
	$server->{'xen_disksize'} = $plan->{'max_disk'} / (1024*1024);
	$server->{'xen_disktotal'} += $server->{'xen_disksize'}*1024*1024;
	}
elsif (!$in{'xen_disk_def'}) {
	$in{'xen_disk'} =~ /^\d+$/ || &error($text{'create_exendisksize'});
	$server->{'xen_disksize'} = ($in{'xen_disk'} * $in{'xen_disk_units'}) /
				    (1024*1024);
	$server->{'xen_disktotal'} += $server->{'xen_disksize'}*1024*1024;
	}
else {
	$server->{'xen_disktotal'} += $img->{'rawsize'};
	}
if (!$empty &&
    $server->{'xen_disksize'} &&
    $img->{'rawsize'} &&
    $img->{'rawsize'} > $server->{'xen_disksize'}*1024*1024) {
	&error(&text('create_exendisksize2',
		     &nice_size($img->{'rawsize'})));
	}

# Validate CPU
if ($plan && $plan->{'max_cpu'}) {
	$server->{'xen_credit_cpu'} = $plan->{'max_cpu'};
	}
elsif (!$in{'xen_cpu_def'}) {
	$in{'xen_cpu'} =~ /^[1-9]\d*$/ || &error($text{'create_exencpu'});
	$server->{'xen_credit_cpu'} = $in{'xen_cpu'};
	}

# Validate network rate
if (!$in{'xen_rate_def'}) {
	$in{'xen_rate'} =~ /^\d+$/ || &error($text{'limits_erate'});
	$server->{'xen_network_rate'} = $in{'xen_rate'}*$in{'xen_rate_units'};
	}

# Select host system
my $vhost = &parse_host_server_field($server, 'xen', 'xen', $in,
				     $server->{'xen_memory'}*1024*1024,
				     0,
				     $server->{'xen_disktotal'});
$server->{'xen_on'} = $vhost->{'id'};

# Make sure the hostname doesn't clash
my $instances = &list_xen_instances($vhost);
my ($clash) = grep { $_->{'instance'} eq $in->{'host'} } @$instances;
$clash && &error($text{'create_exenclash'});
$server->{'xen'} = $server->{'host'};

my $bridges;
if ($in->{'xen_ip_def'} == 0 && &can_action(undef, "ips")) {
	# Validate IP addresses
	my @allhostifaces = grep { &can_use_bridge($_) }
				 split(/\s+/, $vhost->{'xen_iface'});
	if (!@allhostifaces) {
		# No host interfaces set, so just assume default
		@allhostifaces = ( '*' );
		}

	my (@ips, @cidrs, @gws, @hostifaces);
	for(my $i=0; defined($in->{"xen_ip_$i"}); $i++) {
		my $iface = $in->{"xen_iface_$i"};
		if ($i == 0 && @allhostifaces == 1 &&
		    $allhostifaces[0] eq "*") {
			# If this host has no interfaces set, assumed the
			# default
			$iface = "*";
			}
		next if (&indexof($iface, @allhostifaces) < 0);
		next if (!$in->{"xen_ip_$i"});
		&check_ipaddress($in->{"xen_ip_$i"}) ||
			&error($text{'create_exenip'});
		push(@ips, $in{"xen_ip_$i"});
		&check_ipaddress($in->{"xen_netmask_$i"}) ||
			&error($text{'iface_enetmask'});
		push(@cidrs, &net::mask_to_prefix(
				$in->{"xen_netmask_$i"}));
		if ($in{"xen_gw_${i}_def"}) {
			push(@gws, "*");
			}
		else {
			&check_ipaddress($in{"xen_gw_$i"}) ||
				 &error($text{'create_exengw'});
			push(@gws, $in{"xen_gw_$i"});
			}
		push(@hostifaces, $iface);
		}
	@ips || &error($text{'create_exenips'});
	scalar(@ips) <= scalar(@allhostifaces) ||
		&error(&text('create_exenifaces2', $vhost->{'host'},
			     scalar(@allhostifaces), scalar(@ips)));
	$server->{'xen_hostifaces'} = join(' ', @hostifaces);
	$server->{'xen_ip'} = join(' ', @ips);
	$server->{'xen_cidr'} = join(' ', @cidrs);
	$server->{'xen_gw'} = join(' ', @gws);
	}
else {
	# Allocate the IP now, by looking for one that isn't pingable or
	# use by any system
	if ($in{'xen_ip_def'} == 2) {
                $bridges = [ grep { &can_use_bridge($_) }
				  split(/\0/, $in->{'xen_bridge'}) ];
                }
	my $err = &allocate_xen_ip_addresses($server, $vhost, $bridges);
	&error($err) if ($err);
	}
$server->{'ip_addresses'} = $server->{'xen_ip'};

if (!$empty) {
	if ($in->{'xen_ip6_mode'} == 2 && &can_action(undef, "ips")) {
		# Validate manually entered IPv6 address
		&check_ip6address($in{'xen_ip6'}) ||
			&error($text{'create_ekvmip6'});
		$in{'xen_netmask6'} =~ /^\d+$/ && $in{'xen_netmask6'} > 0 ||
			&error($text{'create_ekvmnetmask6'});
		$server->{'xen_ip6'} = $in{'xen_ip6'};
		$server->{'xen_netmask6'} = $in{'xen_netmask6'};

		# Validate gateway
		if ($in{'xen_gw6_def'}) {
			$server->{'xen_gw6'} = '*';
			}
		else {
			&check_ip6address($in{'xen_gw6'}) ||
				&error($text{'create_exengw6'});
			$server->{'xen_gw6'} = $in{'xen_gw6'};
			}
		}
	elsif ($in->{'xen_ip6_mode'} == 1 && &can_action(undef, "ips") ||
	       $config{'alloc_ipv6'}) {
		# Allocate automatically
		my $err = &allocate_xen_ip6_addresses($server, $vhost,$bridges);
		&error($err) if ($err);
		}
	else {
		# IPv6 not in use .. clear
		$server->{'xen_ip6'} = undef;
		$server->{'xen_netmask6'} = undef;
		$server->{'xen_gw6'} = undef;
		}
	$server->{'ip6_addresses'} = $server->{'xen_ip6'};
	}

# MAC address
if (!$in{'xen_mac_def'} && &can_action(undef, "advanced")) {
	$in{'xen_mac'} =~ /^([0-9a-f]{1,2}:)*[0-9a-f]{1,2}$/i ||
		&error($text{'create_exenmac'});
	$server->{'xen_mac'} = $in{'xen_mac'};
	}

# Interface name
if (!$in{'xen_vif_def'} && &can_action(undef, "advanced")) {
	$in{'xen_vif'} =~ /^[a-z0-9]+$/ ||
		&error($text{'create_exenvif'});
	$server->{'xen_vif'} = $in{'xen_vif'};
	}

# Instance UUID
if (!$in{'xen_uuid_def'} && &can_action(undef, "advanced")) {
	$in{'xen_uuid'} =~ /^[a-f0-9\-]+$/ ||
		&error($text{'create_exenuuid'});
	$server->{'xen_uuid'} = $in{'xen_uuid'};
	}

# On crash/etc settings
if (&can_action(undef, "advanced")) {
	foreach my $o (&list_xen_on_actions()) {
		if ($in{'xen_on'.$o}) {
			$server->{'xen_on'.$o} = $in{'xen_on'.$o};
			}
		}
	}

# Save manual entries
if (&can_action(undef, "advanced")) {
	$in{'xen_manual'} =~ s/\r//g;
	$in{'xen_manual'} =~ s/^\s+//g;
	$in{'xen_manual'} =~ s/\s+$//g;
	$server->{'xen_manual'} = join("\t", split(/\n/, $in{'xen_manual'}));
	}

# PyGrub / HVM setting
if ($empty || $img && $img->{'xen_hvm'}) {
	$server->{'xen_hvm'} = 1;
	if ($img && !$empty && $img->{'format'} ne 'disk') {
		return "HVM Xen instances must use whole-disk images";
		}
	if ($img && $empty && $img->{'format'} ne 'cd') {
		return "Empty HVM Xen instances must use CD images";
		}
	}
elsif (&can_action(undef, "advanced")) {
	$server->{'xen_pygrub'} = $in{'xen_pygrub'};
	}
else {
	$server->{'xen_pygrub'} = $config{'xen_pygrub'};
	}

# Store disk format flag - use whole disk for HVM virtual machines, or if image
# format is whole-disk
if ($server->{'xen_hvm'}) {
	$server->{'xen_wholedisk'} = 1;
	}
elsif ($img && $img->{'format'} eq 'disk') {
	$server->{'xen_wholedisk'} = 1;
	}
else {
	$server->{'xen_wholedisk'} = 0;
	}

# Storage location setting
my $storages = &type_xen_list_storages($vhost);
my $storage;
if ($in{'xen_storage'}) {
        ref($storages) || &error(&text('create_exenstorages', $storages));
        ($storage) = grep { $_->{'id'} eq $in{'xen_storage'} } @$storages;
        }
else {
        ($storage) = grep { $_->{'default'} } @$storages;
        }
$storage || &error(&text('create_exenstorage',
                         $in{'xen_storage'}, $vhost->{'host'}));
if ($storage->{'type'} eq 'files') {
        $server->{'xen_nolvm'} = 1;
        }
elsif ($storage->{'type'} eq 'lvm') {
        $server->{'xen_force_vg'} = $storage->{'vg'};
        }
elsif ($storage->{'type'} eq 'thin') {
	$server->{'xen_force_vg'} = $storage->{'vg'};
	$server->{'xen_force_thin'} = $storage->{'lv'};
	}
elsif ($storage->{'type'} eq 'iscsi') {
        $server->{'xen_force_iscsi'} = $storage->{'iscsi'};
        }

# Interface model setting
if (!defined($in{'xen_model'})) {
	# Configured default
	if ($server->{'xen_hvm'}) {
		$server->{'xen_model'} = $config{'xen_model'};
		}
	}
elsif ($in{'xen_model'}) {
	# User selection
	$server->{'xen_hvm'} || &error($text{'create_exenmodelhvm'});
	my $models = &type_xen_list_models();
	my ($m) = grep { $_->[0] eq $in{'xen_model'} } @$models;
	$m || &error($text{'create_ekvmmodel'});
	$server->{'xen_model'} = $in{'xen_model'};
	}

# Start at creation
$server->{'xen_start'} = defined($in{'xen_start'}) ? $in{'xen_start'} : 1;

# If we cannot register in DNS, the hostname has to be set as the IP
my @zones = &list_bind_zones();
my $zone;
if ($vhost->{'xen_zone'}) {
	($zone) = grep { $_->{'name'} eq $vhost->{'xen_zone'} } @zones;
	}
if ($zone) {
	$in->{'host'} =~ /\./ && &error($text{'create_exenhost'});
	$server->{'host'} .= ".".$vhost->{'xen_zone'};
	}
}

# type_xen_create_args([empty-flag])
# Returns an array of parameters to type_xen_create_args_parse
sub type_xen_create_args
{
my ($empty) = @_;
my @rv = ( $module_info{'onehost'} eq 'xen' ? ( ) :
		  ( { 'name' => 'host', 'value' => 'host-system' },
		    { 'name' => 'location', 'value' => 'name' } ),
	 { 'name' => 'ip', 'value' => 'address[,address]', 'opt' => 1 },
	 { 'name' => 'cidr', 'value' => 'number[,number]', 'opt' => 1 },
	 { 'name' => 'netmask', 'value' => 'mask,[mask]', 'opt' => 1 },
	 { 'name' => 'gw', 'value' => 'address[,address|*]', 'opt' => 1 },
	 { 'name' => 'bridge', 'value' => 'name[,name]', 'opt' => 1 },
	 { 'name' => 'disksize', 'value' => 'megabytes', 'opt' => 1 },
	 { 'name' => 'memory', 'value' => 'megabytes', 'opt' => 1 },
	 { 'name' => 'cpu', 'value' => 'percent', 'opt' => 1 },
	 { 'name' => 'vcpus', 'value' => 'number', 'opt' => 1 },
	 $empty ? ( ) : ( { 'name' => 'swap',
			    'value' => 'megabytes',
			    'opt' => 1 } ),
	 { 'name' => 'rate', 'value' => 'bits/second', 'opt' => 1 },
	 { 'name' => 'ip6', 'value' => 'ipv6-address,[ipv6-address]', 'opt' => 1 },
	 { 'name' => 'netmask6', 'value' => 'mask,[mask]', 'opt' => 1 },
	 { 'name' => 'gw6', 'value' => 'ipv6-address,[ipv6-address]', 'opt' => 1 },
	 { 'name' => 'allocate-ip6', 'value' => 'true|false', 'opt' => 1 },
	);
if (&can_action(undef, "advanced")) {
	my $models = join('|', map { $_->[0] } @{&type_xen_list_models()});
	push(@rv,
	 { 'name' => 'mac', 'value' => 'ethernet-address', 'opt' => 1 },
	 { 'name' => 'uuid', 'value' => 'instance-uuid', 'opt' => 1 },
	 { 'name' => 'vif', 'value' => 'interface-name', 'opt' => 1 },
	 (map { { 'name' => 'on-'.$_,
		 'value' => join('|', &list_xen_on_options($_)),
		 'opt' => 1 } } &list_xen_on_actions()),
	 { 'name' => 'manual', 'value' => 'manual-config-entries', 'opt' => 1 },
	 { 'name' => 'lvm', 'value' => 'true|false', 'opt' => 1,
	   'deprecated' => 1 },
	 { 'name' => 'storage', 'value' => 'storage-id', 'opt' => 1 },
	 $empty ? ( ) : ( { 'name' => 'pygrub',
			    'value' => 'true|false',
			    'opt' => 1 } ),
	 { 'name' => 'model', 'value' => $models, 'opt' => 1 },
	 { 'name' => 'start', 'value' => 'true|false', 'opt' => 1 },
	 { 'name' => 'iscsi', 'value' => 'hostname', 'opt' => 1,
	   'deprecated' => 1 },
	       );
	}
return @rv;
}

# type_xen_create_args_parse(&args, &server, [empty-flag])
# Update a server object with parsed command-line args
sub type_xen_create_args_parse
{
my ($args, $server, $empty) = @_;

# Validate memory flags
if ($args->{'memory'}) {
	$args->{'memory'} =~ /^\d+$/ || return "Non-numeric memory allocation";
	$server->{'xen_memory'} = $args->{'memory'};
	}
else {
	$server->{'xen_memory'} = $config{'xen_mem'};
	}
if ($config{'xen_mem_max'} &&
    $server->{'xen_memory'} > $config{'xen_mem_max'}) {
	return "Memory limit cannot be larger than the Xen maximum of ".		       $config{'xen_mem_max'};
	}

# Validate swap flag
$server->{'xen_disktotal'} = 0;
if (!$empty && &can_action(undef, 'swap')) {
	if ($args->{'swap'} ne '') {
		$args->{'swap'} =~ /^\d+$/ ||
			return "Non-numeric swap allocation";
		$server->{'xen_swap'} = $args->{'swap'};
		}
	else {
		$server->{'xen_swap'} = $config{'xen_swap'};
		}
	$server->{'xen_disktotal'} += $args->{'swap'}*1024*1024;
	}

# Validate filesystem size flag
my $img = $server->{'image'} ? &get_server_image($server->{'image'})
			     : undef;
if ($args->{'disksize'}) {
	$args->{'disksize'} =~ /^\d+$/ ||
		return "Non-numeric disk size";
	$server->{'xen_disksize'} = $args->{'disksize'};
	if ($img &&
	    $img->{'rawsize'} &&
	    $img->{'rawsize'} > $args->{'disksize'}*1024*1024) {
		return "--xen-disksize must be at least $img->{'rawsize'} ".
		       "bytes, as that is the size of the selected image";
		}
	$server->{'xen_disktotal'} += $args->{'disksize'}*1024*1024;
	}
else {
	$server->{'xen_disktotal'} += $img->{'rawsize'};
	}

# Validate CPU flag
if ($args->{'cpu'}) {
	$args->{'cpu'} =~ /^[1-9]\d*$/ || return "Non-numeric CPU percentage";
	$server->{'xen_credit_cpu'} = $args->{'cpu'};
	}

# Validate VCPUs flag
if ($args->{'vcpus'}) {
	$args->{'vcpus'} =~ /^[1-9]\d*$/ || return "Non-numeric VCPU count";
	$server->{'xen_vcpus'} = $args->{'vcpus'};
	}

# Validate rate flag
if ($args->{'rate'}) {
	$args->{'rate'} =~ /^[1-9]\d*$/ || return "Non-numeric network rate";
	$server->{'xen_network_rate'} = $args->{'rate'};
	}

# Hosting server
my $vhost;
if ($module_info{'onehost'} eq 'xen') {
	# Xen host is always this box
	($vhost) = grep { $_->{'manager'} eq 'this' } &list_managed_servers();
	}
elsif ($args->{'host'}) {
	# Use a specific host
	($vhost) = grep { $_->{'host'} eq $args->{'host'} }
			      &list_managed_servers();
	$vhost || return "Xen host system $args->{'host'} does not exist";
	}
elsif ($args->{'location'}) {
	# Select a host from the location
	my ($loc) = grep { $_->{'id'} eq $args->{'location'} ||
			   $_->{'desc'} eq $args->{'location'} }
			 &list_locations();
	$loc || return "Location $args->{'location'} does not exist";
	$vhost = &find_host_from_location($server, $loc, 'xen',
					  $server->{'xen_memory'}*1024*1024,
					  0,
					  $server->{'xen_disktotal'});
	$vhost || return "Failed to select host system from ".
			 "location $loc->{'desc'}";
	}
else {
	# Use default host
	($vhost) = grep { $_->{'xen_default'} } &list_managed_servers();
	$vhost || return "Missing --xen-host";
	}
$vhost->{'xen_host'} ||
	return "System $args->{'host'} is not registered to host Xen instances";
my ($ok, $err) = &usable_host_system($vhost, "xen",
				     $server->{'xen_memory'}*1024*1024,
				     0,
				     $server->{'xen_disktotal'},
				     $server,
				     1);
$ok || return "Host system $vhost->{'host'} cannot be used : $err";
$server->{'xen_on'} = $vhost->{'id'};

# Xen instance name
my $instances = &list_xen_instances($vhost);
my ($clash) = grep { $_->{'instance'} eq $server->{'host'} } @$instances;
$clash && return "A Xen instance with the same name already exists";
$server->{'xen'} = $server->{'host'};

if ($args->{'ip'}) {
	# Make sure IPs are valid
	&can_action(undef, "ips") ||
		return "You are not allowed to specify IP addresses";
	my @ips = split(/[, ]+/, $args->{'ip'});
	foreach my $ip (@ips) {
		&check_ipaddress($ip) || return "Invalid Xen IP address $ip";
		}
	my @cidrs;
	if ($args->{'cidr'}) {
		@cidrs = split(/[, ]+/, $args->{'cidr'});
		foreach my $cidr (@cidrs) {
			$cidr =~ /^\d+$/ && $cidr > 0 && $cidr <= 32 ||
				return "Invalid Xen CIDR $cidr";
			}
		}
	elsif ($args->{'netmask'}) {
		foreach my $nm (split(/[, ]+/, $args->{'netmask'})) {
			&check_ipaddress($nm) ||
				return "Invalid Xen netmask $nm";
			push(@cidrs, &net::mask_to_prefix($nm));
			}
		}
	else {
		return "Missing --xen-cidr or --xen-netmask flags";
		}
	scalar(@ips) == scalar(@cidrs) ||
		return "The number of IPs must match the number of CIDRs";
	if ($args->{'bridge'}) {
		# Bridges set explicitly
		my @hostifaces = grep { &can_use_bridge($_) }
				      split(/[, ]+/, $args->{'bridge'});
		scalar(@ips) == scalar(@hostifaces) ||
		    return "The number of IPs must match the number of bridges";
		$server->{'xen_hostifaces'} = join(' ', @hostifaces);
		}
	else {
		# Automatically determine from host system
		my @hostifaces = grep { &can_use_bridge($_) }
				      split(/\s+/, $vhost->{'xen_iface'});
		if (!@hostifaces) {
			# No host interfaces set, so just assume default
			$server->{'xen_hostifaces'} = '*';
			@ips == 1 ||
			    return "Host system $vhost->{'host'} does not ".
				   "have any bridges set, so only one IP ".
				   "address can be specified";
			}
		else {
			scalar(@ips) <= scalar(@hostifaces) ||
			    return "Host system $vhost->{'host'} only has ".
				   scalar(@hostifaces)." bridges set, but you ".
				   "specified ".scalar(@ips)." IP addresses";
			$server->{'xen_hostifaces'} =
				join(' ', @hostifaces[0..$#ips]);
			}
		}
	$server->{'xen_ip'} = join(' ', @ips);
	$server->{'xen_cidr'} = join(' ', @cidrs);

	# Gateway IPs
	if ($args->{'gw'}) {
		# Set on command line
		my @gws = split(/[, ]+/, $args->{'gw'});
		foreach my $gw (@gws) {
			$gw eq '*' || &check_ipaddress($gw) ||
				return "Invalid gateway IP address $gw";
			}
		scalar(@ips) == scalar(@gws) ||
		  return "The number of IPs must match the number of gateways";
		$server->{'xen_gw'} = join(' ', @gws);
		}
	else {
		# Assume all are using default
		$server->{'xen_gw'} = join(' ', map { "*" } @ips);
		}
	}
else {
	# Allocate IP automatically
	my $err = &allocate_xen_ip_addresses($server, $vhost,
			$args->{'bridge'} ? [ split(/,/, $args->{'bridge'}) ]
					  : undef);
	return $err if ($err);
	}
$server->{'ip_addresses'} = $server->{'xen_ip'};

# Validate IPv6 flags
if ($args->{'ip6'}) {
	# Use specified addresses
	&can_action(undef, "ips") ||
		return "You are not allowed to specify IPv6 addresses";
	my @ips = split(/[, ]+/, $args->{'ip6'});
	foreach my $ip (@ips) {
		&check_ip6address($ip) || return "Invalid KVM IPv6 address $ip";
		}
	my @netmasks;
	if ($args->{'netmask6'}) {
		@netmasks = split(/[, ]+/, $args->{'netmask6'});
		foreach my $n (@netmasks) {
			$n =~ /^\d+$/ && $n > 0 ||
				return "Invalid Xen IPv6 netmask $n";
			}
		}
	else {
		@netmasks = map { 64 } @ips;
		}
	scalar(@ips) == scalar(@netmasks) ||
		return "The number of IPv6 addresses must match the number ".
		       "of netmasks";
	$server->{'xen_ip6'} = join(' ', @ips);
	$server->{'xen_netmask6'} = join(' ', @netmasks);

        # Gateway IPs
        if ($args->{'gw6'}) {
                # Set on command line
                my @gws6 = split(/[, ]+/, $args->{'gw6'});
                foreach my $gw6 (@gws6) {
                        $gw6 eq '*' || &check_ip6address($gw6) ||
                                return "Invalid gateway IPv6 address $gw6";
                        }
                scalar(@ips) == scalar(@gws6) ||
                        return "The number of IPv6 addresses must match the ".
                               "number of gateways";
                $server->{'xen_gw6'} = join(' ', @gws6);
                }
        else {
                # Assume all are using default
                $server->{'xen_gw6'} = join(' ', map { "*" } @ips6);
                }
	}
elsif ($args->{'allocate-ip6'} =~ /true|yes/ ||
       !$args->{'allocate-ip6'} && $config{'alloc_ipv6'}) {
	# Allocate addresses
	my $err = &allocate_xen_ip6_addresses($server, $vhost,
			$args->{'bridge'} ? [ split(/,/, $args->{'bridge'}) ]
					  : undef);
	return $err if ($err);
	}
else {
	# IPv6 not in use .. clear
	$server->{'xen_ip6'} = undef;
	$server->{'xen_netmask6'} = undef;
	$server->{'xen_gw6'} = undef;
	}
$server->{'ip6_addresses'} = $server->{'xen_ip6'};

my $storages = &type_xen_list_storages($vhost);
ref($storages) || return "Failed to list storages : $storages";
my $storage;

if (&can_action(undef, "advanced")) {
	# MAC address, UUID and vif name
	if ($args->{'mac'}) {
		$args->{'mac'} =~ /^([0-9a-f]{1,2}:)*[0-9a-f]{1,2}$/i ||
			return "Invalid MAC address";
		$server->{'xen_mac'} = $args->{'mac'};
		}
	if ($args->{'uuid'}) {
		$args->{'uuid'} =~ /^[a-f0-9\-]+$/ ||
			return "Invalid UUID";
		$server->{'xen_uuid'} = $args->{'uuid'};
		}
	if ($args->{'vif'}) {
		$args->{'vif'} =~ /^[a-z0-9]+$/ ||
			return "Invalid interface name";
		$server->{'xen_vif'} = $args->{'vif'};
		}

	# On crash/etc options
	foreach my $o (&list_xen_on_actions()) {
		if ($args->{'on-'.$o}) {
			&indexof($args->{'on-'.$o},
				 &list_xen_on_options($o)) >= 0 ||
				return "Invalid option for --xen-on-$o";
			$server->{'xen_on'.$o} = $args->{'on-'.$o};
			}
		elsif ($config{'xen_on_'.$o}) {
			$server->{'xen_on'.$o} = $config{'xen_on_'.$o};
			}
		}

	# Manual config entries
	if ($args->{'manual'}) {
		$server->{'xen_manual'} = $args->{'manual'};
		}

	# Validate storage location flag
	my ($fstorage) = grep { $_->{'id'} eq 'files' } @$storages;
	if ($args->{'storage'}) {
		# Find requested storage
		($storage) = grep { $_->{'id'} eq $args->{'storage'} } 
				  @$storages;
		$storage ||return "No storage with ID $args->{'storage'} found";
		}
	elsif ($args->{'iscsi'}) {
		# Validate forced iSCSI server flag (deprecated)
		my $iscsiserver = &get_managed_server_by(
					"host", $args->{'iscsi'});
		$iscsiserver ||
			return "iSCSI server $args->{'iscsi'} does not exist";
		$iscsiserver->{'iscsi_module'} ||
			return "System $args->{'iscsi'} is not an iSCSI server";
		($storage) = grep { $_->{'iscsi'} eq $iscsiserver->{'id'} }
				  @$storages;
		}
	elsif ($args->{'lvm'}) {
		# Validate LVM flag (deprecated)
		if ($args->{'lvm'} =~ /true|yes/i) {
			($storage) = grep { $_->{'id'} =~ /^lvm_/ } @$storages;
			$storage || return "Host system $vhost->{'host'} ".
					   "does not use LVM";
			$server->{'directory'} &&
				return "LVM cannot be enabled when a disk ".
				       "directory is specified";
			}
		elsif ($args->{'lvm'} =~ /false|no/i) {
			$storage = $fstorage;
			}
		}
	else {
		# Default storage
		if ($server->{'directory'}) {
			$storage = $fstorage;
			}
		else {
			($storage) = grep { $_->{'default'} } @$storages;
			}
		}
	}
else {
	# Cannot use advanced options, so use default storage
	($storage) = grep { $_->{'default'} } @$storages;
	}

# Use the storage chosen
$storage || return "No storage location found!";
if ($storage->{'type'} eq 'files') {
	$server->{'xen_nolvm'} = 1;
	}
elsif ($storage->{'type'} eq 'lvm') {
	$server->{'xen_force_vg'} = $storage->{'vg'};
	}
elsif ($storage->{'type'} eq 'thin') {
	$server->{'xen_force_vg'} = $storage->{'vg'};
	$server->{'xen_force_thin'} = $storage->{'lv'};
	}
elsif ($storage->{'type'} eq 'iscsi') {
	$server->{'xen_force_iscsi'} = $storage->{'iscsi'};
	}

# Validate directory flag
if ($args->{'image-dir'}) {
	return "The --xen-image-dir flag has been replaced by ".
	       "--disk-directory";
	}

# Validate PyGrub flag
if ($empty || $img->{'xen_hvm'}) {
	$server->{'xen_hvm'} = 1;
	if ($img && !$empty && $img->{'format'} ne 'disk') {
		return "HVM Xen instances must use whole-disk images";
		}
	if ($img && $empty && $img->{'format'} ne 'cd') {
                return "Empty HVM Xen instances must use CD images";
                }
	}
elsif (!&can_action(undef, "advanced") || !$args->{'pygrub'}) {
	# Use default
	$server->{'xen_pygrub'} = $config{'xen_pygrub'};
	}
elsif ($args->{'pygrub'} =~ /true|yes/i) {
	$server->{'xen_pygrub'} = 1;
	}
elsif ($args->{'pygrub'} =~ /false|no/i) {
	$server->{'xen_pygrub'} = 0;
	}
else {
	return "The pygrub flag must be set to true or false";
	}

# Store disk format flag - use whole disk for HVM virtual machines, or if image
# format is whole-disk
if ($server->{'xen_hvm'}) {
	$server->{'xen_wholedisk'} = 1;
	}
elsif ($img && $img->{'format'} eq 'disk') {
	$server->{'xen_wholedisk'} = 1;
	}
else {
	$server->{'xen_wholedisk'} = 0;
	}

# Validate interface model flag
if (&can_action(undef, "advanced")) {
	# Validate model flag
	if ($args->{'model'}) {
		# Selection from arg
		$server->{'xen_hvm'} ||
			return "A network interface device model can only be ".
			       "set for HVM Xen instances";
		my $models = &type_xen_list_models();
		my ($m) = grep { $_->[0] eq $args->{'model'} } @$models;
		$m || return "Invalid network interface model";
		$server->{'xen_model'} = $args->{'model'};
		}
	elsif ($server->{'xen_hvm'}) {
		# Use configured default
		$server->{'xen_model'} = $config{'xen_model'};
		}
	}
else {
	# Use default model
	if ($server->{'xen_hvm'}) {
		$server->{'xen_model'} = $config{'xen_model'};
		}
	}

if (&can_action(undef, "advanced")) {
	# Validate start at creation flag
	if ($args->{'start'} =~ /false|no/i) {
                $server->{'xen_start'} = 0;
                }
        else {
                $server->{'xen_start'} = 1;
                }
        }
else {
	$server->{'xen_start'} = 1;
	}

# Adjust hostname for DNS
my @zones = &list_bind_zones();
my $zone;
if ($vhost->{'xen_zone'}) {
	($zone) = grep { $_->{'name'} eq $vhost->{'xen_zone'} } @zones;
	}
if ($zone) {
	$server->{'host'} =~ /\./ && return "Xen hostnames cannot contain .";
	$server->{'host'} .= ".".$vhost->{'xen_zone'};
	}

return undef;
}

# type_xen_create(&server, [serial, key], [reset], [empty-flag], [nostart])
# Create a new Xen instance, by unzipping the filesystem image in the background
sub type_xen_create
{
my ($server, $serial, $key, $reset, $empty, $nostart) = @_;
my $vhost = &type_xen_parent($server);
$vhost && $vhost->{'xen_host'} || return "Host system does not exist or is ".
                                         "not a Xen host";
my $img = $server->{'image'} ? &get_server_image($server->{'image'})
			     : undef;
my $xen_dir = $server->{'directory'} || $vhost->{'xen_dir'};
delete($server->{'xen_filesystem_offset'});

# Create base dir if needed
my @st = &stat_file_on_server($vhost, $xen_dir);
if (!@st) {
	&make_dir_on_server($vhost, $xen_dir, 0755, 1);
	}

# Validate CD image on host
if ($empty && !$server->{'image'} && $server->{'imagefile'}) {
	&stat_file_on_server($vhost, $server->{'imagefile'}) ||
		return "CD image $server->{'imagefile'} on host system ".
		       "$vhost->{'host'} does not exist";
	}

# Work out the config and image file locations
my ($fsfile, $swfile, $lvmsuffix);
if ($reset) {
	# Filesystem remains the same
	$fsfile = $server->{'xen_filesystem'};
	$swfile = $server->{'xen_swapfile'};
	if (!$fsfile) {
		return "A system with no primary disk cannot be reset";
		}
	}
elsif ($server->{'xen_force_iscsi'} ne '' ||
       $vhost->{'xen_iscsi'} ne '' && !$server->{'xen_nolvm'}) {
	# On iSCSI server
	$server->{'xen_filesystem_iscsi'} = $server->{'xen_force_iscsi'} ne '' ?
                $server->{'xen_force_iscsi'} : $vhost->{'xen_iscsi'};
	delete($server->{'xen_filesystem_vg'});
	delete($server->{'xen_filesystem_lv'});
	delete($server->{'xen_swapfile_vg'});
	delete($server->{'xen_swapfile_lv'});
        if ($server->{'xen_swap'}) {
                $server->{'xen_swapfile_iscsi'} =
			$server->{'xen_filesystem_iscsi'};
                }
	else {
		delete($server->{'xen_swapfile_iscsi'});
		}
	$lvmsuffix = "phy";
        }
elsif ($server->{'xen_force_vg'} ||
       $vhost->{'xen_vg'} && !$server->{'xen_nolvm'}) {
	# On LVM logical volumes
	$server->{'xen_filesystem_vg'} = $server->{'xen_force_vg'} ||
                                         $vhost->{'xen_vg'};
	$server->{'xen_filesystem_lv'} = $server->{'xen'}.'_img';
	$server->{'xen_filesystem_thin'} = $server->{'xen_force_thin'};
	$fsfile = "/dev/".$server->{'xen_filesystem_vg'}."/".
		  $server->{'xen_filesystem_lv'};
	delete($server->{'xen_filesystem_iscsi'});
	delete($server->{'xen_swapfile_iscsi'});
	if ($server->{'xen_swap'}) {
		$server->{'xen_swapfile_vg'} = $vhost->{'xen_vg'};
		$server->{'xen_swapfile_lv'} = $server->{'xen'}.'_swap';
		$server->{'xen_swapfile_thin'} = $server->{'xen_force_thin'};
		$swfile = "/dev/".$server->{'xen_swapfile_vg'}."/".
			  $server->{'xen_swapfile_lv'};
		}
	else {
		delete($server->{'xen_swapfile_vg'});
		delete($server->{'xen_swapfile_lv'});
		}
	$lvmsuffix = "phy";
	}
else {
	# On regular files
	$fsfile = "$xen_dir/$server->{'xen'}.img";
	my @st = &stat_file_on_server($vhost, $fsfile);
	return "Disk image file $fsfile already exists" if (@st);
	delete($server->{'xen_filesystem_vg'});
	delete($server->{'xen_filesystem_lv'});
	delete($server->{'xen_filesystem_thin'});
	delete($server->{'xen_filesystem_iscsi'});
	delete($server->{'xen_swapfile_vg'});
	delete($server->{'xen_swapfile_lv'});
	delete($server->{'xen_swapfile_thin'});
	delete($server->{'xen_swapfile_iscsi'});
	if ($server->{'xen_swap'}) {
		$swfile = "$xen_dir/$server->{'xen'}.swap";
		my @st = &stat_file_on_server($vhost, $swfile);
		return "Swap image file $fsfile already exists" if (@st);
		}
	}

# Check again if host supports iSCSI
if ($server->{'xen_filesystem_iscsi'} ne '') {
        my $err = &check_iscsi_client($vhost);
        $err && return "Host system $vhost->{'host'} does not support iSCSI : $err";
        }

# Work out config file location. Keep existing one if resetting
my $cfgfile;
if ($reset) {
	$cfgfile = $server->{'xen_conf'};
	}
else {
	$cfgfile = "$xen_dir/$server->{'xen'}.cfg";
	$server->{'xen_conf'} = $cfgfile;
	}

# Check for Pv-Grub or PyGrub support if requested
my $bootloader;
if ($server->{'xen_pygrub'}) {
	$bootloader = &find_xen_grub_bootloader($vhost, $img);
	$bootloader ||
		return &text('create_exenpygrub', $vhost->{'host'});
	$server->{'xen_bootloader'} = $bootloader;
	}

# Make sure we have a kernel
if (!$bootloader && !$server->{'xen_hvm'}) {
	my ($kok, $kfile, $ifile) = &setup_xen_host_kernel($vhost);
	$kok || return $kfile;
	}

# Make sure we have the hvm loader
if ($server->{'xen_hvm'}) {
	&find_xen_hvmloader($vhost) ||
		return &text('create_exenhvmloader', $vhost->{'host'});
	&find_xen_qemu_dm($vhost) ||
		return &text('create_exendm', $vhost->{'host'});
	}

# Make sure HVM is supported by the CPU
if ($server->{'xen_hvm'} && &check_xen_hvm_support($vhost) == 0) {
	return &text('create_exenhvm', $vhost->{'host'});
	}

# Find existing LVs if resetting
&remote_finished();
my (@lvs, $fslv, $swaplv);
if ($reset && $server->{'xen_filesystem_vg'}) {
	&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
	@lvs = &remote_foreign_call($vhost, "lvm",
		"list_logical_volumes", $server->{'xen_filesystem_vg'});
	($fslv) = grep { $_->{'vg'} eq $server->{'xen_filesystem_vg'} &&
			 $_->{'name'} eq $server->{'xen_filesystem_lv'} } @lvs;
	($swaplv) = grep { $_->{'vg'} eq $server->{'xen_swapfile_vg'} &&
			   $_->{'name'} eq $server->{'xen_swapfile_lv'} } @lvs;
	}

# Create swap LV or iSCSI target
$last_remote_error = undef;
if ($server->{'xen_swapfile_vg'}) {
	&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
	my $lv = { 'name' => $server->{'xen_swapfile_lv'},
		   'size' => int($server->{'xen_swap'}*1024),
		   'perm' => 'rw',
		   'alloc' => 'n',
		   'vg' => $server->{'xen_swapfile_vg'},
		   'thin_in' => $server->{'xen_swapfile_thin'}, };
	if (!($swaplv && $swaplv->{'size'} > $lv->{'size'} &&
		         $swaplv->{'size'}-$lv->{'size'} < 100*1024)) {
		# Only delete and re-create if an old LV with close enough to
		# the same size doesn't exist
		if ($reset && $swfile) {
			# Delete the old LV
			&delete_all_disk_image_partitions($vhost, $swfile);
			my $err = &remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume",
					     { 'device' => $swfile });
			$err ||= $last_remote_error;
			return "LVM logical volume deletion for swap failed : ".
			       $err if ($err);
			}
		my $err = &remote_foreign_call($vhost, "lvm",
					 	"create_logical_volume", $lv);
		$err ||= $last_remote_error;
		return "LVM logical volume creation for swap failed : $err"
			if ($err);
		}
	}
elsif ($server->{'xen_swapfile_iscsi'} ne '') {
	# Create iSCSI disk for swap
	my $disk = { 'size' => $server->{'xen_swap'}*1024*1024,
		     'iscsi_server' => $server->{'xen_swapfile_iscsi'},
		   };
	my $keep_disk = 0;
	if ($reset) {
		# If resettting, disk already exists .. fill in object
		# with info from VM
		$disk->{'iscsi_name'} = $server->{'xen_swapfile_iscsi_name'};
		$disk->{'iscsi_target'} = $server->{'xen_swapfile_iscsi_target'};
		$disk->{'iscsi_ip'} = $server->{'xen_swapfile_iscsi_ip'};
		$disk->{'iscsi_port'} = $server->{'xen_swapfile_iscsi_port'};
		$disk->{'iscsi_connect'} = -1;	# May or may not be connected

		# Get info on existing disk on iSCSI server
		my $iscsiserver = &get_managed_server(
					$server->{'xen_swapfile_iscsi'});
		my $idisk = &get_iscsi_disk_info($iscsiserver, $disk);
		if (ref($idisk) &&
		    &size_close_enough($idisk->{'size'}, $disk->{'size'})) {
			$keep_disk = 1;
			}
		}
	if (!$keep_disk) {
		# Disk needs to be deleted (if re-setting) and then created
		# on iSCSI server
		if ($reset) {
			my $err = &delete_iscsi_disk($server, $disk);
			}
		my $err = &create_iscsi_disk($server, $disk);
		$err && return "iSCSI swap disk creation failed : $err";
		$swfile = $disk->{'real'};
		$server->{'xen_swapfile_iscsi_name'} = $disk->{'iscsi_name'};
		$server->{'xen_swapfile_iscsi_target'} = $disk->{'iscsi_target'};
		$server->{'xen_swapfile_iscsi_ip'} = $disk->{'iscsi_ip'};
		$server->{'xen_swapfile_iscsi_port'} = $disk->{'iscsi_port'};
		$server->{'xen_swapfile_iscsi_real'} = $disk->{'iscsi_real'};
		}
	}

# Create filesystem LV or iSCSI target
$last_remote_error = undef;
if ($server->{'xen_filesystem_vg'}) {
	&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
	my $lv = { 'name' => $server->{'xen_filesystem_lv'},
		   'size' =>
			($img && !$empty ? int($img->{'rawsize'}/1024) : 0) ||
			int($server->{'xen_disksize'}*1024) ||
			1024*1024,
		   'perm' => 'rw',
		   'alloc' => 'n',
		   'vg' => $server->{'xen_filesystem_vg'},
		   'thin_in' => $server->{'xen_filesystem_thin'}, };
	if (!$fslv ||
	    !&size_close_enough($fslv->{'size'}*1024, $lv->{'size'}*1024)) {
		# Only delete and re-create if an old LV with close enough to
		# the same size doesn't exist
		if ($reset) {
			# Delete the old LV
			&delete_all_disk_image_partitions($vhost, $fsfile);
			my $err = &remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume",
					     { 'device' => $fsfile });
			$err ||= $last_remote_error;
			return "LVM logical volume deletion failed : $err"
				if ($err);
			}
		my $err = &remote_foreign_call($vhost, "lvm",
					       "create_logical_volume", $lv);
		$err ||= $last_remote_error;
		return "LVM logical volume creation failed : $err" if ($err);
		}
	}
elsif ($server->{'xen_filesystem_iscsi'} ne '') {
	# On iSCSI, so create iSCSI disk and attach to host
	my $disk = { 'size' => ($img && !$empty ? $img->{'rawsize'} : 0) ||
			       $server->{'xen_disksize'}*1024*1024 ||
			       1024*1024,
		     'iscsi_server' => $server->{'xen_filesystem_iscsi'},
		   };
	my $keep_disk = 0;
	if ($reset) {
		# If resettting, disk already exists .. fill in object
		# with info from VM
		$disk->{'iscsi_name'} = $server->{'xen_filesystem_iscsi_name'};
		$disk->{'iscsi_target'} = $server->{'xen_filesystem_iscsi_target'};
		$disk->{'iscsi_ip'} = $server->{'xen_filesystem_iscsi_ip'};
		$disk->{'iscsi_port'} = $server->{'xen_filesystem_iscsi_port'};
		$disk->{'iscsi_connect'} = -1;	# May or may not be connected

		# Get info on existing disk on iSCSI server
		my $iscsiserver = &get_managed_server(
					$server->{'xen_filesystem_iscsi'});
		my $idisk = &get_iscsi_disk_info($iscsiserver, $disk);
		if (ref($idisk) &&
		    &size_close_enough($idisk->{'size'}, $disk->{'size'})) {
			$keep_disk = 1;
			}
		}
	if (!$keep_disk) {
		# Disk needs to be deleted (if re-setting) and then created
		# on iSCSI server
		if ($reset) {
			my $err = &delete_iscsi_disk($server, $disk);
			}
		my $err = &create_iscsi_disk($server, $disk);
		$err && return "iSCSI disk creation failed : $err";
		$fsfile = $disk->{'real'};
		$server->{'xen_filesystem_iscsi_name'} = $disk->{'iscsi_name'};
		$server->{'xen_filesystem_iscsi_target'} = $disk->{'iscsi_target'};
		$server->{'xen_filesystem_iscsi_ip'} = $disk->{'iscsi_ip'};
		$server->{'xen_filesystem_iscsi_port'} = $disk->{'iscsi_port'};
		$server->{'xen_filesystem_iscsi_real'} = $disk->{'iscsi_real'};
		}
	}

# Set drive in config, unless resetting in which case they stay the same
$server->{'xen_filesystem'} = $fsfile;
$server->{'xen_swapfile'} = $swfile;

# On some systems (like Debian 6), the bridge created by xend by default
# is eth0, but if none is set in the .cfg file then xenbr0 will be used ...
# which doesn't exist! However, Xen 4.x appears to have fixed this.
my @ifaces = split(/\s+/, $server->{'xen_hostifaces'});
my @newifaces;
my @host_bridges = &list_host_bridges($vhost);
my $xenver = &get_xen_software_version($vhost);
if ($xenver < 4.0) {
	foreach my $i (@ifaces) {
		if (($i eq "" || $i eq "*") &&
		    &indexof("xenbr0", @host_bridges) < 0 &&
		    @host_bridges > 0) {
			$i = $host_bridges[0];
			}
		push(@newifaces, $i);
		}
	@ifaces = @newifaces;
	$server->{'xen_hostifaces'} = join(" ", @ifaces);
	}

# Work out interface parameters
my @ips = split(/\s+/, $server->{'xen_ip'});
my @gws = split(/\s+/, $server->{'xen_gw'});
my @vifs;
for(my $i=0; $i<@ips; $i++) {
	my @vifparams;
	push(@vifparams, "ip=".$ips[$i]);
	my $mac;
	if ($server->{'xen_mac'} && $i == 0) {
		my $macclash = &get_managed_server_by(
			"xen_mac", $server->{'xen_mac'});
		if (!$macclash || $macclash->{'id'} eq $server->{'id'}) {
			$mac = $server->{'xen_mac'};
			}
		}
	$mac ||= &generate_mac_address($server, $i);
	push(@vifparams, "mac=".$mac);
	if ($server->{'xen_vif'}) {
		push(@vifparams, "vifname=".$server->{'xen_vif'});
		}
	if ($ifaces[$i] && $ifaces[$i] ne '*') {
		push(@vifparams, "bridge=".$ifaces[$i]);
		}
	if ($server->{'xen_network_rate'}) {
		push(@vifparams,
		     "rate=".int($server->{'xen_network_rate'}/1024)."K");
		}
	if ($server->{'xen_model'} && $server->{'xen_hvm'}) {
		push(@vifparams, "model=".$server->{'xen_model'});
		}
	my $vifparams = join(",", @vifparams);
	push(@vifs, "'$vifparams'");
	}
my $vifs = join(",", @vifs);

# Create the config file, if missing
if (!$reset) {
	my $temp = &transname();
	my $netmask = &net::prefix_to_mask($server->{'xen_cidr'});
	&open_tempfile(CFG, ">$temp");
	&print_tempfile(CFG, "memory = $server->{'xen_memory'}\n");
	if ($config{'xen_mem_max'}) {
		&print_tempfile(CFG, "maxmem = $config{'xen_mem_max'}\n");
		}
	&print_tempfile(CFG, "maxvcpus = ",($config{'xen_maxvcpus'} || 64),"\n");
	&print_tempfile(CFG, "name = '$server->{'xen'}'\n");
	&print_tempfile(CFG, "vif = [ $vifs ]\n");
	&print_tempfile(CFG, "address = '$ips[0]'\n");
	&print_tempfile(CFG, "netmask = '$netmask'\n");
	my $suffix = $lvmsuffix || $vhost->{'xen_filetype'} || "file";

	# Work out device for disks in .cfg file. Note that this may be changed
	# from sda to xvda later, if the /etc/fstab in the image uses xvda paths
	my ($fsdev, $swdev) = $server->{'xen_hvm'} ? ("hda", "hdb") :
			      $server->{'xen_wholedisk'} ? ("sda", "sdb") :
							   ("sda1", "sda2");
	if ($swfile) {
		# Swap and real disks
		&print_tempfile(CFG, "disk = ['$suffix:$fsfile,$fsdev,w',".
				     "'$suffix:$swfile,$swdev,w']\n");
		}
	else {
		# Real disk only
		&print_tempfile(CFG, "disk = ['$suffix:$fsfile,$fsdev,w']\n");
		}
	if ($empty) {
		# Will boot from CD
		&print_tempfile(CFG, "boot = 'dc'\n");
		}
	if ($server->{'xen_uuid'}) {
		&print_tempfile(CFG, "uuid = '$server->{'xen_uuid'}'\n");
		}
	foreach my $o (&list_xen_on_actions()) {
		if ($server->{'xen_on'.$o}) {
			&print_tempfile(CFG, "on_".$o." = '".
					$server->{'xen_on'.$o}."'\n");
			}
		}
	&close_tempfile(CFG);
	my $err = &put_file_to_server($vhost, $temp, $cfgfile);
	delete($get_xen_config_cache{$server->{'id'}});
	return "Failed to create config file : $err" if ($err);
	$server->{'ip_addresses'} = join(" ", @ips);
	}

# Create empty whole disk file and partition
if ($empty) {
	my $dksize = $server->{'xen_disksize'}*1024*1024;
	$dksize ||= $server->{'xen_disktotal'};
	if (!$server->{'xen_filesystem_vg'} &&
	    $server->{'xen_filesystem_iscsi'} eq '') {
		my $cmd = &dd_command("/dev/zero", $fsfile, $dksize);
		my ($out, $ex, $err) = &execute_command_on_server(
						$vhost, $cmd);
		$err = $out if ($ex);
		$err && return "Failed to create file $disk->{'real'} : $err";
		}
	my $err = &create_disk_image_partition($vhost, $fsfile,
					       undef, $dksize, 1);
	return $err if ($err);
	}

# Add CD to list of disks
my $conf = &get_xen_config($server, 0);
my $disks = &xen_find_value("disk", $conf);
my $imgdestfile;
my $copyserver = $vhost;  # Where the image is at
if ($empty && $server->{'image'}) {
	# CD will be copied to a new file
	$imgdestfile = "$vhost->{'xen_dir'}/$server->{'host'}.cd";
	$server->{'xen_cdimage'} = $imgdestfile;
	push(@$disks, "file:$imgdestfile,hdc:cdrom,r");
	}
elsif ($empty && !$server->{'image'}) {
	# CD is in an existing file, or no image needed
	if ($server->{'imagefile'}) {
		push(@$disks, "file:$server->{'imagefile'},hdc:cdrom,r");
		}
	}
else {
	# Copying image to first drive on the system
	$copyserver = &get_transfer_image_dest($server);
	if ($copyserver->{'id'} eq $vhost->{'id'}) {
		# Copy to disk image file on the host
		$imgdestfile = $fsfile;
		}
	else {
		# Copy to the underlying iSCSI disk file
		$imgdestfile = $server->{'xen_filesystem_iscsi_real'};
		}
	}
&xen_save_directive($conf, "disk", $disks);
&put_xen_config($server, $conf);

if ($imgdestfile) {
	# Put the image into place, in the background
	my $err = &deploy_disk_image($server, $img, $imgdestfile, $copyserver);
	return $err if ($err);
	}

return undef;
}

# allocate_xen_ip_addresses(&server, &vhost, [&bridges])
# Allocate one or more IP addresses for a Xen instance from the host's ranges.
# On success, returns undef and sets xen_ip, xen_cidr and xen_iface to space-
# separated lists. On failure returns an error message.
sub allocate_xen_ip_addresses
{
my ($server, $vhost, $bridges) = @_;
$vhost->{'xen_start'} || return &text('create_exenalloc', $vhost->{'host'});

# Work out interfaces on host system
my @hostifaces;
if ($bridges) {
	@hostifaces = @$bridges;
	}
elsif ($vhost->{'xen_iface'}) {
	@hostifaces = grep { &can_use_bridge($_) }
			   split(/\s+/, $vhost->{'xen_iface'});
	}
else {
	@hostifaces = ( "*" );
	}
@hostifaces || return $text{'create_exennoifaces'};

# For each, allocate an IP from a range
foreach my $iface (@hostifaces) {
	my ($ip, $cidr, $gw) = &allocate_ip_address(
		$vhost->{'xen_start'}, $vhost->{'xen_end'},
	        $vhost->{'xen_cidr'}, $vhost->{'xen_gws'},
		$vhost->{'xen_ifaces'}, $iface);
	$ip || return &text('create_exenalloc', $vhost->{'host'},
			    &describe_ip_range($vhost->{'xen_start'},
					       $vhost->{'xen_end'},
					       $vhost->{'xen_cidr'},
                                               $vhost->{'xen_ifaces'},
                                               $iface));
	push(@ips, $ip);
	push(@cidrs, $cidr);
	push(@gws, $gw || "*");
	}
$server->{'xen_ip'} = join(" ", @ips);
$server->{'xen_cidr'} = join(" ", @cidrs);
$server->{'xen_hostifaces'} = join(" ", @hostifaces);
$server->{'xen_gw'} = join(" ", @gws);
return undef;
}

# allocate_xen_ip6_addresses(&server, &vhost, [&bridges])
# Allocate one or more IPv6 addresses for a KVM instance from the host's ranges.
# On success, returns undef and sets xen_ip6, xen_netmask6 and xen_iface6 to
# space-separated lists. On failure returns an error message.
sub allocate_xen_ip6_addresses
{
my ($server, $vhost, $bridges) = @_;
$vhost->{'xen_pool6'} || return &text('create_exenallocnone6',$vhost->{'host'});

# Work out interfaces on host system
my @hostifaces;
if ($bridges) {
	@hostifaces = @$bridges;
	}
elsif ($vhost->{'xen_iface'}) {
	@hostifaces = split(/\s+/, $vhost->{'xen_iface'});
	}
else {
	@hostifaces = ( "*" );
	}

# For each, allocate an IP from a range
my (@ips, @netmasks, @gws);
foreach my $iface (@hostifaces) {
	my ($ip, $netmask, $gw) = &allocate_ip6_address(
		$vhost->{'xen_pool6'}, $vhost->{'xen_ifaces'}, $iface);
	$ip || return &text('create_exenalloc6', $vhost->{'host'},
			    &describe_ip6_pool($vhost->{'xen_pool6'},
                                               $vhost->{'xen_ifaces'},
                                               $iface));
	push(@ips, $ip);
	push(@netmasks, $netmask);
	push(@gws, $gw || "*");
	}
$server->{'xen_ip6'} = join(" ", @ips);
$server->{'xen_netmask6'} = join(" ", @netmasks);
$server->{'xen_hostifaces6'} = join(" ", @hostifaces);
$server->{'xen_gw6'} = join(" ", @gws);
return undef;
}

# type_xen_available_ip_addresses(&server)
# Returns a list of hash refs for IPs available on some system
sub type_xen_available_ip_addresses
{
my ($server) = @_;
my $vhost = &get_parent_server($server);
return &available_ip_addresses($vhost->{'xen_start'},
			       $vhost->{'xen_end'},
			       $vhost->{'xen_cidr'},
			       $vhost->{'xen_gws'});
}

# setup_xen_host_kernel(&vhost)
# Copies the kernel and initrd file (if needed) into place on a Xen host.
# Returns 1, the kernel file and initrd file if successful, or 0 and an
# error message on failure.
sub setup_xen_host_kernel
{
my ($vhost) = @_;

# Get kernel
my ($uout, $ex, $err) = &execute_command_on_server($vhost, "uname -r");
$uout =~ s/\r|\n//g;
$uout || return (0, "Could not work out host kernel version : ".
		    "uname reported $uout");

# Work out kernel to use, and copy to Xen dir if needed
my $kfile = "$vhost->{'xen_dir'}/vmlinuz-vm2-xenU";
my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"test -r ".quotemeta($kfile));
if ($ex) {
	# First check for Debian-style Xen kernel
	my $srckfile = "/boot/xen0-linux-".$uout;
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"test -r ".quotemeta($srckfile));
	if ($ex) {
		# Try same kernel we are currently running, as seen on CentOS
		$srckfile = "/boot/vmlinuz-".$uout;
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
				"test -r ".quotemeta($srckfile));
		if ($ex) {
			return (0, "Could not find kernel image at $srckfile");
			}
		}
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"cp ".quotemeta($srckfile)." ".quotemeta($kfile));
	if ($ex) {
		return (0, "Failed to copy kernel image : $out");
		}
	}

# Create initrd from kernel if needed
my $ifile = "$vhost->{'xen_dir'}/initrd.vm2.xenU.img";
($out, $ex, $err) = &execute_command_on_server($vhost,
		"test -r ".quotemeta($ifile));
if ($ex) {
	# First check for Debian-style existing initrd file
	my $debifile = "/boot/xen0-linux-".$uout.".initrd.img";
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"test -r ".quotemeta($debifile));
	if ($ex) {
		# Newer Debian-style initrd file
		$debifile = "/boot/initrd.img-".$uout;
		($out, $ex, $err) = &execute_command_on_server($vhost,
				"test -r ".quotemeta($debifile));
		}
	if (!$ex) {
		# Found it .. just copy into place
		($out, $ex, $err) = &execute_command_on_server($vhost,
			"cp ".quotemeta($debifile)." ".quotemeta($ifile));
		if ($ex) {
			return (0, "Failed to copy initial ramdisk : $out");
			}
		}
	else {
		($out, $ex, $err) = &execute_command_on_server($vhost,
			"mkinitrd ".quotemeta($ifile)." $uout --with xenblk ".
			"--with xennet --preload xenblk --preload xennet");
		if ($ex) {
			return (0, "Failed to create initial ramdisk : $out");
			}
		}
	}

return (1, $kfile, $ifile);
}

# type_xen_create_done(&server, [empty])
# Checks if the filesystem gunzip is complete, and returns OK if so.
# Returns undef if still running, or an error message on failure.
sub type_xen_create_done
{
my ($server, $empty) = @_;
my $vhost = &type_xen_parent($server);
my $copyserver = $empty ? $vhost : &get_transfer_image_dest($server);
return &done_deploy_disk_image($server, $copyserver);
}

# type_xen_create_post(&server, [serial, key], [reset], [empty], [no-start])
# Called after creation is complete. Here we can mount and fix up config
# files in the filesytem, then start the instance.
sub type_xen_create_post
{
my ($server, $serial, $key, $reset, $empty, $nostart) = @_;
my $vhost = &type_xen_parent($server);
my $img = $server->{'image'} ? &get_server_image($server->{'image'}) : undef;
my @ips = split(/\s+/, $server->{'xen_ip'});
my @cidrs = split(/\s+/, $server->{'xen_cidr'});
my @netmasks = map { &net::prefix_to_mask($_) } @cidrs;
my @ips6 = split(/\s+/, $server->{'xen_ip6'});
my @netmasks6 = split(/\s+/, $server->{'xen_netmask6'});
$server->{'status'} = 'down';
$nostart = 1 if (!$server->{'xen_start'});
$nostart = 0 if ($reset);

# Gateways can either be set per IP address, or copy from host system.
# If none are set, assume the first IP uses the host system gateway.
my $defgw = &get_server_default_gateway($vhost);
my @gws = split(/\s+/, $server->{'xen_gw'});
my @setgws = grep { $_ ne "*" } @gws;
if (!@setgws && $defgw) {
	@gws = ( $defgw );
	}
my $defgw6 = &get_server_default_ipv6_gateway($vhost);
my @gws6 = split(/\s+/, $server->{'xen_gw6'});
my @setgws6 = grep { $_ ne "*" } @gws6;
if (!@setgws6 && $defgw6) {
        @gws6 = ( $defgw6 );
        }

my ($fsfile, $swsize, $swfile, $mp);
if (!$empty) {
	# Create the swap file, if needed
	if ($server->{'xen_swapfile'}) {
		$swsize = $server->{'xen_swap'}*1024*1024;
		$swfile = $server->{'xen_swapfile'};
		my @st = &recursive_stat_file_on_server($vhost, $swfile);
		if (!@st || $server->{'xen_swapfile_vg'} ||
		    $server->{'xen_swapfile_iscsi'} ne '') {
			&$first_print(&text('create_xenswap',
					    &nice_size($swsize)));
			my ($out, $ex, $err) = &execute_command_on_server(
			  $vhost,
			  &dd_command("/dev/zero", $swfile, $swsize)." && ".
			  "mkswap ".quotemeta($swfile));
			if ($err || $ex) {
				&$second_print(&text('create_exenswap',
						     $err || $out));
				}       
			else {
				&$second_print($text{'create_done'});
				}
			}
		}

	if (&mountable_filesystem($server)) {
		# Clean up /etc/fstab to remove non-existant devices
		&$first_print($text{'create_xencleanfstab'});
		my $err = &cleanup_server_fstab($server);
		if ($err) {
			&$second_print(&text('create_exencleanfstab', $err));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}

	# Expand the filesystem image, if needed
	$fsfile = $server->{'xen_filesystem'};
	if ($server->{'xen_disksize'}) {
		my $dksize = $server->{'xen_disksize'}*1024*1024;
		&start_background_dot_printer(
			&text('create_xenresize', &nice_size($dksize)));
		my $disks = &type_xen_list_disks($server, 1);
		my $disk;
		if (ref($disks)) {
			($disk) = grep { $_->{'real'} eq $fsfile } @$disks;
			}
		if (!$disk || !$disk->{'fs'}) {
			&stop_background_dot_printer();
			&$second_print(&text('create_exenresize2'));
			}
		else {
			my $olddisk = { %$disk };
			$disk->{'size'} = $dksize;
			my $err = &type_xen_modify_disk($server, $disk,
							$olddisk);
			&stop_background_dot_printer();
			if ($err) {
				&$second_print(&text('create_exenresize',$err));
				}       
			else {
				&$second_print($text{'create_done'});
				}
			}
		}

	# Mount the created filesystem
	&$first_print($text{'create_xenmount'});
	my $err = &mount_filesystem_if_needed($server);
	$mp = $server->{'xen_mount'};
	if ($err || !$mp) {
		&$second_print(&text('create_exenmount', $err));
		$mp = undef;
		}
	else {
		&$second_print(&text('create_xenmounted', "<tt>$mp</tt>"));
		}
	}

# Make sure swap and root filesystem files are writable
foreach my $f ($fsfile, $swfile ? ( $swfile ) : ()) {
	if ($f !~ /^\/dev\//) {
		&execute_command_on_server($vhost,
			"chmod 700 ".quotemeta($f));
		}
	}

my $kerr;
my $grubfile;
my $cfg = &get_xen_config($server);
if ($server->{'xen_hvm'}) {
	# Setup HVM
	&xen_save_directive($cfg, "kernel", &find_xen_hvmloader($vhost) ||
					    "/usr/lib/xen/boot/hvmloader");
	&xen_save_directive($cfg, "builder", "hvm");
	&xen_save_directive($cfg, "device_model", &find_xen_qemu_dm($vhost) ||
						  "/usr/lib/xen/bin/qemu-dm");
	&xen_save_directive($cfg, "acpi", 1);
	&xen_save_directive($cfg, "pae", 1);
	}
else {
	# Setup kernel in config file
	&$first_print($text{'create_xenkernel'});
	if ($server->{'xen_pygrub'}) {
		# Make sure filesystem has menu.lst
		foreach my $f ("/boot/grub/grub.conf", "/boot/grub/menu.lst") {
			my @st = &stat_file_on_server($server, $f);
			if (@st > 1) {
				$grubfile = $f;
				}
			}
		if (!$grubfile) {
			delete($server->{'xen_pygrub'});
			delete($server->{'xen_bootloader'});
			$kerr = $text{'create_xenkernelhost2'};
			}
		}
	$server->{'xen_grubfile'} = $grubfile;
	my $bootloader = $server->{'xen_bootloader'};
	if ($server->{'xen_pygrub'} && $bootloader =~ /pygrub/) {
		# Configure to use pygrub
		$kerr = $text{'create_xenkernelpy'};
		&xen_save_directive($cfg, "bootloader", $bootloader);
		&xen_save_directive($cfg, "root");
		&xen_save_directive($cfg, "ramdisk");
		&xen_save_directive($cfg, "kernel");
		&xen_save_directive($cfg, "extra");
		}
	elsif ($server->{'xen_pygrub'} && $bootloader =~ /pv-grub/) {
		# Configure to use pv-grub
		$kerr = $text{'create_xenkernelpv'};
		&xen_save_directive($cfg, "kernel", $bootloader);
		&xen_save_directive($cfg, "extra", "(hd0)".$grubfile);
		&xen_save_directive($cfg, "root");
		&xen_save_directive($cfg, "ramdisk");
		&xen_save_directive($cfg, "bootloader");
		}
	else {
		# Configure to use kernel from host. Root gets adjusted again
		# below to match /etc/fstab if possible.
		($kok, $kfile, $ifile) = &setup_xen_host_kernel($vhost);
		$kerr ||= $text{'create_xenkernelhost'};
		&xen_save_directive($cfg, "kernel", $kfile);
		&xen_save_directive($cfg, "ramdisk", $ifile);
		&xen_save_directive($cfg, "root", "/dev/sda1 ro");
		&xen_save_directive($cfg, "extra");
		&xen_save_directive($cfg, "bootloader");
		}
	}

# Add manual entries to Xen config file
if ($server->{'xen_manual'} && !$reset) {
	&open_tempfile(TEMP, ">>$cfg->[0]->{'file'}");
	foreach my $m (split(/\t/, $server->{'xen_manual'})) {
		&print_tempfile(TEMP, "$m\n");
		}
	&close_tempfile(TEMP);
	}
if ($img && $img->{'xen_cfg'} && !$reset) {
	&open_tempfile(TEMP, ">>$cfg->[0]->{'file'}");
	foreach my $m (split(/\t/, $img->{'xen_cfg'})) {
		&print_tempfile(TEMP, "$m\n");
		}
	&close_tempfile(TEMP);
	}

# Write out the Xen config
my $err = &put_xen_config($server, $cfg);
$kerr = $err if ($err);
&$second_print($kerr);

if (!$empty && $mp) {
	# Fix root in grub config
	if ($server->{'xen_pygrub'}) {
		&$first_print($text{'create_xengrub'});
		my ($grubconf, $err) =
			&get_file_data_from_server($server, $grubfile);
		if ($grubconf) {
			$grubconf =~ s/root\s+\(hd\d+(,\d+)?\)/root \(hd0\)/;
			if ($grubconf =~ /root=\/dev\/(s|h|xv)d[a-z]\s/ &&
			    !$server->{'xen_wholedisk'}) {
				# Root device in grub config is like /dev/sda,
				# but it should be /dev/sda1
				$grubconf =~ s/(root=\/dev\/(s|h|xv)d[a-z])/${1}1/g;
				}
			&put_file_data_to_server($server, $grubconf, $grubfile);
			&$second_print($text{'create_done'});
			}
		else {
			&$second_print(&text('create_exengrub', $err));
			}
		}

	# Copy kernel modules to the filesystem, if missing and if not
	# booting from pygrub / pv-grub
	if (!$server->{'xen_pygrub'} && !$server->{'xen_hvm'}) {
		my ($uout, $ex, $err) = &execute_command_on_server($vhost,
								   "uname -r");
		$uout =~ s/\r|\n//g;
		my $moddir = "$mp/lib/modules/$uout";
		($out, $ex, $err) = &execute_command_on_server($vhost,
			"test -d ".quotemeta($moddir));
		if ($ex) {
			&$first_print(&text('create_xenmods', $uout));
			($out, $ex, $err) = &execute_command_on_server($vhost,
			    "cp -r ".quotemeta("/lib/modules/$uout")." ".
				     quotemeta($moddir));
			if ($ex) {
				&$second_print(&text('create_exenmods',
						     &html_escape($out)));
				}
			else {
				&$second_print($text{'create_done'});
				}
			}
		}

	# Set the root password, by directly editing /etc/shadow
	if ($server->{'ssh_pass'}) {
		&$first_print($text{'create_xensetpass'});
		my $err = &set_shadow_root_password($server,
						    $server->{'ssh_pass'});
		if ($err) {
			&$second_print(&text('create_exensetpass', $err));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}
	elsif ($server->{'ssh_key'}) {
		# Using key, so clear password
		&set_shadow_root_password($server, undef);
		}

	# Create authorized keys file
	if ($server->{'key'}) {
		&$first_print($text{'create_xensetkey'});
		my $err = &create_authorized_key($server);
		&$second_print($err ? &text('create_exensetkey', $err)
				    : $text{'create_done'});
		}
	elsif (!$img->{'keepkeys'}) {
		&clear_authorized_key($server);
		}

	# Add extra SSH keys
	if (&has_addnew_keys($server)) {
		&$first_print($text{'create_xenaddnewkeys'});
		my $err = &addnew_authorized_keys($server);
		&$second_print($err ? &text('create_exenaddnewkeys', $err)
				    : $text{'create_done'});
		}

	# Fix other config files that refer to the hostname or IP
	&$first_print($text{'create_xenfix'});
	my $fullhost = $server->{'unix_hostname'} ?
				$server->{'unix_hostname'} :
		       $vhost->{'xen_zone'} ? 
				$server->{'xen'}.".".$vhost->{'xen_zone'} :
				$server->{'xen'};
	my $err = &fix_hostname_config_files($server, $ips[0],
					$server->{'xen'}, $fullhost, $vhost,
					$mp, $vhost->{'xen_resolv'});
	if ($err) {
		&$second_print(&text('create_exenfix', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}

	# Setup networking config files to use the IP.
	&$first_print($text{'create_xennet'});
	my $err = &fix_network_config_files($server, $ips[0], $netmasks[0],
					    $gws[0], $vhost, $mp);
	if ($err) {
		&$second_print(&text('create_exennet', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}

	# Add network configs for extra IPs
	if (@ips > 1) {
		&$first_print($text{'create_xennet2'});
		my $err = &fix_additional_network_interface_files($server,
			[ @ips[1..$#ips] ], [ @netmasks[1..$#netmasks] ],
			[ @gws[1..$#gws] ], $vhost, $mp);
		if ($err) {
			&$second_print(&text('create_exennet2', $err));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}

	# Add the swap file to /etc/fstab, if any
	&$first_print($text{'create_xenfstab'});
	my $fstab = &get_system_fstab($server);
	my $root_device_name;
	my $rootdev;
	if (!ref($fstab)) {
		&$second_print(&text('create_exenswaptab', $fstab));
		}
	else {
		my $changed_fstab;
		my ($root) = grep { $_->[1] eq '/' } @$fstab;
		if ($root->[0] =~ /^\/dev\/(s|h|xv)d[a-z]$/ &&
		    !$server->{'xen_wholedisk'}) {
			# Root device is like /dev/sda, but it should be
			# /dev/sda1
			$root->[0] .= "1";
			$changed_fstab++;
			}
		$root_device_name = $root->[0];
		$rootdev = $root ? $root->[0] :
			   $server->{'xen_hvm'} ? "/dev/hda1" : "/dev/sda1";
		if ($server->{'xen_swapfile'}) {
			# Add swap device
			my $swapdev;
			if ($server->{'xen_wholedisk'}) {
				# Change /dev/sda1 to /dev/sdb1
				if ($rootdev =~ /^(.*)([a-z])(\d+)$/) {
					$swapdev = $1.chr(ord($2)+1).$3;
					}
				}
			else {
				# Change /dev/sda1 to /dev/sda2
				if ($rootdev =~ /^(.*)([a-z])(\d+)$/) {
					$swapdev = $1.$2.($3+1);
					}
				}
			my ($sw) = grep { $_->[0] eq $swapdev } @$fstab;
			if ($swapdev && !$sw) {
				push(@$fstab, [ $swapdev, 'swap', 'swap',
						'defaults', 0, 0, ]);
				$changed_fstab++;
				}
			}
		if ($changed_fstab) {
			# Write out again
			my $err = &put_system_fstab($server, $fstab);
			if ($err) {
				&$second_print(
					&text('create_exenswaptab', $err));
				}
			else {
				&$second_print($text{'create_done'});
				}
			}
		else {
			&$second_print(&text('create_xenswaptabok'));
			}
		}

	# Fix up root again in Xen config file
	if (!$server->{'xen_hvm'} && !$server->{'xen_pygrub'} && $rootdev) {
		my $cfg = &get_xen_config($server);
		&xen_save_directive($cfg, "root", "$rootdev ro");
		&put_xen_config($server, $cfg);
		}

	# If root is like /dev/xvdaN in /etc/fstab, it should be the same
	# in the .cfg file, instead of sda
	if ($root_device_name =~ /^\/dev\/(xvd[a-z]+([0-9]*))$/) {
		my $cfg = &get_xen_config($server);
		my $disks = &xen_find_value("disk", $cfg);
		foreach my $d (@$disks) {
			$d =~ s/,sd([a-z]+)/,xvd$1/;
			}
		&xen_save_directive($cfg, "disk", $disks);
		&put_xen_config($server, $cfg);
		}

	# Apply IPv6 addresses to interfaces
	if (@ips6) {
		&$first_print($text{'create_xenipv6'});
		my $err = &apply_ipv6_addresses($server, \@ips6, \@netmasks6,
						\@gws6);
		if ($err) {
			&$second_print(&text('create_exenipv6', $err));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}

	# Fix Virtualmin licence files
	if ($serial) {
		&$first_print($text{'create_xenlicence'});
		my $err = &fix_virtualmin_licence_files($server, $serial, $key);
		if ($err) {
			&$second_print(&text('create_exenlicence', $err));
			}       
		else {
			&$second_print($text{'create_done'});
			}
		}

	# Configure SSH to allow logins by root
	&$first_print($text{'create_zonesrootssh'});
	my $err = &fix_ssh_root_login($server);
	if ($err eq "OK") {
		&$second_print($text{'create_zonesrootsshok'});
		}
	elsif ($err eq "NONE") {
		&$second_print($text{'create_zonesrootsshnone'});
		}
	elsif ($err) {
		&$second_print($err);
		}
	else {
		&$second_print($text{'create_done'});
		}

	# Set SSH port
	&$first_print($text{'create_sshport'});
	my $err = &set_ssh_port($server);
	if ($err) {
		&$second_print(&text('create_esshport', $err));
		}
	else {
		&$second_print($text{'create_done'});
		}

	# Make sure /dev/null is writable, and nash is killed
	my $rcl = &transname();
	my $err = &get_file_from_server($server, "/etc/rc.local", $rcl);
	my $lref = &read_file_lines($rcl);
	my ($foundnull, $foundnash, $lnum, $exitline);
	foreach my $l (@$lref) {
		if ($l =~ /^chmod.*\/dev\/null/) {
			$foundnull++;
			}
		elsif ($l =~ /^killall(\s+\-\d+)nash/) {
			$foundnash++;
			}
		elsif ($l =~ /^exit/) {
			$exitline = $lnum;
			}
		$lnum++;
		}
	$exitline ||= scalar(@$lref);
	if (!$foundnull || !$foundnash) {
		splice(@$lref, $exitline, 0, "chmod 777 /dev/null")
			if (!$foundnull);
		splice(@$lref, $exitline, 0, "killall -9 nash nash-hotplug")
			if (!$foundnash);
		&flush_file_lines($rcl);
		&put_file_to_server($server, $rcl, "/etc/rc.local");
		}
	}

# Add DHCP entries
if ($config{'dhcp_add'} && !$reset) {
	my $fakeifaces = &type_xen_list_interfaces($server, 1);
	if (ref($fakeifaces)) {
		&add_all_iface_dhcpd($server, $fakeifaces);
		}
	}

if ($mp) {
	# Un-mount
	&$first_print($text{'create_xenumount'});
	&umount_filesystem_if_needed($server);
	&$second_print($text{'create_done'});
	}

# Compute total disk size
&type_xen_compute_disksize($server);

# Register in DNS, if we can
if ($vhost->{'xen_zone'} && !$reset) {
	my $dnsname = $server->{'xen'}.".".$vhost->{'xen_zone'}.".";
	if (&add_bind8_record($dnsname, $ips[0],
			      $vhost->{'xen_zone'})) {
		$server->{'xen_dns'} = $dnsname;
		}
	if ($ips6[0]) {
		&add_bind8_record($dnsname, $ips6[0], $vhost->{'xen_zone'});
		}
	}

# Setup VNC console in config file
&$first_print($text{'create_xenvnc'});
my $err = &setup_xen_vnc_console($server);
&$second_print($err ? &text('create_xenvncfailed', $err)
		    : &text('create_xenvncdone',
			    $server->{'xen_vnc_display'}));

# For Xen 4.2, need to (re-)register the config file
my $xenver = &get_xen_software_version($vhost);
if ($xenver >= 4.2 && $vhost->{'xen_api_type'} ne 'xl') {
	&$first_print($text{'create_xenregister'});
	if ($reset) {
		&run_xen_api_command($vhost,
			"xm delete $server->{'xen'}");
		}
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm new $server->{'xen_conf'}");
	$err = $out if ($ex && !$err);
	if ($err) {
		&$second_print(&text('create_xenvncfailed', $err));
		}
	else {
		&$second_print($text{'create_done'});
		}
	}

# Start up the instance
my $started = 0;
if (!$nostart) {
	&$first_print($text{'create_xenstartup'});
	my ($ok, $err) = &type_xen_startup($server,
				$empty || $server->{'ssh_nologin'});
	if ($ok != 1) {
		&$second_print(&text('create_exenstartup', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		$started = 1;
		}
	}

# Import Virtualmin package keys
if ($serial && $started && !$empty && !$server->{'ssh_nologin'}) {
	&$first_print($text{'create_xenvkeys'});
	my $err = &fix_virtualmin_after_boot($server);
	if ($err) {
		&$second_print(&text('create_exenvkeys', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Regenerate SSH host key
if ($started && !$empty && !$server->{'ssh_nologin'}) {
	&$first_print($text{'create_xenhostkey'});
	my $err = &regenerate_ssh_host_keys($server);
	&$second_print($err ? &text('create_exensetkey', $err)
			    : $text{'create_done'});
	}

return undef;
}

# type_xen_delete(&server, [keep-disks])
# Shut down the server, then delete the filesystem file, config file and
# swap files.
sub type_xen_delete
{
my ($server, $keep) = @_;

# Force shutdown
my $vhost = &type_xen_parent($server);
delete($list_xen_instances_cache{$vhost->{'id'}});
&run_xen_api_command($vhost, "xm destroy $server->{'xen'}");
&run_xen_api_command($vhost, "xm destroy Zombie-$server->{'xen'}");
&type_xen_set_atboot($server, 0);
$server->{'status'} = 'down';

# Delete virtual disks
if (!$keep) {
	my $disks = &type_xen_list_disks($server, 0);
	if (ref($disks)) {
		foreach my $d (reverse(@$disks)) {
			my @st = &recursive_stat_file_on_server(
					$vhost, $d->{'real'});
			next if (!@st);			# Already gone
			next if ($d->{'media'} eq 'cdrom' &&
				 $d->{'real'} ne $server->{'xen_cdimage'} &&
				 !$d->{'device'});	# CD image in file
			next if ($d->{'device'} &&      # Attached real device
				 $d->{'iscsi_server'} eq '' &&
                                 !$d->{'lv'});
			next if (!$d->{'device'} &&	# File that is a device
				 !$d->{'lv'} &&
				 $d->{'real'} =~ /^\/dev\//);
			my $err = &type_xen_delete_disk($server, $d, 0, 1);
                        return "Failed to delete $d->{'real'} : $err" if ($err);
			}
		}
	else {	
		my @st = &stat_file_on_server($vhost, $server->{'xen_conf'});
		if (@st > 1) {
			return "Failed to get list of disks : $disks";
			}
		}
	}

# Delete mount dir
if (!$keep) {
	&delete_file_on_server($vhost, "/mnt/xen-".$server->{'host'});
	}

# For Xen 4.2, delete from Xen management
my $xenver = &get_xen_software_version($vhost);
if ($xenver >= 4.2) {
	&run_xen_api_command($vhost, "xm delete $server->{'xen'}");
	}

# Delete config files
foreach my $f ($server->{'xen_conf'}, $server->{'xen_console'}) {
	next if (!$f);
	my $err = &delete_file_on_server($vhost, $f);
	return $err if ($err);
	}
delete($get_xen_config_cache{$server->{'id'}});

# Remove the DNS entry, if we created one
&delete_bind8_record($server->{'xen_dns'}, $vhost->{'xen_zone'},
		     &to_ipaddress($server->{'host'}));
if ($server->{'xen_ip6'}) {
	&delete_bind8_record($server->{'xen_dns'}, $vhost->{'xen_zone'},
			     &to_ip6address($server->{'host'}));
	}

# Remove DHCP entries
&delete_all_iface_dhcpd($server);

return undef;
}

# type_xen_get_file(&server, remotefile, localfile)
# If the filesystem for this Xen instance is locally mounted, get the file
# directly from there
sub type_xen_get_file
{
my ($server, $remotefile, $localfile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &get_file_from_server($vhost, $server->{'xen_mount'}.$remotefile,
					     $localfile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_put_file(&server, localfile, remotefile)
# If the filesystem for this Xen instance is locally mounted, write the file
# directly to there
sub type_xen_put_file
{
my ($server, $localfile, $remotefile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &put_file_to_server($vhost, $localfile,
				   $server->{'xen_mount'}.$remotefile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_put_multiple_files(&server, &localfiles, remotedir)
# If the filesystem for this Xen instance is locally mounted, write the file
# directly to there
sub type_xen_put_multiple_files
{
my ($server, $localfiles, $remotedir) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &put_multiple_files_to_server($vhost, $localfiles,
				   $server->{'xen_mount'}.$remotedir);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_delete_file(&server, remotefile, noquote)
# If the filesystem for this Xen instance is locally mounted, delete the file
# directly from there
sub type_xen_delete_file
{
my ($server, $remotefile, $noquote) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &delete_file_on_server($vhost,
			      $server->{'xen_mount'}.$remotefile, $noquote);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_checksum_file(&server, remotefile, [data-size])
# If the filesystem for this Xen instance is locally mounted, checksum the file
# directly from there
sub type_xen_checksum_file
{
my ($server, $file, $size) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &checksum_file_on_server($vhost,
			      $server->{'xen_mount'}.$file, $size);
	}
else {
	return ( 'FALLBACK' );
	}
}

# type_xen_rename_file(&server, oldfile, newfile)
# If the filesystem for this Xen instance is locally mounted, rename the
# file directly from there
sub type_xen_rename_file
{
my ($server, $oldfile, $newfile) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &rename_file_on_server($vhost, $server->{'xen_mount'}.$oldfile, $server->{'xen_mount'}.$newfile);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_stat_file(&server, remotefile, [lookup-usernames])
# If the filesystem for this Xen instance is locally mounted, stat the file
# directly from there
sub type_xen_stat_file
{
my ($server, $remotefile, $usernames) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &stat_file_on_server($vhost, $server->{'xen_mount'}.$remotefile, $usernames, $server->{'xen_mount'});
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_make_dir(&server, dir, perms, recur)
# If the filesystem for this Xen instance is locally mounted, make the dir
# directly from there
sub type_xen_make_dir
{
my ($server, $remotedir, $perms, $recur) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &make_dir_on_server($vhost, $server->{'xen_mount'}.$remotedir,
				   $perms, $recur);
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_list_dir(&server, dir, [full-stat], [lookup-usernames])
# If the filesystem for this Xen instance is locally mounted, list the dir
# directly from there
sub type_xen_list_dir
{
my ($server, $remotedir, $fullstat, $usernames) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &list_dir_on_server($vhost, $server->{'xen_mount'}.$remotedir,
				   $fullstat, $usernames,
				   $server->{'xen_mount'});
	}
else {
	return 'FALLBACK';
	}
}

# type_xen_readlink(&server, file)
# If the filesystem for this KVM instance is locally mounted, check the link
# from there
sub type_xen_readlink
{
my ($server, $file) = @_;
if ($server->{'xen_mount'}) {
        my $vhost = &type_xen_parent($server);
        return &readlink_on_server($vhost, $server->{'xen_mount'}.$file);
        }
else {
        return 'FALLBACK';
        }
}

# type_xen_symlink_file(&server, source, dest)
# If the filesystem for this Xen instance is locally mounted, create the
# link from there
sub type_xen_symlink_file
{
my ($server, $src, $dest) = @_;
if ($server->{'xen_mount'}) {
        my $vhost = &type_xen_parent($server);
        return &symlink_file_on_server($vhost, $src,
				       $server->{'xen_mount'}.$dest);
        }
else {
        return 'FALLBACK';
        }
}

# type_xen_set_ownership_permissions(&server, user, group, perms, file+)
# If the filesystem for this xen instance is locally mounted, change the
# permissions there
sub type_xen_set_ownership_permissions
{
my ($server, $user, $group, $perms, @files) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	my $dir = $server->{'xen_mount'};
	return &set_ownership_permissions_on_server($server, $user, $group,
			$perms, map { $dir.$_ } @files);
	}
else {
	return 'FALLBACK';
        }
}
# type_xen_disk_usage(&server, dir, [include-counts])
# If the filesystem for this xen instance is locally mounted, run the du
# command there.
sub type_xen_disk_usage
{
my ($server, $dir, $counts) = @_;
if ($server->{'xen_mount'}) {
	my $vhost = &type_xen_parent($server);
	return &disk_usage_on_server($vhost, $server->{'xen_mount'}.$dir,
				     $counts);
	}
else {
	return 'FALLBACK';
        }
}

# type_xen_get_atboot(&server)
# Returns 1 if some some Xen instance is started at boot time, by looking
# for a link to its config from /etc/xen/auto
sub type_xen_get_atboot
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $cfile = $server->{'xen_conf'};
$cfile =~ s/^(.*)\///;
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"readlink /etc/xen/auto/$cfile");
$out =~ s/\r|\n//g;
return $out eq $server->{'xen_conf'};
}

# type_xen_set_atboot(&server, yesno)
# Enable some Xen system at boot time, by linking it's config file to
# /etc/xen/auto
sub type_xen_set_atboot
{
my ($server, $boot) = @_;
my $vhost = &type_xen_parent($server);
my $cfile = $server->{'xen_conf'};
$cfile =~ s/^(.*)\///;
my ($out, $ex, $err);
if ($boot) {
	&execute_command_on_server($vhost,	# For Redhat
		"chkconfig xendomains on");
	&make_dir_on_server($vhost, "/etc/xen/auto", 0755, 1);
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"rm -f ".quotemeta("/etc/xen/auto/$cfile")." ; ".
		"ln -s ".quotemeta($server->{'xen_conf'})." ".
			 quotemeta("/etc/xen/auto/$cfile"));
	}
else {
	($out, $ex, $err) = &execute_command_on_server($vhost,
		"rm -f ".quotemeta("/etc/xen/auto/$cfile"));
	}
return $err ? $err : $ex ? $out : undef;
}

# type_xen_check_image(&server, &image)
# Returns an error message if a system cannot be imaged for some reason
sub type_xen_check_image
{
my ($server, $image) = @_;
my $fstab = &get_system_fstab($server);
return undef if (!ref($fstab));
my ($root) = grep { $_->[1] eq '/' } @$fstab;
return "No root filesystem found in /etc/fstab" if (!$root);
$root->[0] =~ /^\/dev\/(h|s|xv)d[a-z]/ ||
	return "Root filesystem in /etc/fstab is mounted from $root->[0], ".
	       "not a real device path. This will cause systems created from ".
	       "the image to use incorrect device paths";
return undef;
}

# type_xen_can_direct_image(&server)
# Returns 1 since imaging direct to the master is supported
sub type_xen_can_direct_image
{
return 1;
}

# type_xen_can_stream_image(&server, &image)
# Returns 1 because piping the image directly in from the master is supported
sub type_xen_can_stream_image
{
return 1;
}

# type_xen_create_image(&server, &image, tempdir, [nomunge],
# 			[&image-from-server], [&image-to-server])
# Convert a Xen system into an image file, which is really just the Xen disk.
# Outputs text, and returns 0 on failure.
# No-munge flag 0 = munge all, 1 = munge none, 2 = mungle all but serials
sub type_xen_create_image
{
my ($server, $image, $image_to_dir, $nomunge, $image_from_server,
    $image_to_server) = @_;
my $vhost = &type_xen_parent($server);
$image_from_server ||= $vhost;

# Mount the disk image
&$first_print($text{'mkimage_xenmount'});
my $fsfile = $server->{'xen_filesystem'};
if (!$fsfile) {
	&$second_print($text{'mkimage_nofilesystem'});
	return 0;
	}
my $err = &mount_filesystem_if_needed($server);
if ($err) {
	&$second_print(&text('create_exenmount', $err));
	}
else {
	&$second_print(&text('create_xenmounted',
			     "<tt>$server->{'xen_mount'}</tt>"));

	if ($nomunge != 1) {
		# Munge some or all files
		my ($mok, $munged) = &munge_all_files($server, 0,
						      $nomunge == 2);
		return 0 if (!$mok);
		$server->{'munged'} = $munged;
		}

	# Check if the system has grub
	if (&stat_file_on_server($server, "/boot/grub/grub.conf") ||
            &stat_file_on_server($server, "/boot/grub/menu.lst")) {
		$img->{'xen_pygrub'} = 1;
		}

	# Un-mount
	&$first_print($text{'mkimage_xenumount'});
	&umount_filesystem_if_needed($server);
	&$second_print($text{'create_done'});
	}

# Get the size of the raw disk
my $disks = &type_xen_list_disks($server, 0);
my $rawsize;
my $d;
if (ref($disks)) {
	($d) = grep { $_->{'real'} eq $fsfile } @$disks;
	if ($d) {
		$rawsize = $d->{'size'};
		$image->{'format'} = $d->{'format'};
		}
	}
if (!$rawsize) {
	my @st = &stat_file_on_server($vhost, $fsfile);
	$rawsize = $st[7];
	}
$image->{'rawsize'} = $rawsize;
$image->{'xen_hvm'} = $server->{'xen_hvm'};
my $conf = &get_xen_config($server, 1);
if (&xen_find_value("pae", $conf)) {
	$image->{'xen_pae'} = 1;
	}

# If the disk is on iSCSI, take the image from the iSCSI server
my $isdevice = $server->{'xen_filesystem_lv'} ||
               $server->{'xen_filesystem_iscsi'} ne '';
if ($server->{'xen_filesystem_iscsi'} ne '') {
        my $iscsiserver = &get_managed_server(
                                $server->{'xen_filesystem_iscsi'});
        if (!$iscsiserver) {
                &$second_print($text{'mkimage_eiscsiserver'});
                return 0;
                }
        if (!$d) {
                &$second_print($text{'mkimage_efirstdisk'});
                return 0;
                }
        my $idisk = &get_iscsi_disk_info($iscsiserver, $d);
        if (!$idisk) {
                &$second_print($text{'mkimage_eiscsidisk'});
                return 0;
                }
        $isdevice = $idisk->{'type'} eq 'file' ? 0 : 1;
        $fsfile = $idisk->{'real'};
        }

# Build the compression command
my $rv;
my ($cmd, $outfile);
if (!$image->{'nocompress'}) {
	# Compress disk image
	&start_background_dot_printer($text{'mkimage_xengzip'});
	my $gzip = &get_gzip_command($vhost);
	$cmd = $isdevice ? &dd_command($fsfile, undef)." | ".$gzip." -c"
			 : $gzip." -c ".quotemeta($fsfile);
	$outfile = $image->{'id'}.".ext3.gz";
	}
else {
	# Just use the image file as-is
	&start_background_dot_printer($text{'mkimage_xengzip2'});
	$outfile = $image->{'id'}.".ext3";
	$cmd = &dd_command($fsfile, undef);
	}

my ($out, $ex, $err);
if ($image_to_server) {
	# Write to a file on the image destination server (likely the master)
	my $perr = &pipe_command_to_server($image_from_server, $image_to_server,
			$cmd, "$image_to_dir/$outfile");
	if ($perr) {
		$out = $perr;
		$ex = 1;
		}
	$rv = 2;
	}
else {
	# Write to a file on the host
	($out, $ex, $err) = &execute_command_on_server($image_from_server,
		$cmd." > ".quotemeta("$image_to_dir/$outfile"));
	$rv = 1;
	}
&stop_background_dot_printer();
if ($ex) {
	&$second_print(&text('mkimage_exenlink', "<pre>$out</pre>"));
	$rv = 0;
	}
else {
	&$second_print($text{'create_done'});
	$image->{'files'} = $outfile;
	}

return $rv;
}

# type_xen_get_image_server(&server)
# If a system's disk is on iSCSI, return the iSCSI server as the disk image
# temp file will be created there. Otherwise it is created on the host.
sub type_xen_get_image_server
{
my ($server) = @_;
if ($server->{'xen_filesystem_iscsi'} ne '') {
	my $iscsiserver = &get_managed_server(
				$server->{'xen_filesystem_iscsi'});
	if ($iscsiserver) {
		return $iscsiserver;
		}
	}
return &get_parent_server($server);
}

# type_xen_cleanup_image(&server, &image, tempdir, [&image-from-server],
#                        [&image-to-server])
# Delete all temp files created on the host system for an image, and reverse
# any munge processes that were done to prepare the imaged system.
sub type_xen_cleanup_image
{
my ($server, $image, $tempdir, $image_from_server, $image_to_server) = @_;
my $vhost = &type_xen_parent($server);
$image_from_server ||= $vhost;

# Remove compressed image temp file, if there was one
if (!$image_to_server) {
	&$first_print($text{'mkimage_xendelete'});
	my $gzfile = $image->{'id'}.".ext3.gz";
	my ($out, $ex, $err) = &execute_command_on_server($image_from_server,
					"rm -f ".quotemeta("$tempdir/$gzfile"));
	$err = $out if ($ex);
	if ($err) {
		&$second_print(&text('mkimage_exendelete', $out));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}
	}

# Re-mount Xen filesystem
&$first_print($text{'mkimage_xenmount'});
my $err = &mount_filesystem_if_needed($server);
if ($err) {
	&$second_print(&text('create_exenmount', $err));
	}
else {
	&$second_print(&text('create_xenmounted',
			     "<tt>$server->{'xen_mount'}</tt>"));

	# Undo all munged files
	&unmunge_all_files($server, $server->{'munged'});
	delete($server->{'munged'});

	# Un-mount Xen filesystem
	&$first_print($text{'mkimage_xenumount'});
	&umount_filesystem_if_needed($server);
	&$second_print($text{'create_done'});
	}

return 1;
}

# type_xen_can_memory(&server)
# Returns 1, indicating that memory can be changed without a reboot for PV
# instances, or 2 for HVM
sub type_xen_can_memory
{
my ($server) = @_;
return $server->{'xen_hvm'} ? 2 : 1;
}

sub type_xen_can_unlimited_memory
{
return 0;
}

sub type_xen_shared_memory
{
return 0;
}

# type_xen_get_memory(&server)
# Returns the amount of RAM in bytes this server is granted, or undef if
# not available.
sub type_xen_get_memory
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return undef if (!ref($conf));
my $m = &xen_find_value("memory", $conf);
return $m ? $m*1024*1024 : undef;
}

# type_xen_save_memory(&server, ram)
# Updates the memory granted in the .cfg file, and apply it with xm set-mem
sub type_xen_save_memory
{
my ($server, $mem) = @_;
my $mbs = int($mem/(1024*1024));
my $conf = &get_xen_config($server, 0);
return "Failed to fetch current configuration" if (!ref($conf));
&xen_save_directive($conf, "memory", $mbs);
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# Apply for real, if possible
if ($server->{'status'} ne 'nohost' && !$config{'xen_mem_mode'}) {
	my $vhost = &type_xen_parent($server);
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm mem-set ".$server->{'xen'}." ".$mbs);
	if ($server->{'status'} ne 'down') {
		# Memory set can fail for older Xen versions if VM is down
		$err = $out if ($ex);
		return "The Xen configuration was updated, but an error ".
		      "occurred applying the new memory limit : $err" if ($err);
		}
	}

$server->{'xen_memory'} = $mbs;
return undef;
}

# Returns 5, indicating that CPU and weighting can be changed without a reboot
sub type_xen_can_cpu
{
return 5;
}

# type_xen_get_cpu(&server)
# Returns the percentage of a CPU granted to this server, or 0 for unlimited,
# and the weighting factor
sub type_xen_get_cpu
{
my ($server) = @_;
if ($server->{'status'} eq 'down') {
	# Use limits from config
	return ( $server->{'xen_credit_cpu'} || 0,
		 $server->{'xen_credit_weight'} || 256 );
	}
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
				"xm sched-credit -d ".$server->{'xen'});
return undef if ($err || $ex);
if ($out =~ /'cap'\s*:\s*(\d+),\s*'weight'\s*:\s*(\d+)/) {
	# Old Xen 3.0 format
	return ($1, $2);
	}
elsif ($out =~ /\S+\s+\d+\s+(\d+)\s+(\d+)/) {
	# New Xen 3.3 format
	return ($2, $1);
	}
else {
	return undef;
	}
}

# type_xen_save_cpu(&server, cpu, weight)
# Updates the CPU cap and weighting in our config file, and applies them
sub type_xen_save_cpu
{
my ($server, $cpu, $weight) = @_;
my $vhost = &type_xen_parent($server);
if (defined($cpu)) {
	# Apply percent now, and save in Cloudmin server object
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm sched-credit -d ".$server->{'xen'}." -c $cpu");
	if ($server->{'status'} ne 'down') {
		return $err if ($err);
		return $out if ($ex);
		}
	$server->{'xen_credit_cpu'} = $cpu;
	}
if (defined($weight)) {
	# Apply weight now, and save in Cloudmin server object
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm sched-credit -d ".$server->{'xen'}." -w $weight");
	if ($server->{'status'} ne 'down') {
		return $err if ($err);
		return $out if ($ex);
		}
	$server->{'xen_credit_weight'} = $weight;
	}
if (defined($cpu) || defined($weight)) {
	# Save in Xen config
	my $conf = &get_xen_config($server, 0);
	if (ref($conf)) {
		if (defined($cpu)) {
			&xen_save_directive($conf, "cpu_cap", $cpu);
			}
		if (defined($weight)) {
			&xen_save_directive($conf, "cpu_weight", $weight);
			}
		&put_xen_config($server, $conf);
		}
	}
return undef;
}

# type_xen_get_cpu_load(&server)
# Computes the current CPU utilization of the given system, by seeing how many
# CPU seconds increased since the last check. Returns a percentage and number
# of seconds it was over, or an empty array of this could not be computed.
sub type_xen_get_cpu_load
{
my ($server) = @_;
return undef if ($server->{'status'} eq 'down' ||
	         $server->{'status'} eq 'nohost');
my $vhost = &type_xen_parent($server);
my $now = time();
my $instances = &list_xen_instances($vhost);
return ( ) if (!ref($instances));
my ($instance) = grep { $server->{'xen'} eq $_->{'instance'} } @$instances;
if ($instance) {
	# Got current .. compare with last time
	my $current = $instance->{'time'};
	my %secs;
	my @rv;
	&read_file_cached($xen_cpu_seconds_file, \%secs);
	if ($secs{$server->{'id'}}) {
		my ($lastsecs, $lasttime) =
			split(/\s+/, $secs{$server->{'id'}});
		if ($lasttime && $lastsecs < $current) {
			@rv = ( 100 * ($current-$lastsecs) / ($now-$lasttime),
				$now-$lasttime );
			}
		}
	$secs{$server->{'id'}} = $current." ".$now;
	&write_file($xen_cpu_seconds_file, \%secs);
	return @rv;
	}
return ( );
}

# type_xen_can_vcpus(&server)
# Returns 2, meaning that virtual CPUs can be managed but a reboot is needed
sub type_xen_can_vcpus
{
return 2;
}

# type_xen_get_vcpus(&server)
# Returns a list of virtual CPU hash refs for this server. Each contains
# a 'num' and 'pin' fields, where the pin is the list of real CPUs it is
# restricted to. On failure, returns an error message.
# Each pin can either be a single number, or a range like 0-1
sub type_xen_get_vcpus
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return $conf if (!ref($conf));
my $vcpus = &xen_find_value("vcpus", $conf);
my @cpus = split(/,/, &xen_find_value("cpus", $conf));
my $cpu = &xen_find_value("cpu", $conf);
$vcpus ||= 1;
my @rv;
my $allcpus = &type_xen_list_pin_cpus($server);
my $pinall;
if (ref($allcpus) && @$allcpus > 1) {
	$pinall = "0-".(scalar(@$allcpus)-1);
	}
for(my $i=0; $i<$vcpus; $i++) {
	my $c = { 'num' => $i };
	if (@cpus) {
		$c->{'pin'} = $cpus[$i];
		}
	elsif ($cpu ne '' && $i == 0) {
		$c->{'pin'} = $cpu;
		}
	if ($c->{'pin'} && $c->{'pin'} eq $pinall) {
		# Really pinned to any
		delete($c->{'pin'});
		}
	push(@rv, $c);
	}
return \@rv;
}

# type_xen_save_vcpus(&server, &vcpus)
# Updates the list of virtual CPUs, and applies them where possible
sub type_xen_save_vcpus
{
my ($server, $vcpus) = @_;
my $conf = &get_xen_config($server, 0);
return $conf if (!ref($conf));

# Work out list of all real CPUs
my $allcpus = &type_xen_list_pin_cpus($server);
my $pinall;
if (ref($allcpus) && @$allcpus > 1) {
	$pinall = "0-".(scalar(@$allcpus)-1);
	}
else {
	$pinall = 0;
	}

&xen_save_directive($conf, "vcpus", scalar(@$vcpus));
if (@$vcpus == 1) {
	# Set single CPU pinning
	&xen_save_directive($conf, "cpu",
		$vcpus[0]->{'pin'} eq '' ? undef : $vcpus[0]->{'pin'});
	&xen_save_directive($conf, "cpus", undef);
	}
else {
	# Set multi-CPU pinning, if any
	&xen_save_directive($conf, "cpu", undef);
	my @pin = grep { $_->{'pin'} ne '' } @$vcpus;
	if (@pin) {
		my @cpus = map { $_->{'pin'} eq '' ? $pinall : $_->{'pin'} }
				  @$vcpus;
		&xen_save_directive($conf, "cpus", join(",", @cpus));
		}
	else {
		# Several CPUs, no pinning
		&xen_save_directive($conf, "cpus", undef);
		}
	}

# Write out the Xen config
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# Apply CPU count and pins now
my $vhost = &type_xen_parent($server);
&run_xen_api_command($vhost,
	"xm vcpu-set ".quotemeta($server->{'xen'})." ".scalar(@$vcpus));
foreach my $cpu (@$vcpus) {
	&run_xen_api_command($vhost,
		"xm vcpu-pin ".quotemeta($server->{'xen'})." ".
		$cpu->{'num'}." ".
		($cpu->{'pin'} eq '' ? $pinall : $cpu->{'pin'}));
	}

$server->{'xen_vcpus'} = scalar(@$vcpus);
return undef;
}

# type_xen_list_pin_cpus(&server)
# Returns a list of CPUs on a host system. Each is a hash ref with 'num' and
# 'desc' fields.
sub type_xen_list_pin_cpus
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $rv = &list_server_cpus($vhost);
return $rv if (!ref($rv));
my ($out, $ex, $err) = &run_xen_api_command($vhost, "xm info");
if ($out =~ /nr_cpus\s*:\s*(\d+)/ && $1 > scalar(@$rv)) {
	# System really has more CPUs than /proc/cpuinfo shows, because some
	# are not available to dom0
	my $realcpus = $1;
	my $desc = $rv->[0]->{'desc'};
	$desc =~ s/,\s+IC.*//;
	$rv = [ ];
	for(my $i=0; $i<$realcpus; $i++) {
		push(@$rv, { 'num' => $i, 'desc' => $desc });
		}
	}
return $rv;
}

# type_xen_get_cpu_count(&server)
# Returns the number of virtual CPUs, cached
sub type_xen_get_cpu_count
{
my ($server) = @_;
return $server->{'xen_vcpus'};
}

# type_xen_can_interfaces(&server)
# Xen interface management is via Webmin or via SSH, so it needs to be up.
# The return value of 2 means that interfaces can also be 'real'
sub type_xen_can_interfaces
{
my ($server) = @_;
if (&can_ssh_status($server)) {
	# Can manage via Webmin or SSH
	return 2;
	}
elsif ($server->{'status'} eq 'down' &&
       (&redhat_server_type($server) || &debian_server_type($server))) {
	# Can manage via direct file manipulation
	return 2;
	}
elsif ($server->{'status'} ne 'nohost') {
	# IPs can still be managed, but in config file .. and cannot be virtual
	return 4;
	}
else {
	# Cannot edit
	return 0;
	}
}

# type_xen_can_mac_address(&server, [&iface])
# Returns 1 if the MAC address for an interface can be edited
sub type_xen_can_mac_address
{
my ($server, $iface) = @_;
return $iface && $iface->{'virtual'} eq '';
}

# Regular interface IP allocation works for Xen
sub type_xen_can_allocate_address
{
return 1;
}

# type_xen_can_gateway(&server)
# Returns 1 indicating that Xen instances can have their default gateway 
# read and set.
sub type_xen_can_gateway
{
my ($server) = @_;
return $server->{'status'} eq 'virt' || $server->{'status'} eq 'novirt' ||
       &redhat_server_type($server) || &debian_server_type($server);
}

# type_xen_can_ipv6(&server)
# Returns 1 if IPv6 addresses can be managed on a system
sub type_xen_can_ipv6
{
my ($server) = @_;
my $can = &type_xen_can_interfaces($server);
return $can == 4 ? 0 : 1;
}

# type_xen_list_interfaces(&server, [force-from-cfg])
# Returns a list of network interfaces on the system, or an error message
sub type_xen_list_interfaces
{
my ($server, $fromcfg) = @_;
my $rv;
my $conf = &get_xen_config($server, 1);
my $vifs = &xen_find_value("vif", $conf);
my $can = &type_xen_can_interfaces($server);
$fromcfg = 1 if ($can == 4);

if ($fromcfg) {
	# Build fake interfaces list from .cfg file only
	my $i = 0;
	$rv = [ ];
	foreach my $vif (@$vifs) {
		my %info = map { split(/=/, $_) } split(/\s*,\s*/, $vif);
		my @ips = split(/\s+/, $info{'ip'});
		next if (!@ips);
		my $iface = { 'name' => 'eth'.$i,
			      'fullname' => 'eth'.$i,
			      'address' => shift(@ips),
			      'netmask' => '255.255.255.0',
			      'ether' => $info{'mac'},
			      'model' => $info{'model'},
			      'bridge' => $info{'bridge'},
			      'rate' => $info{'rate'},
			      'active' => 0, };
		$iface->{'unknown'} = 1 if (!$iface->{'address'});
		push(@$rv, $iface);
		my $j = 0;
		foreach my $vip (@ips) {
			my $viface = { 'name' => $iface->{'name'},
				       'fullname' => $iface->{'name'}.':'.$j,
				       'virtual' => $j,
				       'address' => $vip,
				       'netmask' => $iface->{'netmask'},
				       'active' => 0, };
			push(@$rv, $viface);
			$j++;
			}
		$i++;
		}
	if ($server->{'status'} ne 'down') {
		&set_interface_active_from_ping($rv);
		}
	}
else {
	if ($server->{'status'} eq 'down') {
		# Get from network config files
		my $err = &mount_filesystem_if_needed($server);
		return "Failed to mount filesystem : $err" if ($err);
		$rv = &list_boot_server_interfaces($server);
		&umount_filesystem_if_needed($server);
		}
	else {
		# Get active and boot interfaces
		$rv = &list_active_server_interfaces($server);
		$bootrv = &list_boot_server_interfaces($server);
		if (ref($rv)) {
			foreach my $iface (@$rv) {
				$iface->{'active'} = 1;
				}
			}
		if (ref($rv) && ref($bootrv)) {
			my %got = map { $_->{'fullname'}, $_ } @$rv;
			push(@$rv, grep { !$got{$_->{'fullname'}} &&
					  (&check_ipaddress($_->{'address'}) ||
					   $_->{'address'} eq '') }
					@$bootrv);
			}
		}

	# Set nodelete flag on first real interface, filter out interfaces
	# with no IP or non-ethernet
	if (ref($rv)) {
		$rv = [ grep { $_->{'fullname'} =~ /^eth/ &&
			       $_->{'up'} } @$rv ];
		foreach my $iface (@$rv) {
			$iface->{'active'} ||= 0;
			if ($iface->{'fullname'} eq 'eth0') {
				$iface->{'nodelete'} = 1
				}
			$iface->{'noedit'} = 1 if (!$iface->{'address'} &&
						   !$iface->{'unknown'});
			}
		}

	# Set MAC addresses, models and bridges based on .cfg file
	my $i = 0;
	foreach my $vif (@$vifs) {
		my %info = map { split(/=/, $_) } split(/\s*,\s*/, $vif);
		my ($iface) = grep { $_->{'fullname'} eq "eth".$i } @$rv;
		if ($iface) {
			$iface->{'ether'} = $info{'mac'};
			$iface->{'bridge'} = $info{'bridge'};
			$iface->{'model'} = $info{'model'};
			$iface->{'rate'} = $info{'rate'};
			}
		$i++;
		}
	}

# Set device IDs based on network-list output
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
			"xm network-list $server->{'xen'}");
my $n = 0;
foreach my $l (split(/\r?\n/, $out)) {
	if ($l =~ /^(\d+)\s+(\-?\d+)\s+([0-9a-f:]+)/i) {
		my ($devidx, $devmac) = ($1, $3);
		my ($iface) = grep { lc($_->{'ether'}) eq
				     lc($devmac) } @$rv;
		if (!$iface) {
			($iface) = grep { $_->{'fullname'} eq
					  'eth'.$n } @$rv;
			}
		if ($iface) {
			$iface->{'devidx'} = $devidx;
			}
		$n++;
		}
	}

return $rv;
}

# type_xen_list_parent_interfaces(&server)
# Just lists non-virtual interfaces
sub type_xen_list_parent_interfaces
{
my ($server) = @_;
my $rv = &type_xen_list_interfaces($server);
return [ grep { $_->{'virtual'} eq '' } @$rv ];
}

# type_xen_create_interface(&server, &iface, [non-virtual])
# Add and activate a virtual interface
sub type_xen_create_interface
{
my ($server, $iface, $nonvirt) = @_;
my $can = &type_xen_can_interfaces($server);

# Find a free interface number, if needed
my $vhost = &type_xen_parent($server);
if ($nonvirt) {
	my $conf = &get_xen_config($server, 1);
	my $ips = &xen_find_value("vif", $conf);
	$iface->{'name'} =~ /^eth(\d+)$/ ||
		return "Only Ethernet intefaces can be created";
	$1 == scalar(@$ips) ||
		return "A new real interface must be named eth".scalar(@nets);
	$iface->{'fullname'} = $iface->{'name'};
	}
else{
	my $ifaces = &type_xen_list_interfaces($server);
	return $ifaces if (!ref($ifaces));
	my $n = 0;
	foreach my $oldiface (@$ifaces) {
		if ($oldiface->{'virtual'} >= $n &&
		    $oldiface->{'name'} eq $iface->{'name'}) {
			$n = $oldiface->{'virtual'} + 1;
			}
		}
	$iface->{'virtual'} = $n;
	$iface->{'fullname'} = $iface->{'name'}.":".$n;
	}

# Allocate IP, if needed
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;
my @xen_iface = split(/\s+/, $vhost->{'xen_iface'});
$iface->{'bridge'} ||= $xen_iface[$idx];
if (!$iface->{'address'}) {
	my $cidr;
	($iface->{'address'}, $cidr) = &allocate_ip_address(
	    $vhost->{'xen_start'}, $vhost->{'xen_end'}, $vhost->{'xen_cidr'},
	    $vhost->{'xen_gws'}, $vhost->{'xen_ifaces'}, $iface->{'bridge'});
	$iface->{'address'} ||
		return &text('create_exenalloc', $vhost->{'host'},
			    &describe_ip_range($vhost->{'xen_start'},
					       $vhost->{'xen_end'},
					       $vhost->{'xen_cidr'},
					       $vhost->{'xen_ifaces'},
					       $iface->{'bridge'}));
	$iface->{'netmask'} = &net::prefix_to_mask($cidr);
	}

# Compute broadcast and network
$iface->{'up'} = 1;
$iface->{'broadcast'} = &net::compute_broadcast($iface->{'address'},
						$iface->{'netmask'});
$iface->{'network'} = &net::compute_network($iface->{'address'},
					    $iface->{'netmask'});

# Add a route to the new IP if system is up
if ($server->{'status'} ne 'down' && !$nonvirt) {
	my $instances = &list_xen_instances($vhost);
	if (ref($instances)) {
		my ($instance) = grep { $_->{'instance'} eq $server->{'xen'} }
				      @$instances;
		if ($instance) {
			my $vhostip = &get_server_ip($vhost);
			my $cmd = "ip route add $iface->{'address'} dev ".
				  "vif".$instance->{'id'}.".0 src ".$vhostip;
			&execute_command_on_server($vhost, $cmd);
			}
		}
	}

# Activate now if, which can fail if the system isn't up
my $mac = &generate_mac_address($server, $idx);
if (!$server->{'xen_hvm'} || 1) {
	if ($nonvirt) {
		# Add Xen network device
		my ($out, $ex, $err) = &run_xen_api_command($vhost,
			"xm network-attach $server->{'xen'}".
			" mac=$mac".
			($iface->{'bridge'} ?
				" bridge=$iface->{'bridge'}" : "").
			($iface->{'rate'} ?
				" rate=$iface->{'rate'}" : "").
			" ip=$iface->{'address'}");
		if (($ex || $err) && $server->{'status'} ne 'down') {
			return "Failed to connect network interface : ".
			       ($err || $out);
			}
		}
	if (&can_ssh_status($server) && $can != 4) {
		# Actually ifconfig it up
		my $err = &create_active_server_interface($server, $iface);
		return $err if ($err);
		}
	}

if ($can != 4) {
	# Activate at boot
	my $err = &mount_filesystem_if_needed($server);
	return "Failed to mount filesystem : $err" if ($err);
	$err = &create_boot_server_interface($server, $iface);
	&umount_filesystem_if_needed($server);
	return $err if ($err);
	}

# Add IP to vif= line
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $savevif;
if ($nonvirt) {
	# Add a new vif entry
	$ips->[$idx] = "ip=$iface->{'address'}";
	$ips->[$idx] .= ",mac=$mac";
	if ($iface->{'bridge'}) {
		$ips->[$idx] .= ",bridge=$iface->{'bridge'}";
		}
	if ($iface->{'model'}) {
		$ips->[$idx] .= ",model=".$iface->{'model'};
		}
	$savevif = 1;
	}
else {
	# Add to IP list in existing vif entry
	if ($ips->[$idx]) {
		my @ethips = &split_xen_vif_ips($ips->[$idx]);
		push(@ethips, $iface->{'address'});
		$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
		$savevif = 1;
		}
	}
if ($savevif) {
	&xen_save_directive($conf, "vif", $ips);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);
	}

&add_ip_address($server, $iface->{'address'});
&add_delete_ip6_addresses($server, undef, $iface->{'address6'});
return undef;
}

# type_xen_delete_interface(&server, &iface)
# Delete and de-activate a virtual interface
sub type_xen_delete_interface
{
my ($server, $iface) = @_;
my $vhost = &type_xen_parent($server);
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;
my $can = &type_xen_can_interfaces($server);

if ($can != 4) {
	# Delete boot interface
	my $err = &mount_filesystem_if_needed($server);
	return "Failed to mount filesystem : $err" if ($err);
	$err = &delete_boot_server_interface($server, $iface);
	&umount_filesystem_if_needed($server);
	return $err if ($err);
	}

# De-activate interface if possible. Ignore errors for real interfaces on HVM
# systems, as they generally can't be taken down without a reboot anyway
if (&can_ssh_status($server) && $can != 4) {
	my $err = &delete_active_server_interface($server, $iface);
	return $err if ($err && ($iface->{'virtual'} ne '' ||
				 !$server->{'xen_hvm'}));
	}

if ($iface->{'virtual'} eq '' && $iface->{'devidx'} ne '') {
	# Detach Xen interface
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
	    "xm network-detach $server->{'xen'} $iface->{'devidx'}");
	if (($ex || $err) && $server->{'status'} ne 'down') {
		return "Failed to detach network interface : ".
		       ($err || $out);
		}
	}

# Remove from vif= line
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $savevif;
if ($iface->{'virtual'} eq '') {
	# Delete vif entry
	if ($idx == scalar(@$ips)-1) {
		# Take off last one
		pop(@$ips);
		}
	else {
		# Set to empty
		$ips->[$idx] = '';
		}
	$savevif = 1;
	}
else {
	# Remove IP from vif entry
	if ($ips->[$idx]) {
		my @ethips = &split_xen_vif_ips($ips->[$idx]);
		@ethips = grep { $_ ne $iface->{'address'} } @ethips;
		$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
		$savevif = 1;
		}
	}
if ($savevif) {
	&xen_save_directive($conf, "vif", $ips);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);
	}

if (!$iface->{'unknown'}) {
	&delete_ip_address($server, $iface->{'address'});
	&add_delete_ip6_addresses($server, $iface->{'address6'}, undef);
	}
return undef;
}

# type_xen_modify_interface(&server, &iface, &oldiface)
# Changes the active and boot-time IPs for an interface
sub type_xen_modify_interface
{
my ($server, $iface, $oldiface) = @_;
my $vhost = &type_xen_parent($server);
my $can = &type_xen_can_interfaces($server);

if ($can != 4) {
	# Modify boot interface
	my $err = &mount_filesystem_if_needed($server);
	return "Failed to mount filesystem : $err" if ($err);
	$err = &modify_boot_server_interface($server, $iface, $oldiface);
	&umount_filesystem_if_needed($server);
	return $err if ($err);
	}

# Change active interface if possible
if (&can_ssh_status($server) && $can != 4) {
	my $err = &modify_active_server_interface($server, $iface, $oldiface);
	return $err if ($err);
	}

# Update vif= line in .cfg file
my $conf = &get_xen_config($server, 0);
my $ips = &xen_find_value("vif", $conf);
my $idx = $iface->{'name'} =~ /^eth(\d+)/ ? $1 : 0;
if ($ips->[$idx]) {
	my @ethips = &split_xen_vif_ips($ips->[$idx]);
	@ethips = grep { $_ ne $oldiface->{'address'} } @ethips;
	push(@ethips, $iface->{'address'});
	$ips->[$idx] = &join_xen_vif_ips($ips->[$idx], \@ethips);
	if ($iface->{'virtual'} eq '') {
		$ips->[$idx] =~ s/,mac=[0-9a-f:]+//i;
		if ($iface->{'ether'}) {
			$ips->[$idx] .= ",mac=".$iface->{'ether'};
			}
		$ips->[$idx] =~ s/,model=[^, ]+//i;
		if ($iface->{'model'}) {
			$ips->[$idx] .= ",model=".$iface->{'model'};
			}
		$ips->[$idx] =~ s/,bridge=[^, ]+//i;
		if ($iface->{'bridge'}) {
			$ips->[$idx] .= ",bridge=".$iface->{'bridge'};
			}
		}
	&xen_save_directive($conf, "vif", $ips);
	}

# Change the address line in the .cfg file, if needed
my $oldaddress = &xen_find_value("address", $conf);
if ($oldaddress eq $oldiface->{'address'}) {
	&xen_save_directive($conf, "address", $iface->{'address'});
	}

# Change netmask in the .cfg file
my $oldnetmask = &xen_find_value("netmask", $conf);
if ($oldnetmask && $oldnetmask eq $oldiface->{'netmask'}) {
	&xen_save_directive($conf, "netmask", $iface->{'netmask'});
	}

# Change active bridge if needed, by detaching and re-attaching interface
if ($iface->{'bridge'} ne $oldiface->{'bridge'} ||
    $iface->{'ether'} ne $oldiface->{'ether'} || 
    $iface->{'rate'} ne $oldiface->{'rate'} || 
    $iface->{'address'} ne $oldiface->{'address'}) {
	my $err = &re_attach_xen_interface($vhost, $server, $oldiface, $iface);
	if ($err && $server->{'status'} ne 'down') {
		return $err;
		}

	# Re-activate, in case the system lost the interface
	if (&can_ssh_status($server) && $can != 4) {
		my $err = &modify_active_server_interface(
				$server, $iface, $oldiface);
		return $err if ($err);
		}
	}

# Write out the config
my $err = &put_xen_config($server, $conf);
return "Failed to save Xen config : $err" if ($err);

if (!$iface->{'unknown'}) {
	&delete_ip_address($server, $oldiface->{'address'});
	&add_ip_address($server, $iface->{'address'});
	}
&add_delete_ip6_addresses($server, $oldiface->{'address6'},
				   $iface->{'address6'});
return undef;
}

# type_xen_real_interfaces(&server, &vhost)
# Returns a list of interfaces on the Xen host system associated with some
# virtual server. Each is a hash ref with keys name, bytes_in, bytes_out,
# packets_in, packets_out
sub type_xen_real_interfaces
{
my ($server, $vhost) = @_;

# Get the instance, with caching
$instances = &list_xen_instances($vhost);
return $instances if (!ref($instances));
my ($instance) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
$instance || return "Could not find running Xen instance for $server->{'xen'}";

# Get network stats, with caching
my ($out, $ex, $err);
$out = $proc_net_dev_cache{$vhost->{'id'}};
if (!$out) {
	($out, $ex, $err) = &execute_command_on_server($vhost,
				"cat /proc/net/dev");
	return $out if ($ex);
	return $err if ($err);
	$proc_net_dev_cache{$vhost->{'id'}} = $out;
	}

# If this is an HVM system, find the tap interface. Assumes that in the
# brctl show output, the vifN.x interfaces will be followed by the tap
# interfaces for the same system.
my %tapmap;
if ($server->{'xen_hvm'}) {
	my @lastid;
	my ($bout, $bex, $berr) = &execute_command_on_server(
					$vhost, "brctl show");
	foreach my $l (split(/\r?\n/, $bout)) {
		if ($l =~ /vif(\d+)\.(\d+)$/) {
			push(@lastid, [ $1, $2 ]);
			}
		elsif ($l =~ /(tap\d+)$/ && @lastid) {
			$tapmap{$1} = shift(@lastid);
			}
		else {
			$lastid = ( );
			}
		}
	}

# Extract stats for this host
my @rv;
my %bwifaces = map { $_, 1 } split(/\s+/, $server->{'bw_ifaces'});
foreach my $dev (split(/\r?\n/, $out)) {
	$dev =~ s/^\s+//;
	my @w = split(/[ \t:]+/, $dev);
	my $t = $tapmap{$w[0]};
	if ($w[0] =~ /^\s*vif(\d+)\.\d+$/ && $1 eq $instance->{'id'} ||
	    $server->{'xen_vif'} && $w[0] eq $server->{'xen_vif'} ||
	    $t->[0] == $instance->{'id'}) {
		# Work out eth* device on host
		my $eth;
		if ($w[0] =~ /^\s*vif\d+\.(\d+)$/) {
			$eth = "eth".$1;
			}
		elsif ($t) {
			$eth = "eth".$t->[1];
			}
		else {
			$eth = "eth0";
			}
		if (!%bwifaces || $bwifaces{$eth}) {
			# Directions are reversed, as we are looking at a
			# virtual interface that talks *to* the host
			my $iface = { 'name' => $w[0],
				      'bytes_in' => $w[9],
				      'packets_in' => $w[10],
				      'bytes_out' => $w[1],
				      'packets_out' => $w[2] };
			push(@rv, $iface);
			}
		}
	}
return \@rv;
}

# type_xen_can_disks(&server)
# Returns 1, as we can manage disks for xen instances. These are files or
# devices mapped to virtual partition in the instance.
sub type_xen_can_disks
{
my ($server) = @_;
return $server->{'status'} ne 'nohost' ? 1 : 0;
}

# type_xen_list_models([&server])
# Returns a list of supported emulated network interface card models
# for some virtual system.
sub type_xen_list_models
{
my ($server) = @_;
if ($server && !$server->{'xen_hvm'}) {
	return "Network interface device types can only be set for HVM ".
	       "xen instances";
	}
return [ [ "rtl8139", "Realtek 8139" ],
         [ "ne2k_pci", "NE2000 on PCI" ],
	 [ "ne2k_isa", "NE2000 on ISA" ],
	 [ "e1000", "Intel Gigabit Ethernet" ],
	 [ "pcnet", "PC-Net" ] ];
}

# type_xen_can_safely_resize_disk(&server, &disk)
# Returns undef if some disk can be safely resized, or an error message if not
sub type_xen_can_safely_resize_disk
{
my ($server, $disk) = @_;
if (!$disk->{'fs'}) {
	return $text{'disk_eresizefs'};
	}
elsif ($disk->{'fs'} eq 'xfs') {
	$disk->{'size'} >= $olddisk->{'size'} ||
		return $text{'disk_eshrinkxfs'};
	my $vhost = &type_xen_parent($server);
	&has_command_on_server($vhost, "xfs_growfs") ||
		return $text{'disk_eresizefsxfs'};
	return undef;
	}
elsif ($disk->{'fs'} !~ /^ext\d+$/ && $disk->{'fs'} ne 'swap') {
	return $text{'disk_eresizefstype'};
	}
else {
	return undef;
	}
}

# type_xen_list_disks(&server, include-mounts)
# Returns a list of hash refs, one per disk, with fields :
# real - Real file on host system, like /xen/foo.dsk
# virt - Virtual device on instance, like /dev/sda1
# device - Set to 1 if this is a real device, like /dev/md1
# size - Size in bytes, if a file
# vg - LVM volume group, if any
# lv - LVM logical volume
# swap - Set to 1 if this is the swap disk
sub type_xen_list_disks
{
my ($server, $mounts) = @_;
my $vhost = &type_xen_parent($server);
my $conf = &get_xen_config($server, 1);
return "Failed to get Xen config file" if (!ref($conf));

# Check if any disks look to be on LVM
my $disks = &xen_find_value("disk", $conf);
my $lvmcount = 0;
foreach my $d (@$disks) {
	if ($d =~ /^(file|phy|tap:aio):(\/dev\/[^\/]+\/[^\/]+),/) {
		$lvmcount++;
		}
	}

# If LVM is in use, get LVs and VGs from the host system
my (@lvs, @vgs);
my $lvm_error;
$last_remote_error = undef;
if ($lvmcount || $server->{'xen_filesystem_vg'}) {
	if ($main::list_logical_cache{$vhost->{'id'}}) {
		@lvs = @{$main::list_logical_cache{$vhost->{'id'}}};
		@vgs = @{$main::list_volume_cache{$vhost->{'id'}}};
		}
	else {
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		@vgs = &remote_foreign_call($vhost, "lvm",
			"list_volume_groups");
		$lvm_error = $last_remote_error;
		if (!$lvm_error) {
			$main::list_volume_cache{$vhost->{'id'}} = \@vgs;
			@lvs = ( );
			foreach my $vg (@vgs) {
				push(@lvs, &remote_foreign_call($vhost, "lvm",
					"list_logical_volumes", $vg->{'name'}));
				}
			$main::list_logical_cache{$vhost->{'id'}} = \@lvs;
			}
		}
	}
my %vgnames = map { $_->{'name'}, $_ } @vgs;

# Parse disks in Xen config
my @rv;
foreach my $d (@$disks) {
	if ($d =~ /^(file|phy|tap:aio):(\S+),([^:]+)(:(\S+))?,(\S+)$/) {
		# A regular file
		my $disk = { 'real' => $2,
			     'virt' => "/dev/$3",
			     'virtdev' => $3,
			     'suffix' => $1,
			     'media' => $5 || 'disk',
			     'mode' => $6, };
		$disk->{'device'} = $1 eq "phy";
		if ($disk->{'media'} eq 'cdrom') {
			# CD size cannot be changed, but ISO file can
			$disk->{'change'} = 1;
			$disk->{'fixedsize'} = 1;
			}
		if (!$disk->{'device'}) {
			my @st = &stat_file_on_server($vhost, "$2");
			$disk->{'size'} = $st[7];
			}
		if ($disk->{'real'} =~ /^\/dev\/([^\/]+)\/([^\/]+)$/ &&
		    $vgnames{$1}) {
			# Disk is on LVM
			$disk->{'vg'} = $1;
			$disk->{'lv'} = $2;
			my ($lv) = grep { $_->{'name'} eq $disk->{'lv'} } @lvs;
			if ($lv) {
				$disk->{'size'} = $lv->{'size'} * 1024;
				$disk->{'lv_snap_of'} = $lv->{'snap_of'};
				$disk->{'thin'} = $lv->{'thin_in'};
				}
			elsif ($lvm_error) {
				# Disk is on LVM, but failed to get details
				return "Failed to fetch LVM devices : $lvm_error";
				}
			else {
				return "Logical volume $disk->{'lv'} was not found";
				}
			if ($disk->{'thin'}) {
				$disk->{'storage_id'} =
				    'thin_'.$disk->{'vg'}.'_'.$disk->{'thin'};
				$disk->{'storage'} = &text('storage_thin',
							   $disk->{'thin'});
				}
			else {
				$disk->{'storage_id'} = 'lvm_'.$disk->{'vg'};
				$disk->{'storage'} = &text('storage_lvm',
							   $disk->{'vg'});
				}
			}
		elsif ($disk->{'real'} =~ /^\/dev\/disk\/by-path\/ip-(\S+):(\d+)-iscsi-(\S+):(\S+)-lun-(\d+)$/) {
			# Disk is on an iSCSI server
			$disk->{'iscsi_ip'} = $1;
			$disk->{'iscsi_port'} = $2;
			$disk->{'iscsi_name'} = $3;
			$disk->{'iscsi_target'} = $4;
			$disk->{'iscsi_lun'} = $5;
			$disk->{'iscsi_connect'} = 0;
			$disk->{'fixedreal'} = 1;
			my $iscsiserver = &get_managed_server_by(
						"ip", $disk->{'iscsi_ip'});
			if ($iscsiserver) {
				$disk->{'iscsi_server'} = $iscsiserver->{'id'};
				}

			# Get size from /proc/partitions on host
			my $realdevice;
			my ($out, $ex, $err) = &execute_command_on_server(
					$vhost, "readlink $disk->{'real'}");
			if (!$ex && !$err && $out =~ /\/(sd[a-z]+)/) {
				$realdevice = $1;
				}
			($out, $ex, $err) = &execute_command_on_server(
						$vhost, "cat /proc/partitions");
			if (!$ex && !$err && $realdevice) {
				foreach my $l (split(/\r?\n/, $out)) {
					$l =~ s/^\s+//;
					my ($major, $minor, $blocks, $name) =
						split(/\s+/, $l);
					if ($name eq $realdevice) {
						$disk->{'size'} = $blocks*1024;
						$disk->{'iscsi_connect'} = 1;
						}
					}
				}
			$disk->{'storage_id'} = 'iscsi_'.$iscsiserver->{'host'};
			$disk->{'storage'} = &text('storage_iscsi',
						   $iscsiserver->{'host'});
			}
		else {
			# Disk is just in a regular file
			$disk->{'storage_id'} = 'files';
			$disk->{'storage'} = $text{'storage_files'};
			}
		$disk->{'index'} = scalar(@rv);
		if ($disk->{'real'} eq $server->{'xen_swapfile'}) {
			$disk->{'swap'} = 1;
			}
		$disk->{'format'} = $disk->{'virt'} =~ /\d+$/ ? 'disk' : '';
		push(@rv, $disk);
		}
	}

# If the caller asked for mounts, get them too
if ($mounts) {
	# Get fstab mounts
	my $fstab = &get_system_fstab($server);
	return $fstab if (!ref($fstab));
	foreach my $d (@rv) {
		my $f;
		if ($server->{'xen_wholedisk'}) {
			($f) = grep { &same_xen_virt($_->[0], $d->{'virt'},1) &&
				      $_->[2] ne 'swap' }
				    &sort_system_fstab(@$fstab);
			}
		else {
			($f) = grep { &same_xen_virt($_->[0], $d->{'virt'}, 0) }
				    @$fstab;
			}
		if (!$f && $server->{'xen_wholedisk'}) {
			# Check if a partition has an ext label
			if (!$got_label_to_part) {
				%label_to_part = &get_label_map($server);
				$got_label_to_part = 1;
				}
			my $label;
			foreach my $l (keys %label_to_part) {
				my $p = $label_to_part{$l};
				if (&same_xen_virt($p, $d->{'virt'}, 1)) {
					$label = $l;
					last;
					}
				}
			if ($label) {
				($f) = grep { $_->[0] eq 'LABEL='.$label ||
					     uc($_->[0]) eq 'UUID='.uc($label) }
					    @$fstab;
				if ($f) {
					$f->[0] = $label_to_part{$label};
					$d->{'virt_label'} = $label;
					}
				}
			}
		my ($rootf) = grep { $_->[1] eq '/' } @$fstab;
		if (!$f && $server->{'xen_wholedisk'}) {
			# If no fstab entry was found but there is only one
			# disk with one partition, it must be the root
			my $parts = &list_disk_image_partitions($vhost,
								$d->{'real'});
			if ($rootf && ref($parts) && @$parts == 1 &&
			    $d->{'real'} eq $server->{'xen_filesystem'}) {
				$f = $rootf;
				$f->[0] = $d->{'virt'}.'1';
				}
			}
		if (!$f && !$server->{'xen_wholedisk'}) {
			# Similar check if there is only one filesystem image
			if ($rootf &&
			    $d->{'real'} eq $server->{'xen_filesystem'}) {
				$f = $rootf;
				$f->[0] = $d->{'virt'};
				}
			}
		if ($f) {
			$d->{'virt_mount'} = $f->[0];
			$d->{'virt_onsystem'} = $f->[0];
			if ($server->{'xen_wholedisk'}) {
				$d->{'virt_onsystem'} =~ s/\d+$//;
				}
			$d->{'mount'} = $f->[1];
			$d->{'fs'} = $f->[2];
			if ($d->{'virt_mount'} =~ /(\d+)$/ &&
			    $d->{'virt'} !~ /(\d+)$/) {
				# virt_part is the partition number on the
				# underlying whole-disk image
				$d->{'virt_part'} = $1;
				}
			if ($d->{'fs'} eq 'swap') {
				$d->{'swap'} = 1;
				}
			}
		}

	# Check if actually mounted
	if (&can_ssh_status($server)) {
		my $mounted = &get_mounted_map($server);
		foreach my $d (@rv) {
			$d->{'mounted'} = $mounted->{$d->{'virt'}} ||
					  $mounted->{$d->{'virt_mount'}};
			}
		}
	}

# If caller asked for disk usage, get it too if we can
if ($mounts == 2 && &can_ssh_status($server)) {
	my ($out, $ex, $err) = &execute_command_on_server(
		$server, "df -k -l || df -k");
	if (!$ex && !$err) {
		foreach my $l (split(/\r?\n/, $out)) {
			my ($fs, $size, $used, $free) = split(/\s+/, $l);
			my ($disk) = grep { $_->{'virt'} eq $fs ||
					    $_->{'virt_mount'} eq $fs } @rv;
			if ($disk) {
				$disk->{'df_size'} = $size*1024;
				$disk->{'df_used'} = $used*1024;
				$disk->{'df_free'} = $free*1024;
				}
			}
		}
	}

# If the system is up, set disk block IDs
my $xenid = $server->{'status'} eq 'down' ? $server->{'xen'}
					  : &get_xen_instance_id($server);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
				"xm block-list $xenid");
if (!$ex && !$err) {
	my $n = 0;
	foreach my $l (split(/\r?\n/, $out)) {
		if ($l =~ /^(\d+)\s/ && $n < @rv) {
			$rv[$n++]->{'blockid'} = $1;
			}
		}
	}

return \@rv;
}

# type_xen_list_disk_partitions(&server, &disk)
# Returns an array ref of partitions on some disk
sub type_xen_list_disk_partitions
{
my ($server, $disk) = @_;
my $vhost = &type_xen_parent($server);
return &list_disk_image_partitions($vhost, $disk->{'real'});
}

# type_xen_need_mount(&server)
# Returns 1 if a mount call is needed before reading files from this system
sub type_xen_need_mount
{
my ($server) = @_;
return ($server->{'status'} eq 'down' ||
        $server->{'status'} eq 'create') && !$server->{'xen_mount'};
}

# type_xen_mount_filesystem(&server, [wantfile], [mountpoint])
# Mounts a down Xen image's root disk image on /mnt/something, and sets
# xen_mount in the server object
sub type_xen_mount_filesystem
{
my ($server, $wantfile, $mp) = @_;
my $vhost = &type_xen_parent($server);
$mp ||= "/mnt/xen-".$server->{'xen'};
my $err = &make_dir_on_server($vhost, $mp, 0700, 1);
return "Failed to create mount point $mp : $err" if ($err);
my $fsfile = $server->{'xen_filesystem'};
if (!$fsfile) {
	return &text('create_exenfsfile');
	}
if ($server->{'xen_wholedisk'}) {
	# Find the right partition inside the disk image
	# If we don't know which partition is the primary, try them in series
	if (!$server->{'xen_filesystem_offset'}) {
		my $parts = &list_disk_image_partitions($vhost, $fsfile);
		if (!ref($parts)) {
			return &text('create_ekvmfsfdisk', $fsfile, $parts);
			}

		# Find Linux partitions and their offsets
		my @offsets;
		foreach my $p (@$parts) {
			if ($p->{'type'} eq '83' && $p->{'offset'}) {
				push(@offsets, $p->{'offset'});
				}
			}
		@offsets || return &text('create_ekvmfslinux', $fsfile);

		# Try each offset
		foreach my $o (@offsets) {
			my ($out, $ex, $err) = &execute_command_on_server(
				$vhost,
				"mount -o loop,offset=$o ".quotemeta($fsfile).
				" ".quotemeta($mp));
			next if ($ex || $err);
			my @st = &stat_file_on_server($vhost, "$mp/etc");
			if (@st) {
				# Found it! Leave mounted
				$server->{'xen_mount'} = $mp;
				$server->{'xen_filesystem_offset'} = $o;
				return undef;
				}
			else {
				# Un-mount and try next one
				&execute_command_on_server($vhost,
					"umount ".quotemeta($mp));
				}
			}
		return &text('create_ekvmfsnone', $fsfile, "/etc");
		}

	# Mount with the correct offset
	my $o = $server->{'xen_filesystem_offset'};
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"mount -o loop,offset=$o ".quotemeta($fsfile)." ".
					   quotemeta($mp));
	return "Failed to mount $fsfile on $mp : $out" if ($ex);
	}
else {
	# Can just mount the filesystem image directly
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"mount -o loop ".quotemeta($fsfile)." ".
				 quotemeta($mp));
	return "Failed to mount $fsfile on $mp : $out" if ($ex);
	}
$server->{'xen_mount'} = $mp;
return undef;
}

# type_xen_umount_filesystem(&server)
# Removes the mount for a system's disk image
sub type_xen_umount_filesystem
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &execute_command_on_server($vhost,
	"umount ".quotemeta($server->{'xen_mount'}));
delete($server->{'xen_mount'});
return undef;
}

# type_xen_new_disk(&server, &disks, real?, [&storage], [disk-index])
# Returns a structure for a new disk on some system
sub type_xen_new_disk
{
my ($server, $disks, $device, $storage, $index) = @_;
my $rv = { };

# Find which device files are used
my %used;
foreach my $d (@$disks) {
	$used{$d->{'virt'}}++;
	if ($d->{'virt'} =~ /^(\/dev\/(s|h|xv)d)([a-z])$/) {
		$used{"/dev/sd".$3}++;
		$used{"/dev/hd".$3}++;
		$used{"/dev/xvd".$3}++;
		}
	}
my $fstab = &get_system_fstab($server);
if (ref($fstab)) {
	foreach my $f (@$fstab) {
		if ($f->[0] =~ /^\/dev\//) {
			$used{$f->[0]}++;
			}
		}
	}

# Pick a /dev/???? file that is free
my $n;
if ($disks->[0]->{'virt'} =~ /^(\/dev\/(s|h|xv)d[a-z])\d+$/) {
	# System uses /dev/sdaN files, pick a free N
	$prefix = $1;
	for($n=1; $used{$prefix.$n}; $n++) { }
	$rv->{'virt'} = $prefix.$n;
	if ($disks->[0]->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d[a-z])\d+$/) {
		$rv->{'virt_onsystem'} = $1.$n;
		}
	else {
		$rv->{'virt_onsystem'} = $rv->{'virt'};
		}
	$rv->{'virt_mount'} = $rv->{'virt_onsystem'};
	$rv->{'format'} = '';
	}
elsif ($disks->[0]->{'virt'} =~ /^(\/dev\/(s|h|xv)d)[a-z]$/) {
	# System uses whole disks! Pick the next disk
	$prefix = $1;
	for($n='a'; $used{$prefix.$n}; $n=chr(ord($n)+1)) { }
	$prefix .= $n;
	$rv->{'virt'} = $prefix;
	if ($disks->[0]->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d)[a-z]$/) {
		$rv->{'virt_onsystem'} = $1.$n;
		}
	else {
		$rv->{'virt_onsystem'} = $rv->{'virt'};
		}
	$rv->{'virt_mount'} = $rv->{'virt_onsystem'}."1";
	$rv->{'virt_part'} = 1;
	$rv->{'format'} = 'disk';
	}
else {
	# No idea .. just guess
	$rv->{'virt'} = "/dev/sda3";
	$rv->{'format'} = '';
	}
$rv->{'virt_auto'} = $rv->{'virt'};

if (!$device) {
	my $vhost = &type_xen_parent($server);
	my ($vg, $thin);
	my $iscsi;
	if ($storage) {
		if ($storage->{'id'} =~ /^lvm_(\S+)/) {
			$vg = $1;
			}
                elsif ($storage->{'id'} =~ /^thin_(\S+)_(\S+)/) {
                        $vg = $1;
                        $thin = $2;
                        }
		elsif ($storage->{'id'} =~ /^iscsi_(\S+)/) {
			my $iscsiserver = &get_managed_server_by("host", "$1");
			$iscsi = $iscsiserver->{'id'};
			}
		elsif ($storage->{'id'} eq 'iscsi') {
			$iscsi = $server->{'xen_filesystem_iscsi'};
			}
		$rv->{'storage_id'} = $storage->{'id'};
		}
	else {
		$vg = $server->{'xen_filesystem_vg'};
		$iscsi = $server->{'xen_filesystem_iscsi'};
		$rv->{'storage_id'} = $vg ? 'lvm_'.$vg :
				      $iscsi ne '' ? 'iscsi' : 'files';
		}
	my $filenum = defined($index) ? $index : (scalar(@$disks)+1);
	if ($vg) {
		# Create an LVM name
		$rv->{'vg'} = $vg;
		$rv->{'lv'} = $server->{'xen'}."_".$filenum."_img";
		$rv->{'thin'} = $thin;
		$rv->{'real'} = "/dev/".$rv->{'vg'}."/".$rv->{'lv'};
		$device = 1;
		}
	elsif ($iscsi ne '') {
		# Disk will be on an iSCSI server. The real field gets set later
                $rv->{'iscsi_server'} = $iscsi;
                $rv->{'fixedreal'} = 1;
                }
	else {
		# Pick an image file
		my $xen_dir = ($storage ? $storage->{'directory'} : undef) ||
			      $server->{'new_directory'} ||
			      $vhost->{'xen_dir'};
		$rv->{'real'} = "$xen_dir/$server->{'xen'}.".
				$filenum.".img";
		$rv->{'dir'} = $xen_dir;
		}
	}
$rv->{'device'} = $device;
$rv->{'media'} = 'disk';
return $rv;
}

# type_xen_create_disk(&server, &disk, [bootable], [don't-add-to-config])
# Adds a new virtual disk to a Xen system's config.
# If this is a real file that doesn't exist yet, create it and optionally mkfs
# it. Also add to the /etc/fstab file, if requested.
sub type_xen_create_disk
{
my ($server, $disk, $bootable, $noconfig) = @_;
my $vhost = &type_xen_parent($server);

# Fix up virt_mount if virt was changed
if ($disk->{'virt_auto'} && $disk->{'virt'} ne $disk->{'virt_auto'}) {
	if ($disk->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d)[a-z]/) {
		# Convert the entered device path that appears in the .cfg
		# file like /dev/sda1 to the path that is used inside the VM,
		# like /dev/vda1
		my $prefix = $1;
		if ($disk->{'virt'} =~ /^\/dev\/(s|h|xv)d([a-z]\d*)$/) {
			$disk->{'virt_onsystem'} = $prefix.$2;
			}
		}
	if ($disk->{'format'} eq 'disk') {
		$disk->{'virt_mount'} = $disk->{'virt_onsystem'}.'1';
		}
	else {
		$disk->{'virt_mount'} = $disk->{'virt_onsystem'};
		}
	}

my @st = &recursive_stat_file_on_server($vhost, $disk->{'real'});
if ($disk->{'device'} && !$disk->{'lv'} || $disk->{'new'} == 2) {
	# Check if the device exists
	@st || return "Device $disk->{'real'} does not exist ".
		      "on $vhost->{'host'}";
	}
else {
	@st || $disk->{'size'} || return "Missing size for new disk";
	if ($disk->{'lv'} && $disk->{'overwrite'}) {
		# Delete existing LV
		&delete_all_disk_image_partitions($vhost, $disk->{'real'});
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		&remote_foreign_call($vhost, "lvm",
				     "delete_logical_volume",
				     { 'device' => $disk->{'real'} });
		}
	elsif (@st && $disk->{'overwrite'}) {
		# Delete existing file
		&delete_file_on_server($vhost, $disk->{'real'});
		@st = ( );
		}

	if ($disk->{'lv'}) {
		# Create the logical volume
		$last_remote_error = undef;
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		my $lv = { 'name' => $disk->{'lv'},
			   'size' => int($disk->{'size'}/1024),
			   'perm' => 'rw',
			   'alloc' => 'n',
			   'vg' => $disk->{'vg'},
			   'thin' => $disk->{'thin'}, };
		my $err = &remote_foreign_call($vhost, "lvm",
						  "create_logical_volume", $lv);
		$err ||= $last_remote_error;
		return "LVM logical volume creation failed : $err" if ($err);

		# Invalidate LV cache for host
		delete($main::list_logical_cache{$vhost->{'id'}});
		}
	elsif ($disk->{'iscsi_server'} ne '' && $disk->{'iscsi_target'}) {
		# Attach to iSCSI disk
		my $err = &attach_iscsi_disk($server, $disk);
		return $err if ($err);
		@st = &stat_file_on_server($vhost, $disk->{'real'});
		}
	elsif ($disk->{'iscsi_server'} ne '' && !$disk->{'iscsi_target'}) {
		# Create iSCSI disk
		my $err = &create_iscsi_disk($server, $disk);
		return $err if ($err);
		}
	elsif (!@st) {
		# Create the file if missing
		$disk->{'size'} || return "File $disk->{'real'} does not exist on $vhost->{'host'}, but no size to create it as was given";
		my $cmd = &dd_command("/dev/zero", $disk->{'real'},
				      $disk->{'size'});
		my ($out, $ex, $err) = &execute_command_on_server(
						$vhost, $cmd);
		$err = $out if ($ex);
		$err && return "Failed to create file $disk->{'real'} : $err";
		}

	# Create a partition on the disk, if in whole-disk mode
	if (!@st && $server->{'xen_wholedisk'}) {
		my $err = &create_disk_image_partition($vhost, $disk->{'real'},
						       1, $disk->{'size'}, 1);
		return $err if ($err);
		}

	# And make a filesystem on it
	if (!@st && $disk->{'fs'}) {
		my $err;
		if ($server->{'xen_wholedisk'}) {
			my $err = &create_disk_image_filesystem(
				$vhost, $disk->{'real'}, $disk->{'fs'}, 1);
			}
		else {
			my $err = &create_device_filesystem(
				$vhost, $disk->{'real'}, $disk->{'fs'});
			}
		return "Failed to create ".uc($disk->{'fs'}).
		       " filesystem : $err" if ($err);
		}
	}

# Add the disk to the Xen config
my $mode = $disk->{'media'} eq 'cdrom' ? 'r' : 'w';
my $conf = &get_xen_config($server, 0);
return "Failed to get Xen config file : $conf" if (!ref($conf));
my $disks = &xen_find_value("disk", $conf);
my $shortvirt = $disk->{'virt'};
$shortvirt =~ s/^\/dev\///;
my $dtype = &compute_xen_disk_prefix($disk, $vhost);
if (!$noconfig) {
	push(@$disks, $dtype.":".$disk->{'real'}.",".$shortvirt.
		      ($disk->{'media'} eq 'disk' ? '' : ':'.$disk->{'media'}).
		      ",".$mode);
	&xen_save_directive($conf, "disk", $disks);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);
	}

# Add block device on server now
my $xenid = $server->{'status'} eq 'down' ? $server->{'xen'}
					  : &get_xen_instance_id($server);
my $attacherr;
if ($xenid) {
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
	    "xm block-attach $xenid $dtype:$disk->{'real'} ".
	    "$disk->{'virt'} $mode");
	if ($server->{'status'} ne 'down') {
		$attacherr = $err ? $err : $ex ? $out : undef;
		$disk->{'reboot'} = 1 if ($attacherr);
		}
	}

# Add to total disk size
if (!$noconfig && $server->{'xen_disktotal'} ne '' &&
    (!$disk->{'device'} || $disk->{'lv'})) {
	$server->{'xen_disktotal'} += $disk->{'size'};
	}

if (!$server->{'xen_filesystem'} &&
    ($disk->{'media'} eq 'disk' || $disk->{'media'} eq '')) {
	# This is the new root disk
	&set_server_disk_fields($server, $disk, 'filesystem');
	}
if (!$server->{'xen_swapfile'} && $disk->{'fs'} eq 'swap' &&
    ($disk->{'media'} eq 'disk' || $disk->{'media'} eq '')) {
	# This is the new swap disk
	&set_server_disk_fields($server, $disk, 'swapfile');
	}

# If mounting a CD, there are no partitions
if ($disk->{'media'} eq 'cdrom') {
	$disk->{'virt_mount'} = $disk->{'virt'};
	}

# Add to /etc/fstab, if a mount point was given
my $fserr;
if ($disk->{'mount'} && $server->{'status'} ne 'alive') {
	&mount_filesystem_if_needed($server);
	if ($disk->{'fs'} ne 'swap') {
		my $err = &make_dir_on_server(
			$server, $disk->{'mount'}, 0755, 1);
		return "Failed to create mount point : $err" if ($err);
		}
	my $fstab = &get_system_fstab($server);
	$fserr = "Failed to read fstab file : $fstab" if (!ref($fstab));
	if (!$fserr) {
		my $fs = $disk->{'fs'} ? $disk->{'fs'} :
			  $disk->{'media'} eq 'cdrom' ? 'iso9660' : 'ext3';
		push(@$fstab, [ $disk->{'virt_mount'} || $disk->{'virt'},
				$disk->{'mount'},
				$fs, 'defaults', 0, 1 ]);
		my $err = &put_system_fstab($server, $fstab);
		$fserr = "Failed to update fstab file : $err" if ($err);
		}
	&umount_filesystem_if_needed($server);
	}

# Actually mount if possible
if ($disk->{'mount'} && &can_ssh_status($server) && !$attacherr &&
    !$server->{'xen_hvm'}) {
	my $cmd;
	my $virt = $disk->{'virt_mount'} || $disk->{'virt'};
	if ($disk->{'fs'} eq 'swap') {
		$cmd = "swapon ".quotemeta($virt);
		}
	else {
		$cmd = "mount ".quotemeta($virt)." ".
			 	quotemeta($disk->{'mount'});
		}
	my ($out, $ex, $err) = &execute_command_on_server($server, $cmd);
	if ($ex && $out =~ /already\s+mounted/) {
		# Xen instance auto-mounted when attached
		$ex = 0;
		}
	if ($ex || $err) {
		return "Failed to mount $virt on $disk->{'mount'} : ".
		       ($err || $out);
		}
	}

return $fserr;
}

# type_xen_delete_disk(&server, &disk, [detach-only], [leave-fstab],
# 		       [don't-remove-from-config])
# Removes a disk from the Xen configuration, the system's fstab file, and
# deletes the underlying config file. The disk object must come from a call
# to type_xen_list_disks with the mount flag set to 1.
sub type_xen_delete_disk
{
my ($server, $disk, $detach, $nofstab, $noconfig) = @_;
&temp_umount_filesystem_if_needed($server);
my $vhost = &type_xen_parent($server);

# Actually un-mount if mounted
if (&can_ssh_status($server) && $disk->{'mounted'}) {
	my $dev = $disk->{'virt_mount'} || $disk->{'virt'};
	my ($out, $ex, $err) = &execute_command_on_server($server,
		$disk->{'fs'} eq 'swap' ?
			"swapoff ".$dev :
			"umount ".$dev);
	if ($err || $ex) {
		return "Failed to un-mount $disk->{'virt'} : ".
		       ($err || $out);
		}
	}

# Dis-connect block device
my $xenid = $server->{'status'} eq 'down' ? $server->{'xen'}
					  : &get_xen_instance_id($server);
if ($server->{'status'} ne 'down' && $xenid && $disk->{'blockid'}) {
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm block-detach $xenid $disk->{'blockid'}");
	if ($server->{'status'} ne 'down') {
		if ($ex || $err) {
			$disk->{'reboot'} = 1;
			return "Failed to detach Xen block device ".
			       "$disk->{'blockid'} :".($err || $out);
			}
		if (!&wait_for_xen_block_detach($vhost, $xenid,
						$olddisk->{'blockid'})) {
			$disk->{'reboot'} = 1;
			return "Xen block device did not actually detach";
			}
		}
	}

# Remove from the Xen config
if (!$noconfig) {
	my $conf = &get_xen_config($server, 0);
	return "Failed to get Xen config file : $conf" if (!ref($conf));
	my $disks = &xen_find_value("disk", $conf);
	my $suffix = $disk->{'suffix'};
	@$disks = grep { !/^\Q$suffix\E:\Q$disk->{'real'}\E,/ } @$disks;
	&xen_save_directive($conf, "disk", $disks);
	my $err = &put_xen_config($server, $conf);
	return "Failed to save Xen config : $err" if ($err);

	# Remove from total disk size
	if ($server->{'xen_disktotal'} ne '' &&
	    (!$disk->{'device'} || $disk->{'lv'})) {
		$server->{'xen_disktotal'} -= $disk->{'size'};
		}

	# Main drive has been removed, so remove reference to it in config
	if ($server->{'xen_filesystem'} eq $disk->{'real'}) {
		delete($server->{'xen_filesystem'});
		}
	if ($server->{'xen_swapfile'} eq $disk->{'real'}) {
		delete($server->{'xen_swapfile'});
		}
	}

# Remove from fstab
if (($server->{'status'} eq 'down' || &can_ssh_status($server)) && !$nofstab) {
	my $fstab = &get_system_fstab($server);
	return "Failed to read fstab file : $fstab" if (!ref($fstab));
	my @newfstab = grep { !&same_xen_virt($_->[0], $disk->{'virt'},
					      $server->{'xen_wholedisk'}) }
			    &sort_system_fstab(@$fstab);
	if (scalar(@newfstab) != scalar(@$fstab)) {
		my $err = &put_system_fstab($server, \@newfstab);
		$fserr = "Failed to update fstab file : $err" if ($err);
		}
	}

if (!$detach) {
	&delete_all_disk_image_partitions($vhost, $disk->{'real'});
	if (!$disk->{'device'} && $disk->{'real'} !~ /^\/dev\//) {
		# Delete the image file
		&execute_command_on_server($vhost,
			"rm -f ".quotemeta($disk->{'real'}));
		}
	elsif ($disk->{'lv'}) {
		# Delete mapper entries for LV, if any
		&delete_mapper_entries($vhost, $disk->{'vg'}, $disk->{'lv'});

		# Delete the logical volume
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		&remote_foreign_call($vhost, "lvm", "delete_logical_volume",
				     { 'device' => $disk->{'real'} });

		# Invalidate LV cache for host
		delete($main::list_logical_cache{$vhost->{'id'}});
		}
        elsif ($disk->{'iscsi_server'} ne '') {
                # Delete from iSCSI server
                my $err = &delete_iscsi_disk($server, $disk);
                return $err if ($err);
		}
	if ($disk->{'lv'} && $disk->{'lv_snap_of'}) {
		# LV is a snapshot .. does any existing system use it?
		my $srcreal = &find_snapshot_source_logical_volume(
				$server, $disk->{'vg'}, $disk->{'lv_snap_of'});
		if ($srcreal) {
			# Source of snapshot is un-used, likely because it was
			# deleted before this clone. Delete the source LV.
			&delete_all_disk_image_partitions($vhost, $srcreal);
			&delete_mapper_entries($vhost, $disk->{'vg'},
						$disk->{'lv_snap_of'});
			&remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume",
					     { 'device' => $srcreal });
			}
		}
	}

return undef;
}

# type_xen_modify_disk(&server, &disk, &old-disk)
# Modifies the virtual device and size of a disk. Returns undef if OK, or an
# error message on failure.
# The disk objects *must* have the mount and fs fields set, if mounted.
sub type_xen_modify_disk
{
my ($server, $disk, $olddisk) = @_;
&temp_umount_filesystem_if_needed($server);
my $vhost = &type_xen_parent($server);
my $moderr;
my $parts;
my $resizefs = 0;
if ($disk->{'virt'} !~ /\d$/) {
	# If this is a whole disk image, get the partitions on it.
	# Not needed for filesystem images.
	$parts = &list_disk_image_partitions($vhost, $olddisk->{'real'});
	}
my $part = ref($parts) ? $parts->[@$parts - 1] : undef;

# Connect iSCSI now if requested
if ($disk->{'iscsi_ip'} && $disk->{'iscsi_connect'} &&
    !$olddisk->{'iscsi_connect'}) {
        my $err = &connect_iscsi_disk($server, $disk);
        return $err if ($err);
        }

# Perform some basic validation for a supported filesystem type, and
# a partition in the right place
if ($disk->{'size'} != $olddisk->{'size'}) {
	$disk->{'fs'} =~ /^ext\d+$/ || $disk->{'fs'} eq 'swap' ||
 	  $disk->{'fs'} eq 'xfs' || $disk->{'fs'} eq '' ||
	    return "The size of a disk with the ".uc($disk->{'fs'}).
		   " filesystem cannot be changed";
	if (ref($parts) && ($disk->{'fs'} =~ /^ext\d+$/ ||
			    $disk->{'fs'} eq 'xfs' ||
			    $disk->{'fs'} eq 'swap')) {
		$part || return "No partitions were found on the disk image!";
		$disk->{'virt_part'} == scalar(@$parts) ||
			return "Only the last partition on a disk image can ".
			       "be resized";
		$resizefs = 1;
		}
	}
my $conf = &get_xen_config($server, 0);
return "Failed to get Xen config file : $conf" if (!ref($conf));

# Make sure new disk file exists
if ($disk->{'real'} ne $olddisk->{'real'}) {
	my @st = &stat_file_on_server($vhost, $disk->{'real'});
	@st || return "Device $disk->{'real'} does not exist ".
		      "on $vhost->{'host'}";
	}

# If the system is up, un-mount the disk
my $mounted = 0;
if (&can_ssh_status($server) && $olddisk->{'mounted'}) {
	$olddisk->{'virt_mount'} eq '/' &&
		return "The root disk of a running system cannot be resized";
	my $dev = $olddisk->{'virt_mount'} || $olddisk->{'virt'};
	my ($out, $ex, $err) = &execute_command_on_server($server,
		$olddisk->{'fs'} eq 'swap' ?
			"swapoff ".$dev :
			"umount ".$dev);
	if ($err || $ex) {
		return "Failed to un-mount $olddisk->{'virt'} : ".
		       ($err || $out);
		}
	$mounted = 1;
	}

# If the system is up, disconnect the disk from the VM
# XXX is this needed if down??
my $connected = 0;
my $xenid = $server->{'status'} eq 'down' ? $server->{'xen'}
					  : &get_xen_instance_id($server);
if ($server->{'status'} ne 'down' && $xenid && $olddisk->{'blockid'}) {
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm block-detach $xenid $olddisk->{'blockid'}");
	if ($ex || $err) {
		return "Failed to detach Xen block device ".
		       "$olddisk->{'blockid'} : ".($err || $out);
		}
	if (!&wait_for_xen_block_detach($vhost, $xenid,
					$olddisk->{'blockid'})) {
		return "Xen block device did not actually detach";
		}
	$connected = 1;
	}

# Apply a size change on a file that is a single filesystem
if ($disk->{'size'} != $olddisk->{'size'} &&
    !$disk->{'virt_part'}) {
	# Build the commands to run
	my @cmds;

        if ($disk->{'iscsi_server'} ne '') {
		# On iSCSI, so if growing the disk it needs to be expanded
		# on the the remote server now
                if ($disk->{'size'} > $olddisk->{'size'}) {
                        push(@cmds, sub {
                                &modify_iscsi_disk($server, $disk, $olddisk);
                                });
                        }
                }
	elsif ($disk->{'lv'} && $disk->{'size'} > $olddisk->{'size'}) {
		# Expand logical volume (before filesystem)
		push(@cmds, "lvextend -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}
	if ($disk->{'fs'} eq 'swap') {
		# Just re-make
		push(@cmds, &dd_command("/dev/zero", $disk->{'real'},
					$disk->{'size'}));
		push(@cmds, "mkswap ".quotemeta($disk->{'real'}));
		}
	elsif ($disk->{'fs'} =~ /^ext\d+/ || $disk->{'fs'} eq 'xfs') {
		# Expand or shrink the filesystem
		push(@cmds, &get_fsck_command($vhost, $disk->{'fs'},
					      $disk->{'real'}));
		push(@cmds, &get_resize_command($vhost, $disk->{'fs'},
			$disk->{'real'}, int($disk->{'size'}/(1024*1024))));
		}
	elsif ($disk->{'fs'} eq '' && $disk->{'size'} > $olddisk->{'size'}) {
		# Add blanks to the file, unless on LVM
		if (!$disk->{'lv'}) {
			push(@cmds, &dd_command("/dev/zero", undef,
					$disk->{'size'}-$olddisk->{'size'}).
				    " >>".quotemeta($disk->{'real'}));
			}
		}
	elsif ($disk->{'fs'} eq '' && $disk->{'size'} < $olddisk->{'size'}) {
		# Truncate the file, unless on LVM
		if (!$disk->{'lv'}) {
			push(@cmds, &dd_command($disk->{'real'},
						$disk->{'real'}.".tmp",
						$disk->{'size'}));
			push(@cmds, "mv ".quotemeta($disk->{'real'}.".tmp").
			    " ".quotemeta($disk->{'real'}));
			}
		}
	else {
		return "No idea now to resize!";
		}

        if ($disk->{'iscsi_server'} ne '') {
		# Shrink on the iSCSI server after resizing the partition
	        if ($disk->{'size'} < $olddisk->{'size'}) {
                        push(@cmds, sub {
                                &modify_iscsi_disk($server, $disk, $olddisk);
                                });
                        }
                }
	elsif ($disk->{'lv'} && $disk->{'size'} < $olddisk->{'size'}) {
		# Shrink logical volume (after resizing filesystem)
		push(@cmds, "lvreduce -f -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}

	# Run them in turn
	foreach my $cmd (@cmds) {
		my ($out, $ex, $err);
		if (ref($cmd) eq 'CODE') {
			# Call a function (used for iSCSI resizes)
			$err = &$cmd;
                        }
		else {
			# Run a command
			($out, $ex, $err) = &execute_command_on_server(
				$vhost, $cmd);
			}
		if ($cmd =~ /fsck/ && ($ex == 1 || $ex == 256)) {
			# Safe to ignore filesystem fixes
			$ex = 0;
			}
		if ($err) {
			$moderr = $err;
			last;
			}
		elsif ($ex) {
			$moderr = "$cmd failed : $out";
			last;
			}
		}
	}

# Apply a size change on a file that is a whole disk
if ($disk->{'size'} != $olddisk->{'size'} &&
    $disk->{'virt_part'}) {
	# Build the commands to run
	my @cmds;
	my $loop = &get_loop_device($vhost);

	if ($disk->{'lv'} && $disk->{'size'} > $olddisk->{'size'}) {
		# Expand logical volume (before filesystem)
		push(@cmds, "lvextend -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}
	elsif (!$disk->{'lv'} && $disk->{'size'} > $olddisk->{'size'}) {
		# Expand the disk image (before filesystem)
		push(@cmds, &dd_command("/dev/zero", undef,
					$disk->{'size'}-$olddisk->{'size'}).
			    " >>".quotemeta($disk->{'real'}));
		}

	# Work out the new filesystem size, taking into the new total disk
	# size and any preceeding partitions.
	my $newmbs;
	if ($resizefs) {
		$newmbs = int(($disk->{'size'} -
			       $part->{'start_bytes'})/1024/1024);
		$newmbs = int($newmbs / 8) * 8;
		}

	if ($disk->{'size'} < $olddisk->{'size'} && $resizefs) {
		# Shrink the filesystem
		push(@cmds, "losetup -o $part->{'offset'} $loop ".
			    quotemeta($disk->{'real'}));
		if ($disk->{'fs'} eq 'swap') {
			push(@cmds, "mkswap $loop");
			}
		else {
			push(@cmds, &get_fsck_command($vhost, $disk->{'fs'},
						      $loop));
			push(@cmds, &get_resize_command($vhost, $disk->{'fs'},
						        $loop, $newmbs));
			}
		push(@cmds, "losetup -d $loop");
		}

	# Delete the last partition, then re-create it
	my $need_part_resize = 0;
	if ($part && $server->{'type'} ne 'windows' && $part->{'type'} ne '7') {
		push(@cmds, "parted -s ".quotemeta($disk->{'real'})." rm ".
			    $part->{'part'});
		if (@$parts == 1 && $part->{'disklabel'} eq 'gpt' &&
		    $disk->{'size'} > $olddisk->{'size'}) {
			# Re-create the GPT disk label, so it is the right size
			push(@cmds, "parted -s ".quotemeta($disk->{'real'}).
				    " mklabel gpt");
			}
		push(@cmds, "parted -s ".quotemeta($disk->{'real'})." mkpart ".
			    $part->{'parted_type'}." ".
			    $part->{'parted_start'}." ".
			    int($disk->{'size'}/1000000)."M");
		$need_part_resize = 1;
		}

	if ($disk->{'size'} > $olddisk->{'size'} && $resizefs) {
		# Expand the filesystem
		push(@cmds, "losetup -o $part->{'offset'} $loop ".
			    quotemeta($disk->{'real'}));
		if ($disk->{'fs'} eq 'swap') {
			push(@cmds, "mkswap $loop");
			}
		else {
			push(@cmds, &get_fsck_command($vhost, $disk->{'fs'},
						      $loop));
			push(@cmds, &get_resize_command($vhost, $disk->{'fs'},
							$loop, $newmbs));
			}
		push(@cmds, "losetup -d $loop");
		}

	if (!$disk->{'lv'} && $disk->{'size'} < $olddisk->{'size'}) {
		# Truncate the file (after resizing filesystem)
		push(@cmds, &dd_command($disk->{'real'},
					$disk->{'real'}.".tmp",
					$disk->{'size'}));
		push(@cmds, "mv ".quotemeta($disk->{'real'}.".tmp").
		    " ".quotemeta($disk->{'real'}));
		}
	elsif ($disk->{'lv'} && $disk->{'size'} < $olddisk->{'size'}) {
		# Shrink logical volume (after resizing filesystem)
		push(@cmds, "lvreduce -f -L".int($disk->{'size'}/1024)."k ".
			    quotemeta($disk->{'real'}));
		}

	# For GPT format disks, the disk label has to be re-generated
	# AFTER shrinking so that it matches the new device size
	if ($need_part_resize && $part->{'disklabel'} eq 'gpt' &&
	    $disk->{'size'} < $olddisk->{'size'} && @$parts == 1) {
		push(@cmds, "parted -s ".quotemeta($disk->{'real'}).
			    " mklabel gpt");
		push(@cmds, "parted -s ".quotemeta($disk->{'real'}).
			    " mkpart ".
			    $part->{'parted_type'}." ".
			    $part->{'parted_start'}." ".
			    int($disk->{'size'}/1000000)."M");
		}

	# Run them in turn
	foreach my $cmd (@cmds) {
		my ($out, $ex, $err) = &execute_command_on_server(
			$vhost, $cmd);
		if ($cmd =~ /fsck/ && ($ex == 1 || $ex == 256)) {
			# Safe to ignore filesystem fixes
			$ex = 0;
			}
		if ($err) {
			$moderr = $err;
			last;
			}
		elsif ($ex) {
			$moderr = "$cmd failed : $out";
			last;
			}
		}

	# Always remove loopback
	&execute_command_on_server($vhost, "losetup -d /dev/loop7");
	}

if ($disk->{'size'} != $olddisk->{'size'}) {
	# Update total disk size
	if (!$moderr &&
	    $server->{'xen_disktotal'} ne '' &&
	    (!$disk->{'device'} || $disk->{'lv'})) {
		$server->{'xen_disktotal'} -= $olddisk->{'size'};
		$server->{'xen_disktotal'} += $disk->{'size'};
		}

	# Invalidate LV cache for host, due to size change
	delete($main::list_logical_cache{$vhost->{'id'}});
	}

# If this was the primary disk, update disk details in the VM
if ($olddisk->{'real'} eq $server->{'xen_filesystem'}) {
	&set_server_disk_fields($server, $disk, 'filesystem');
	}
if ($olddisk->{'real'} eq $server->{'xen_swapfile'}) {
	&set_server_disk_fields($server, $disk, 'swapfile');
	}

if (($disk->{'virt'} ne $olddisk->{'virt'} ||
     $disk->{'real'} ne $olddisk->{'real'} ||
     $disk->{'media'} ne $olddisk->{'media'}) && !$moderr) {
	# Change the virtual device, both in the Xen config file and fstab

	# Fix the Xen config
	my $disks = &xen_find_value("disk", $conf);
	my $suffix = $olddisk->{'suffix'};
	my $newsuffix = $suffix;
	if ($disk->{'real'} ne $olddisk->{'real'}) {
		# May have moved from LV to file, or vice-versa
		$newsuffix = &compute_xen_disk_prefix($disk, $vhost);
		}
	my $oldshort = $olddisk->{'virt'};
	$oldshort =~ s/^\/dev\///;
	$oldshort .= ":".$olddisk->{'media'}
		if ($olddisk->{'media'} ne 'disk');
	foreach my $d (@$disks) {
		if ($d =~ /^\Q$suffix\E:\Q$olddisk->{'real'}\E,\Q$oldshort\E,(.*)$/) {
			my $mode = $1;
			my $short = $disk->{'virt'};
			$short =~ s/^\/dev\///;
			$short .= ":".$disk->{'media'}
				if ($disk->{'media'} ne 'disk');
			$d = "$newsuffix:$disk->{'real'},$short,$mode";
			}
		}
	&xen_save_directive($conf, "disk", $disks);
	my $err = &put_xen_config($server, $conf);
	if ($err) {
		$moderr = "Failed to save Xen config : $err";
		}

	# Refresh the Xen config file
	my $xenver = &get_xen_software_version($vhost);
	if ($server->{'status'} eq 'down' && $xenver >= 4) {
		if ($vhost->{'xen_api_type'} eq 'xl') {
			&run_xen_api_command($vhost,
				"xm config-update $server->{'xen_conf'}");
			}
		else {
			&run_xen_api_command($vhost,
				"xm delete $server->{'xen'}");
			&run_xen_api_command($vhost,
				"xm new $server->{'xen_conf'}");
			}
		}

	# Fix virt_onsystem parameter
	if ($disk->{'virt_onsystem'} =~ /^(\/dev\/(s|h|xv)d)[a-z].*$/) {
		my $prefix = $1;
		if ($disk->{'virt'} =~ /([a-z]\d*)$/) {
			$disk->{'virt_onsystem'} = $prefix.$1;
			}
		}

	# Update fstab, if in there
	if ($server->{'status'} ne 'alive' && !$moderr) {
		my $fstab = &get_system_fstab($server);
		if (!ref($fstab)) {
			$moderr = "Failed to read fstab file : $fstab";
			}
		else {
			my ($f) = grep { &same_xen_virt($_->[0],
					     $olddisk->{'virt'}, 0) } @$fstab;
			if ($f) {
				$f->[0] = $disk->{'virt_onsystem'} ||
					  $disk->{'virt'};
				my $err = &put_system_fstab($server, $fstab);
				$moderr = "Failed to update fstab file : $err"
					if ($err);
				}
			}
		}
	}

# If it was connected before, re-connect it now
if ($connected) {
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
	    "xm block-attach $xenid $disk->{'suffix'}:$disk->{'real'} ".
	    "$disk->{'virt'} w");
	if ($err || $ex) {
		return "Failed to attach $disk->{'virt'} : ".($err || $out);
		}
	}

# For xen 4, if the real device changed need to re-import the config
my $xenver = &get_xen_software_version($vhost);
if (!$connected && $server->{'status'} eq 'down' &&
    $xenver >= 4 && $vhost->{'xen_api_type'} ne 'xl' &&
    $disk->{'real'} ne $olddisk->{'real'}) {
	&run_xen_api_command($vhost, "xm delete $server->{'xen'}");
	&run_xen_api_command($vhost, "xm new $server->{'xen_conf'}");
	}

# If it was mounted before, mount it now
if ($mounted && $disk->{'mount'}) {
	my $dev = $disk->{'virt_mount'} || $disk->{'virt'};
	my ($out, $ex, $err) = &execute_command_on_server($server,
		$disk->{'fs'} eq 'swap' ?
			"swapon ".$dev :
			"mount ".$dev." ".$disk->{'mount'});
	if ($ex && $out =~ /already\s+mounted/) {
		# Xen instance auto-mounted when attached
		$ex = 0;
		}
	if ($err || $ex) {
		return "Failed to mount $disk->{'virt'} : ".($err || $out);
		}
	}

# Disconnect iSCSI now if requested
if ($disk->{'iscsi_ip'} && !$disk->{'iscsi_connect'} &&
    $olddisk->{'iscsi_connect'}) {
        my $err = &disconnect_iscsi_disk($server, $disk);
        return $err if ($err);
        }

return $moderr;
}

# type_xen_list_disk_filesystems(&server)
# Returns supported filesystem types. Assumes Linux.
sub type_xen_list_disk_filesystems
{
return ('ext3', 'ext4', 'ext2', 'reiserfs', 'xfs', 'jfs', 'swap');
}

# type_xen_list_storages(&vhost, [always-show-hosts])
# Returns a list of possible storage locations on some host system. Each is a
# hash ref with a id and desc fields. Example storages include local files and
# LVM volume groups.
sub type_xen_list_storages
{
my ($vhost, $always) = @_;
my @rv;

# Regular files
push(@rv, { 'id' => 'files',
	    'type' => 'files',
	    'desc' => $text{'storage_files'},
	    'adddesc' => $text{'storage_addfiles'} });

# Add iSCSI servers
my @iscsis = &list_iscsi_target_servers();
my $showi = $always || @iscsis > 1;
foreach my $iscsi (@iscsis) {
	push(@rv, {
	    'id' => 'iscsi_'.$iscsi->{'host'},
	    'type' => 'iscsi',
	    'iscsi' => $iscsi->{'id'},
	    'desc' => $showi ? &text('storage_iscsi', $iscsi->{'host'})
			     : $text{'storage_iscsi1'},
	    'adddesc' => $showi ? &text('storage_addiscsi', $iscsi->{'host'})
				: $text{'storage_addiscsi1'} });
	}

if ($vhost->{'status'} eq 'virt' || $vhost->{'status'} eq 'novirt') {
	# Check for and add LVM volume groups
	my $info = &get_server_collected($vhost);
	my @vgs = grep { $_->{'vg'} ne 'gpt_vg' } @{$info->{'vg_list'}};
	my $showv = $always || @vgs > 1;
	foreach my $vg (@vgs) {
		push(@rv, {
		    'id' => 'lvm_'.$vg->{'vg'},
		    'type' => 'lvm',
		    'vg' => $vg->{'vg'},
		    'desc' => $showv ? &text('storage_lvm', $vg->{'vg'})
				     : $text{'storage_lvm1'},
		    'adddesc' => $showv ? &text('storage_addlvm', $vg->{'vg'})
					: $text{'storage_addlvm1'} });
		}

	# Also add thin pool LVs
	my @lvs = @{$info->{'lv_thin_list'}};
	my $showv = $always || @lvs > 1;
	foreach my $lv (@lvs) {
		push(@rv, {
		    'id' => 'thin_'.$lv->{'vg'}.'_'.$lv->{'lv'},
		    'type' => 'thin',
		    'vg' => $lv->{'vg'},
		    'lv' => $lv->{'lv'},
		    'desc' => $showv ? &text('storage_thin', $lv->{'lv'})
				     : $text{'storage_thin1'},
		    'adddesc' => $showv ? &text('storage_addthin',$lv->{'lv'})
					: $text{'storage_addthin'} });
		}
	}

# Work out the default storage
my $def;
if ($vhost->{'xen_vg'}) {
	($def) = grep { $_->{'vg'} eq $vhost->{'xen_vg'} } @rv;
	}
elsif ($vhost->{'xen_iscsi'} ne '') {
	($def) = grep { $_->{'iscsi'} eq $vhost->{'xen_iscsi'} } @rv;
	}
else {
	$def = $rv[0];
	}
$def || return "No default storage found for $vhost->{'host'}";
$def->{'default'} = 1;

return \@rv;
}

# type_xen_can_fsck(&server, &disk)
# Returns 1 if some disk can have it's filesystem checked. Only true when
# the system is down or the disk is not mounted.
sub type_xen_can_fsck
{
my ($server, $disk) = @_;
return ($server->{'status'} eq 'down' || !$disk->{'mounted'}) &&
       $disk->{'fs'} && $disk->{'fs'} ne 'swap' &&
       $disk->{'media'} ne 'cdrom';
}

# type_xen_fsck_disk(&server, &disk, [only-if-unclean])
# Run fsck on a disk to validate and repair the filesystem. Returns 1 if 
# all was OK to start with, 2 if fixes were made and 0 on failure, plus the
# output from fsck
sub type_xen_fsck_disk
{
my ($server, $disk, $noforce) = @_;
my $flag = $noforce ? "" : "-f";
my $vhost = &get_parent_server($server);
if ($server->{'xen_wholedisk'}) {
	# Need to loopback mount a partition and fsck it
	my ($ok, $loop) = &loopback_mount_partition($vhost, $disk->{'real'});
	$ok || return $loop;
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"fsck -y $flag $loop");
	&loopback_unmount_partition($vhost, $loop);
	if ($ex && $out =~ /fsck\S+:\s+not\s+found/) {
		# Filesystem not supported, so assume OK
		return (1, $out);
		}
	elsif ($ex > 2 || $err) {
		return (0, $out);
		}
	elsif ($ex) {
		return (2, $out);
		}
	else {
		return (1, $out);
		}
	}
else {
	# Fsck the disk image directly
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"fsck -y $flag -t $disk->{'fs'} ".quotemeta($disk->{'real'}));
	if ($ex && $out =~ /fsck\S+:\s+not\s+found/) {
		# Filesystem not supported, so assume OK
		return (1, $out);
		}
	elsif ($ex > 2 || $err) {
		return (0, $out);
		}
	elsif ($ex) {
		return (2, $out);
		}
	else {
		return (1, $out);
		}
	}
}

# type_xen_list_disk_medias(&server)
# Returns a list of possible media types for disks on some system. Each is a
# hash ref with id and desc field
sub type_xen_list_disk_medias
{
my ($server) = @_;
return [ { 'id' => 'disk', 'desc' => 'Hard disk' },
	 { 'id' => 'cdrom', 'desc' => 'CD or DVD drive' } ];
}

# type_xen_compute_disksize(&server)
# Sets a server's xen_disktotal field to the total size of virtual disks,
# and returns it.
sub type_xen_compute_disksize
{
my ($server) = @_;
if ($server->{'status'} ne 'nohost') {
	my $disks = &type_xen_list_disks($server, 0);
	if (ref($disks)) {
		$server->{'xen_disktotal'} = 0;
		foreach my $d (@$disks) {
			$server->{'xen_disktotal'} += $d->{'size'}
				if (!$d->{'device'} || $d->{'lv'} ||
				    $d->{'iscsi_server'} ne '');
			}
		}
	}
$server->{'xen_disktotal'} += &compute_snapshots_size($server);
return $server->{'xen_disktotal'};
}

# type_xen_get_disk_size(&server)
# Returns the cached amount of disk space allocated to this Xen instance
sub type_xen_get_disk_size
{
my ($server) = @_;
if (!$server->{'xen_disktotal'}) {
	&type_xen_compute_disksize($server);
	}
return $server->{'xen_disktotal'};
}

# type_xen_get_disk_space(&server)
# Returns the amount of disk space used on the host by this Xen instance, which
# is the same as the disk allocated
sub type_xen_get_disk_space
{
my ($server) = @_;
return &type_xen_get_disk_size($server);
}

# type_xen_get_memory_size(&server)
# Returns the memory allocated to this Xen instance, from a cache
sub type_xen_get_memory_size
{
my ($server) = @_;
return $server->{'xen_memory'}*1024*1024;
}

# type_xen_get_cpu_size(&server)
# Returns the cached percentage of CPU allocated to this Xen instance
sub type_xen_get_cpu_size
{
my ($server) = @_;
return $server->{'xen_credit_cpu'} || 0;
}

# type_xen_get_network_rate_size(&server)
# Returns the bits/second of network speed for this Xen instance, from a cache
sub type_xen_get_network_rate_size
{
my ($server) = @_;
return $server->{'xen_network_rate'};
}

# type_xen_need_disk_reboot(&server, &disk, &olddisk)
# A reboot is only needed for HVM instances, now that we can use xm block-attach
sub type_xen_need_disk_reboot
{
my ($server, $disk, $olddisk) = @_;
if ($disk && $disk->{'reboot'}) {
	return 1;	# Failed to attach at creation
	}
elsif ($olddisk && $olddisk->{'reboot'}) {
	return 1;	# Failed to detach at deletion
	}
elsif ($server->{'xen_hvm'} && $server->{'status'} ne 'down') {
	return 1;	# Cannot attach to HVM systems
	}
else {
	return 0;
	}
}

# type_xen_can_get_console_output(&server)
sub type_xen_can_get_console_output
{
my ($server) = @_;
return !$server->{'xen_hvm'} && $server->{'xen_console'} ? 1 : 0;
}

# type_xen_get_console_output(&server)
# Returns a status code and text displayed during the Xen boot (if status is 1),
# or an error message (if 0)
sub type_xen_get_console_output
{
my ($server) = @_;
if ($server->{'xen_hvm'}) {
	return (0, "HVM Xen instances have no console messages");
	}
if (!$server->{'xen_console'}) {
	return (0, "No console messages were captured at boot time");
	}
my $vhost = &type_xen_parent($server);
my $out = &get_file_data_from_server($vhost, $server->{'xen_console'});
$out =~ s/SERVER-MANAGER-COMPLETE\r?\n//;
return $out ? (1, $out) : (0, "No console messages were found");
}

# type_xen_supports_console_login(&server)
# Returns 1 if connecting to the console on some system is supported, 0 if not,
# or -1 if possible but not currently setup (which can be done with
# type_xen_create_console)
sub type_xen_supports_console_login
{
my ($server) = @_;
if ($server->{'xen_hvm'}) {
	# Text console not possible for console
	return 0;
	}
elsif ($server->{'status'} eq 'nohost' || $server->{'status'} eq 'down' ||
       $server->{'status'} eq 'paused') {
	# Not even running, so cannot connect
	return 0;
	}
else {
	# Check if special user exists
	return &has_login_user($server) ? 1 : -1;
	}
}

# type_xen_console_command(&server)
# Returns the command to be run on the Xen host system to attach to an
# instance's console.
sub type_xen_console_command
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $api = $vhost->{'xen_api_type'} || 'xm';
my $path = &has_command_on_server($vhost, $api);
$path ||= "xm";
return "$path console ".quotemeta($server->{'xen'});
}

# type_xen_can_pause(&server)
# Returns 1 if a system can be paused, 2 if it can be resumed, 0 if neither
sub type_xen_can_pause
{
my ($server) = @_;
if ($server->{'status'} eq 'nohost' || $server->{'status'} eq 'down') {
	# Down, so neither can be done
	return 0;
	}
elsif ($server->{'status'} eq 'paused') {
	# Currently paused, can resume
	return 2;
	}
else {
	# Now running, can cause
	return 1;
	}
}

# type_xen_pause_system(&server)
# Pauses a Xen instance. Returns undef on success or an error message
# on failure.
sub type_xen_pause_system
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm pause $server->{'xen'}");
return $err ? $err : $ex ? $out : undef;
}

# type_xen_unpause_system(&server)
# Resumes a Xen instance. Returns undef on success or an error message
# on failure.
sub type_xen_unpause_system
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm unpause $server->{'xen'}");
return $err ? $err : $ex ? $out : undef;
}

# type_xen_can_move(&server, [&dest])
# Check if moving a Xen instance is allowed. Only possible if target is up.
# Returns 2 indicating than manual moves are supported too, or 3 indicating
# that only manual moves are possible.
sub type_xen_can_move
{
my ($server, $dest) = @_;
return 0 if ($module_info{'onehost'} eq 'xen');
if (!$dest ||
    $dest->{'status'} eq 'virt' ||
    $dest->{'status'} eq 'nowebmin' ||
    $dest->{'status'} eq 'novirt') {
	# Dest is OK
	if ($server->{'status'} eq 'nohost') {
		# Parent is down, only manual moves possible
		return 3;
		}
	else {
		return 2;
		}
	}
return 0;
}

# type_xen_move_system(&server, &dest, [shutdown], [allocate-ip])
# Actually move a Xen instance to a new host, printing messages as it goes.
sub type_xen_move_system
{
my ($server, $dest, $shutdown, $allocate, $directory) = @_;
my %oldserver = %$server;
my $vhost = &get_parent_server($server);
my (@delfiles, @dellvs);
delete($list_xen_instances_cache{$vhost->{'id'}});
delete($list_xen_instances_cache{$dest->{'id'}});

# If virtual system disks are on iSCSI, make sure the dest vhost is using
# the same iSCSI server
my $disks = &type_xen_list_disks($server, 0);
my $non_iscsi = 0;
foreach my $d (@$disks) {
        if ($d->{'iscsi_ip'}) {
                $d->{'iscsi_server'} eq '' && return $text{'smove_eicsiserver'};
                my $old_iscsiserver = &get_managed_server($d->{'iscsi_server'});
                if ($server->{'xen_force_iscsi'} ne '') {
                        my $new_iscsiserver = &get_managed_server(
                                                $d->{'iscsi_server'});
                        my $err = &check_iscsi_client($dest);
                        $err && return &text('smove_eiscsidest', $err);
                        }
                else {
                        $dest->{'xen_iscsi'} eq '' &&
                                return $text{'smove_eicsidest'};
                        my $new_iscsiserver = &get_managed_server(
                                                $dest->{'xen_iscsi'});
                        $old_iscsiserver->{'id'} eq $new_iscsiserver->{'id'} ||
                                return &text('smove_eiscsidiff',
                                             $old_iscsiserver->{'host'},
                                             $new_iscsiserver->{'host'});
                        }
                }
	else {
		$non_iscsi++;
		}
        }
my $all_iscsi = @$disks && !$non_iscsi;

# Work out old and new dirs. If a custom dir was given for this Xen instance,
# assume that the same should be used on the dest.
my $old_xen_dir = $server->{'directory'} || $vhost->{'xen_dir'};
my $new_xen_dir = (defined($directory) ? $directory : $server->{'directory'}) ||
		  $dest->{'xen_dir'};

# Get Xen versions
my $vhostver = &get_xen_software_version($vhost);
my $destver = &get_xen_software_version($dest);

# Check if a shutdown is needed
my $shutwhy;
if (!$shutdown) {
	if ($server->{'xen_filesystem_vg'}) {
		# VG is being changed
		if ($server->{'xen_filesystem_vg'} ne $dest->{'xen_vg'}) {
			$shutdown = 1;
			$shutwhy = $text{'smove_diffvg'};
			}
		}
	elsif ($old_xen_dir ne $new_xen_dir) {
		# Base directory is being changed
		$shutdown = 1;
		$shutwhy = $text{'smove_diffdir'};
		}
	elsif (int($vhostver) != int($destver)) {
		# Different major Xen version
		$shutdown = 1;
		$shutwhy = &text('smove_diffver', $vhostver, $destver);
		}
	}

# Check if live migration is possible
my $live = 1;
my $disks = &type_xen_list_disks($server, 1);
if (!ref($disks)) {
	$disks = &type_xen_list_disks($server, 0);
	}
my $livehost = $dest->{'xen_livehost'} || &get_server_ip($dest);
my $liveport = $dest->{'xen_liveport'} || $xen_migration_port;
&$first_print($text{'smove_livecheck'});
if ($live && $server->{'status'} eq 'down') {
	&$second_print($text{'smove_elivedown'});
	$live = 0;
	}
elsif ($live && $shutdown) {
	&$second_print($text{'smove_eliveshut'});
	$live = 0;
	}
if ($live) {
	if (&has_command_on_server($vhost, "nc")) {
		# Check for live migration port using nc from the host
		my ($out, $ex, $err) = &execute_command_on_server($vhost,
			"LANG=C nc -w 4 $livehost $liveport </dev/null", $short_timeout);
		if ($out =~ /Connection\s+(refused|failed)/i || $ex == 1) {
			&$second_print(&text('smove_eliveport',
					     $livehost, $liveport));
			$live = 0;
			}
		}
	else {
		# Check from the master
		my $err;
		eval {
			&set_alarm(5);
			local $SIG{'ALRM'} = sub { die "timeout" };
			&open_socket($livehost, $liveport, TESTSOCK, \$err);
			&set_alarm(0);
			close(TESTSOCK);
			};
		if ($err || $@) {
			&$second_print(&text('smove_eliveport',
					     $livehost, $liveport));
			$live = 0;
			}
		}
	}
if ($live) {
	# Check xend version
	if ($vhostver && $destver && $vhostver ne $destver) {
		&$second_print(&text('smove_elivever', $vhostver, $destver));
		$live = 0;
		}
	}
if ($live && !$all_iscsi) {
	# Check if all disks are shared (except for iSCSI)
	foreach my $d (@$disks) {
		my @sst = &recursive_stat_file_on_server($vhost, $d->{'real'});
		my @dst = &recursive_stat_file_on_server($dest, $d->{'real'});
		if (!@dst || $dst[7] != $sst[7] &&
		    ($old_xen_dir eq $new_xen_dir || $d->{'lv'})) {
			&$second_print(&text('smove_elivedisk', $d->{'real'}));
			$live = 0;
			last;
			}
		}
	}
if ($live) {
	&$second_print($text{'smove_liveok'});
	}

# Check if need to allocate new IP
my $ip = &get_server_ip($server);
my @xen_iface = split(/\s+/, $dest->{'xen_iface'});
my ($newip, $newcidr, $newgw);
my $change_ip_later;
if ($allocate && $dest->{'xen_start'} &&
    !&range_check_ip_address($ip, $dest->{'xen_start'}, $dest->{'xen_end'},
				  $dest->{'xen_cidr'}, $dest->{'xen_gws'},
				  $dest->{'xen_ifaces'}, $xen_iface[0])) {
	# Allocate the primary IP
	&$first_print($text{'smove_allocating'});
	($newip, $newcidr, $newgw) = &allocate_ip_address(
		$dest->{'xen_start'}, $dest->{'xen_end'},
	        $dest->{'xen_cidr'}, $dest->{'xen_gws'},
		$dest->{'xen_ifaces'}, $xen_iface[0]);
	$newip || return &text('create_exenalloc', $dest->{'host'},
			    &describe_ip_range($dest->{'xen_start'},
					       $dest->{'xen_end'}));
	&$second_print(&text('smove_allocated', $newip));

	if ($live || ($server->{'status'} ne 'down' && !$shutdown)) {
		# Apply the IP change now, before the move
		my $err = &xen_move_change_ip($server, $vhost, $dest,
					      $ip, $newip, $newcidr, $newgw);
		return $err if ($err);
		}
	else {
		$change_ip_later = 1;
		}
	}

# If live migrating and all disks are on iSCSI, connect to the new host
# system now
if ($live && $all_iscsi) {
	foreach my $d (@$disks) {
		my $iscsiserver = &get_managed_server($d->{'iscsi_server'});
		&$first_print(&text('smove_iscsiconn',
				    $iscsiserver->{'host'},
				    $d->{'iscsi_target'}));
		my $newd = { %$d };
		my $newserver = { %$server };
		$newserver->{'xen_on'} = $dest->{'id'};
		$err = &connect_iscsi_disk($newserver, $newd);
		if ($err) {
			return &text('smove_eiscsiconn', $err);
			}
		&$second_print($text{'create_done'});
		}
	}

my ($dumpsrc, $dumpdest, $was_up);
if ($server->{'status'} ne 'down' && $shutdown) {
	# Shut down before moving
	&$first_print($shutwhy ? &text('smove_shutting2', $shutwhy)
			       : $text{'smove_shutting'});
	$err = &shutdown_server($server);
	if ($err) {
		return &text('smove_shutfailed', $err);
		}
	&check_server_status($server, 0, 1);
	$was_up = 1;
	&$second_print($text{'create_done'});
	}
elsif ($live) {
	# Live migrate system
	&$first_print($text{'smove_living'});
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm migrate --live $server->{'xen'} $livehost".
		($liveport == $xen_migration_port ? "" : ":".$liveport));
	if ($ex || $err) {
		return &text('smove_elived', $err || $out);
		}
	&$second_print($text{'create_done'});
	}
elsif ($server->{'status'} ne 'down') {
	# Save a state file, if running
	&$first_print($text{'smove_dumping'});
	$dumpsrc = &tempname_on_server($vhost, $server->{'host'}.".state");
	if (!$dumpsrc) {
		return $text{'smove_etemp'};
		}
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm save $server->{'xen'} $dumpsrc");
	if ($ex || $err) {
		return &text('smove_edump', $err || $out);
		}
	&$second_print($text{'create_done'});

	# Transfer state file
        &$first_print($text{'smove_dumpsend'});
        $dumpdest = &tempname_on_server($dest, $server->{'host'}.".state");
	if (!$dumpdest) {
		return $text{'smove_etemp'};
		}
	local $dest->{'ip'} = $livehost;
        my $err = &transfer_file_between_servers(
                        $vhost, $dest, $dumpsrc, $dumpdest, 0, 1);
        &delete_file_on_server($vhost, $dumpsrc);
	return $err if ($err);
        &$second_print($text{'create_done'});
	}

# Copy across all disk images, directly if possible. Don't do this if it is a
# live migration, in which case they will already be in place.
my $count = 0;
my $iscsi_count = 0;
my $sz = 0;
foreach my $d (@$disks) {
	$d->{'newreal'} = $d->{'real'};
	}
if (!$live) {
	&$first_print($text{'smove_disks'});
	&$indent_print();
	foreach my $d (@$disks) {
		my $swapdisk = 0;
		my $msg;
		if ($d->{'lv'} && $d->{'swap'} &&
		    $server->{'status'} eq 'down') {
			$msg = &text('smove_newswap', $d->{'real'});
			$swapdisk = 1;
			}
		else {
			$msg = &text('smove_onedisk', $d->{'real'});
			}
		if ($d->{'device'} && !$d->{'lv'} &&
		    $d->{'iscsi_server'} eq '') {
			# A real device, which shouldn't be touched
			&$first_print($msg);
			&$second_print(&text('smove_devdisk'));
			next;
			}

		# Check if disk file exists on the target system already, and
		# that the Xen directory isn't being changed
		my @sst = &recursive_stat_file_on_server($vhost, $d->{'real'});
		my @dst = &recursive_stat_file_on_server($dest, $d->{'real'});
		if (@dst && $dst[7] == $sst[7] &&
		    ($old_xen_dir eq $new_xen_dir || $d->{'lv'})) {
			# Already there! Perhaps from shared LVM
			&$first_print($msg);
			&$second_print($text{'smove_alrdisk'});
			next;
			}

		# If the disk is on iSCSI, the move can be done by simply
		# disconnecting and re-connecting on the new host
		if ($d->{'iscsi_server'} ne '') {
			&remote_finished();
			my $iscsiserver = &get_managed_server(
						$d->{'iscsi_server'});
			&$first_print(&text('smove_iscsi',
					    $iscsiserver->{'host'},
					    $d->{'iscsi_target'}));
			my $err = &disconnect_iscsi_disk($server, $d);
			if ($err) {
				return &text('smove_eiscsidis', $err);
				}
			my $newserver = { %$server };
			$newserver->{'xen_on'} = $dest->{'id'};
			my $newd = { %$d };
			$err = &connect_iscsi_disk($newserver, $newd);
			if ($err) {
				return &text('smove_eiscsiconn', $err);
				}
			$d->{'newreal'} = $newd->{'real'};
			&$second_print($text{'create_done'});
			$iscsi_count++;
			next;
			}

		&start_background_dot_printer($msg);
		my $newreal;
		if ($d->{'lv'}) {
			# LVM logical volume .. transfer to LV of the same name
			if (!$dest->{'xen_vg'}) {
				&stop_background_dot_printer();
				return $text{'smove_errvg'};
				}
			$newreal = "/dev/".$dest->{'xen_vg'}."/".$d->{'lv'};

			# Create LV on target first
			$last_remote_error = undef;
			my $lv = { 'name' => $d->{'lv'},
				      'size' => int($d->{'size'}/1024),
				      'perm' => 'rw',
				      'alloc' => 'n',
				      'vg' => $dest->{'xen_vg'} };
			&remote_finished();
			$last_remote_error = undef;
			&remote_foreign_require($dest, "lvm", "lvm-lib.pl");
			my $err = &remote_foreign_call($dest, "lvm",
						  "create_logical_volume", $lv);
			$err ||= $last_remote_error;
			if ($err) {
				&stop_background_dot_printer();
				return &text('smove_errlv', $err);
				}

			# Transfer over the data, using dd. The size for
			# checksum purposes is rounded down to 1MB to account
			# for differences in LVM block sizes
			my $err;
			if ($swapdisk) {
				# Just re-create swap file
				my ($out, $ex, $swerr) =
					&execute_command_on_server($dest,
						"mkswap ".quotemeta($newreal));
				$err = $swerr ? $swerr : $ex ? $out : undef;
				}
			else {
				# Actually copy disk
				local $dest->{'ip'} = $livehost;
				$err = &transfer_file_between_servers(
				    $vhost, $dest,
				    $d->{'real'}, $newreal, 1, 1,
				    int($d->{'size'}/(1024*1024))*1024*1024);
				}
			if ($err) {
				&stop_background_dot_printer();
				return &text('smove_errdisk', $err);
				}
			push(@dellvs, $d->{'real'});
			}
		else {
			# Work out a new filename, and copy over the file
			$newreal = $d->{'real'};
			$newreal =~ s/^.*[\\\/]//;
			$newreal = "$new_xen_dir/$newreal";
			local $dest->{'ip'} = $livehost;
			my $err = &transfer_file_between_servers($vhost, $dest,
				$d->{'real'}, $newreal, 0, 1);
			if ($err) {
				&stop_background_dot_printer();
				return &text('smove_errdisk', $err);
				}
			push(@delfiles, $d->{'real'});
			}

		&stop_background_dot_printer();
		$d->{'newreal'} = $newreal;
		&$second_print(&text('smove_donedisk',
				     &nice_size($d->{'size'})));
		$count++;
		$sz += $d->{'size'};
		}
	&$outdent_print();
	if ($count) {
		&$second_print(&text('smove_disksdone', $count,
				     &nice_size($sz)));
		}
	elsif ($iscsi_count) {
		&$second_print(&text('smove_disksdone2', $iscsi_count));
		}
	else {
		&$second_print(&text('smove_disksnone'));
		}
	}
my @deldiskfiles = ( @dellvs, @delfiles );

# Read the source Xen config file
&$first_print(&text('smove_xenconf', $server->{'xen_conf'}));
my $conf = &get_xen_config($server, 0);
if (!ref($conf)) {
	return &text('smove_exenconf', $conf);
	}
my @sst = &stat_file_on_server($vhost, $server->{'xen_conf'});
my @dst = &stat_file_on_server($dest, $server->{'xen_conf'});

if ($server->{'xen_hvm'}) {
	# Fix the HVM paths in the config file
	my $hvmloader = &find_xen_hvmloader($dest);
	if ($hvmloader) {
		&xen_save_directive($conf, "kernel", $hvmloader);
		}
	my $dm = &find_xen_qemu_dm($dest);
	if ($dm) {
		&xen_save_directive($conf, "device_model", $dm);
		}
	}
elsif ($server->{'xen_pygrub'}) {
	# Fix up pygrub location if needed
	# XXX missing grubfile
	my @st = &stat_file_on_server($dest, $server->{'xen_bootloader'});
	my $grubfile = $server->{'xen_grubfile'} ||
		       "/boot/grub/menu.lst";
	if (!@st || !$st[7]) {
		# Need to fix up bootloader
		my $img = &get_server_image($server->{'image'});
		my $bootloader = &find_xen_grub_bootloader($dest, $img);
		if ($bootloader) {
			$server->{'xen_bootloader'} = $bootloader;
			&xen_save_directive($cfg, "bootloader", $bootloader);
			&xen_save_directive($cfg, "extra",
				$bootloader =~ /pv-grub/ ? "(hd0)".$grubfile
							 : undef);
			}
		}
	}
else {
	# Fix the kernel and path in the config file, if needed
	my $okernel = &xen_find_value("kernel", $conf);
	if ($okernel) {
		my @st = &stat_file_on_server($dest, $okernel);
		if (!@st) {
			my ($kok, $kfile, $ifile) =
				&setup_xen_host_kernel($dest);
			$kok || return &text('smove_ekernel', $kfile);
			&xen_save_directive($conf, "kernel", $kfile);
			&xen_save_directive($conf, "ramdisk", $ifile);
			}
		}
	}

# Fix disk paths in the config file, unless this was a live migration in
# which they don't actually change
if (!$live) {
	my @disklist;
	foreach my $d (@$disks) {
		if ($d->{'suffix'} eq 'file' || $d->{'suffix'} eq 'tap:aio') {
			# Use file-based prefix for new host
			$d->{'suffix'} = $dest->{'xen_filetype'} || 'file';
			}
		push(@disklist, $d->{'suffix'}.":".$d->{'newreal'}.",".
				$d->{'virtdev'}.",".$d->{'mode'});
		if ($server->{'xen_filesystem'} eq $d->{'real'}) {
			$server->{'xen_filesystem'} = $d->{'newreal'};
			}
		if ($server->{'xen_swap'} eq $d->{'real'}) {
			$server->{'xen_swap'} = $d->{'newreal'};
			}
		}
	&xen_save_directive($conf, "disk", \@disklist);
	}

# Fix Xen VNC IP if set
my $vnclisten = &xen_find_value("vnclisten", $conf);
if ($vnclisten && $vnclisten ne "0.0.0.0") {
	my $ip = &get_server_ip($vhost);
	&xen_save_directive($conf, "vnclisten",
		$ip || &to_ipaddress($vhost->{'host'}));
	}

# Add config file to list to delete, if not shared
if (!(@dst && $dst[7] == $sst[7] &&
      ($old_xen_dir eq $new_xen_dir || $d->{'lv'}))) {
	push(@delfiles, $server->{'xen_conf'});
	}

# Fix location of the config file
if ($server->{'xen_conf'} =~ /^\Q$old_xen_dir\/\E(.*)$/) {
	$server->{'xen_conf'} = "$new_xen_dir/$1";
	}
$server->{'xen_on'} = $dest->{'id'};
$server->{'directory'} = $directory if (defined($directory));
delete($server->{'new_directory'}) if (defined($directory));
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# For Xen 4.2, need to register the config file on the new host. This may
# legitimately fail if it was a live migration. Note that this is not needed
# when the VM state can be restored with xm restore, as this includes all
# configuration.
if ($destver >= 4.2 && $vhost->{'xen_api_type'} ne 'xl' && !$dumpsrc) {
	&run_xen_api_command($dest,
		"xm new $server->{'xen_conf'}");
	}

&$second_print(&text('smove_xenconfdone', $server->{'xen_conf'}));

# Remove old Xen instance if somehow still running
if ($server->{'status'} ne 'down' && !$live) {
	&run_xen_api_command($vhost, "xm destroy $server->{'xen'}");
	}

# Fix up VNC port in config file, if any
if ($server->{'xen_vnc_display'} || $server->{'xen_vnc_unused'}) {
	&$first_print($text{'create_xenvnc'});
	my $err = &setup_xen_vnc_console($server);
	&$second_print($err ? &text('create_xenvncfailed', $err)
			    : &text('create_xenvncdone'));
	}

# Fix LVM volume group
if ($server->{'xen_filesystem_vg'}) {
	$server->{'xen_filesystem_vg'} = $dest->{'xen_vg'};
	}
if ($server->{'xen_swapfile_vg'}) {
	$server->{'xen_swapfile_vg'} = $dest->{'xen_vg'};
	}

# Change IP now if we didn't do it before the move
if ($change_ip_later) {
	my $err = &xen_move_change_ip($server, $vhost, $dest,
				      $ip, $newip, $newcidr, $newgw);
	return $err if ($err);
	}

# Only delete console file if not on shared storage
push(@delfiles, $server->{'xen_console'})
	if ($server->{'xen_console'} &&
	    !&shared_file_on_servers($vhost, $dest, $server->{'xen_console'}));

# Delete the original disk images and config file
if (@delfiles) {
        &$first_print(&text(@deldiskfiles ? 'smove_delfiles'
                                          : 'smove_delfiles2',
                            scalar(@delfiles)+scalar(@dellvs)));
	foreach my $f (@delfiles) {
		my $err = &delete_file_on_server($vhost, $f);
		if ($err) {
			&$second_print(&text('smove_delerr', $f, $err));
			}
		}
	if (@dellvs) {
		&remote_finished();
		&remote_foreign_require($vhost, "lvm", "lvm-lib.pl");
		}
	foreach my $f (@dellvs) {
		&delete_all_disk_image_partitions($vhost, $f);
		my ($vg, $lv) = &split_lvm_device($f);
		&delete_mapper_entries($vhost, $vg, $lv);
		&remote_foreign_call($vhost, "lvm", "delete_logical_volume",
				     { 'device' => $f });
		my $srcf = &find_snapshot_source_logical_volume(
				\%oldserver, $vg, $lv);
		if ($srcf) {
			# Delete snapshot source if no longer in use
			&delete_all_disk_image_partitions($vhost, $srcf);
			my ($srcvg, $srclv) = &split_lvm_device($srcf);
			&delete_mapper_entries($vhost, $srcvg, $srclv);
			&remote_foreign_call($vhost, "lvm",
					     "delete_logical_volume",
					     { 'device' => $srcf });
			}
		}
	&$second_print($text{'create_done'});
	}

# For Xen 4.2, delete from Xen management on old host
if ($vhostver >= 4.2) {
	&run_xen_api_command($vhost, "xm delete $server->{'xen'}");
	}

# If live migrating and all disks are on iSCSI, disconnect from the old host
if ($live && $all_iscsi) {
	foreach my $d (@$disks) {
		my $iscsiserver = &get_managed_server($d->{'iscsi_server'});
		&$first_print(&text('smove_iscsidisc',
				    $iscsiserver->{'host'},
				    $d->{'iscsi_target'}));
		my $oldd = { %$d };
		my $oldserver = { %$server };
		$oldserver->{'xen_host'} = $vhost->{'id'};
		$err = &disconnect_iscsi_disk($oldserver, $oldd);
		if ($err) {
			return &text('smove_eiscsiconn', $err);
			}
		&$second_print($text{'create_done'});
		}
	}

# Fix Xen console path
foreach my $f ('xen_console', 'data_files') {
	$server->{$f} =~ s/^\Q$old_xen_dir\//$new_xen_dir\//;
	}

# If any network interfaces use bridges that don't exist on the new host,
# remove the bridge
my $vifs = &xen_find_value("vif", $conf);
my @newvifs;
my %hostbridges = map { $_, 1 } &list_host_bridges($dest);
my $fixed_bridge = 0;
foreach my $vif (@$vifs) {
	my %info = map { split(/=/, $_) } split(/\s*,\s*/, $vif);
	if ($info{'bridge'} && !$hostbridges{$info{'bridge'}}) {
		delete($info{'bridge'});
		$fixed_bridge = 1;
		}
	push(@newvifs, join(",", map { $_."=".$info{$_} } keys %info));
	}
if ($fixed_bridge) {
	&xen_save_directive($conf, "vif", \@newvifs);
	&put_xen_config($server, $conf);
	}

if ($dumpsrc) {
	# Restore dump on the new host
        &$first_print($text{'smove_undumping'});
        my ($out, $ex, $err) = &run_xen_api_command($dest,
                "xm restore $dumpdest");
	&delete_file_on_server($dest, $dumpdest);
        if ($ex || $err) {
                return &text('smove_eundump', $err || $out);
                }
        &$second_print($text{'create_done'});
        }
elsif ($was_up) {
	# Start on the new host, if it was up before
	&$first_print($text{'smove_starting'});
	my ($ok, $err) = &startup_server($server);
	if (!$ok) {
		return &text('smove_startfailed', $err);
		}
	&$second_print($text{'create_done'});
	}

return undef;
}

# xen_move_change_ip(&server, &vhost, &dest, oldip, newip, newcidr, newgw)
# Change the IP on a xen instance being moved
sub xen_move_change_ip
{
my ($server, $vhost, $dest, $ip, $newip, $newcidr, $newgw) = @_;

# Change the interface IP
&$first_print(&text('smove_ipchanging', $ip, $newip));
my $ifaces = &type_xen_list_interfaces($server);
return $ifaces if (!ref($ifaces));
my ($iface) = grep { $_->{'address'} eq $ip } @$ifaces;
$iface || return $text{'smove_eifaceip'};
my $oldiface = { %$iface };
$iface->{'address'} = $newip;
$iface->{'netmask'} = &net::prefix_to_mask($newcidr);
&modify_primary_ip_address($server, $ip, $newip);
my $err = &type_xen_modify_interface($server, $iface, $oldiface);
return $err if ($err);
$server->{'ip'} = $newip;
&delete_ip_address($server, $ip);
&add_ip_address($server, $newip);
&$second_print($text{'create_done'});

# Change the default gateway if needed
$newgw = "" if ($newgw eq "*");
$newgw ||= &get_server_default_gateway($dest);
my $defgw = &get_server_default_gateway($vhost);
my @gws = split(/\s+/, $server->{'xen_gw'});
my $gw = $gws[0] eq "*" ? $defgw : $gws[0];
if ($newgw ne $gw) {
	&$first_print(&text('smove_gwchanging',
			    $gw || $text{'smove_gwnone'},
			    $newgw || $text{'smove_gwnone'}));
	my $err = &modify_default_gateway($server, $newgw);
	return $err if ($err);
	if ($server->{'xen_gw'}) {
		$gws[0] = $newgw;
		$server->{'xen_gw'} = join(" ", @gws);
		}
	&$second_print($text{'create_done'});
	}

return undef;
}

# type_xen_manual_move_system(&server, &dest, [directory])
# Update the configs to reflect a system move already done
sub type_xen_manual_move_system
{
my ($server, $dest, $directory) = @_;
my $vhost = &get_parent_server($server);

# Work out old and new dirs. If a custom dir was given for this Xen instance,
# assume that the same should be used on the dest.
my $old_xen_dir = $server->{'directory'} || $vhost->{'xen_dir'};
my $new_xen_dir = $directory || $server->{'directory'} || $dest->{'xen_dir'};

# Check for config
&$first_print($text{'smove_checkconf'});
if ($server->{'xen_conf'} =~ /^\Q$old_xen_dir\/\E(.*)$/) {
	$server->{'xen_conf'} = "$new_xen_dir/$1";
	}
my @st = &stat_file_on_server($dest, $server->{'xen_conf'});
if (!@st) {
	return &text('smove_echeckconf', $server->{'xen_conf'});
	}
&$second_print(&text('smove_checkconfok', $server->{'xen_conf'}));
$server->{'xen_on'} = $dest->{'id'};

# Check if disks exist on target
&$first_print($text{'smove_checkdisks'});
my $disks = &type_xen_list_disks($server, 0);
return $disks if (!ref($disks));
foreach my $d (@$disks) {
	my @dst = &recursive_stat_file_on_server($dest, $d->{'real'});
	if (!@dst) {
		return &text('smove_echeckdisk', $d->{'real'});
		}
	}
&$second_print($text{'smove_checkdisksok'});

# Fix up VNC port in config file, if any
if ($server->{'xen_vnc_display'} || $server->{'xen_vnc_unused'}) {
	&$first_print($text{'create_xenvnc'});
	my $err = &setup_xen_vnc_console($server);
	&$second_print($err ? &text('create_xenvncfailed', $err)
			    : &text('create_xenvncdone'));
	}

# Update disk directory if changed
$server->{'directory'} = $directory if (defined($directory));
delete($server->{'new_directory'}) if (defined($directory));

return undef;
}

# type_xen_find_servers(&vhost, &template-server, running-only, [basedir])
# Searches a Xen host system for running or configured instances, and adds
# them to Cloudmin's control. Returns an array of server objects found. May
# print stuff.
sub type_xen_find_servers
{
my ($vhost, $tmplserver, $running, $dir) = @_;
my @rv;

# Find all the servers we currently know about, for dupe checking
my (%alreadyip, %alreadyxen);
foreach my $s (&list_managed_servers()) {
	foreach my $ip (&get_server_ip($s)) {
		$alreadyip{$ip} = $s;
		}
	}

# Find .cfg files in common locations
my @dirs = $dir ? ( $dir )
		   : &unique("/etc/xen", "/xen", $vhost->{'xen_dir'});
my @files;
&$first_print(&text('find_xendirs', "<tt>".join(", ", @dirs)."</tt>"));
foreach my $dir (@dirs) {
	my ($out, $ex, $err) = &execute_command_on_server(
		$vhost, "ls ".quotemeta($dir)."/*.cfg");
	if (!$ex && !$err) {
		push(@files, split(/\r?\n/, $out));
		}
	}
if (@files) {
	&$second_print(&text('find_xendirsdone', scalar(@files)));
	}
else {
	&$second_print(&text('find_xendirsnone', "<tt>*.cfg</tt>"));
	return ( );
	}

# Find running Xen instances
&$first_print($text{'find_xeninstances'});
my $instances = &list_xen_instances($vhost);
if (ref($instances)) {
	&$second_print(&text('find_xeninstancesdone', scalar(@$instances)));
	}
else {
	&$second_print(&text('find_xeninstanceserr', $instances));
	return ( );
	}
my %runningmap = map { $_->{'instance'}, $_ } @$instances;

# Process each of the Xen config files
foreach my $file (@files) {
	&$first_print(&text('find_xenconfig', "<tt>$file</tt>"));

	# Fetch and parse the config file
	my $localfile = &transname();
	my $err = &get_file_from_server($vhost, $file, $localfile);
	if ($err) {
		&$second_print(&text('find_xenconfigerr', $err));
		next;
		}
	my $conf = &parse_xen_config($localfile);
	if (!$conf) {
		&$second_print($text{'find_xenconfigparse'});
		next;
		}

	# Check for critical variables
	my $name = &xen_find_value("name", $conf);
	if (!$name) {
		&$second_print(&text('find_xenconfigmiss', 'name'));
		next;
		}
	my $kernel = &xen_find_value("kernel", $conf);
	my $bootloader = &xen_find_value("bootloader", $conf);
	my $builder = &xen_find_value("builder", $conf);
	if (!$bootloader && !$kernel) {
		&$second_print(&text('find_xenconfigloader'));
		next;
		}
	if ($builder eq "hvm") {
		$server->{'xen_hvm'} = 1;
		}
	elsif ($bootloader =~ /pygrub/) {
		$server->{'xen_pygrub'} = 1;
		$server->{'xen_bootloader'} = $bootloader;
		}
	elsif ($kernel =~ /pv-grub/) {
		$server->{'xen_pygrub'} = 1;
		$server->{'xen_bootloader'} = $kernel;
		}
	my $address = &xen_find_value("address", $conf);
	if (!$address) {
		my $vifs = &xen_find_value("vifs", $conf);
		if ($vifs && @$vifs && $vifs->[0] =~ /ip=([0-9\.]+)/) {
			$address = $1;
			}
		}
	if (!$address) {
		&$second_print(&text('find_xenconfigmiss', 'address'));
		next;
		}

	# Skip if we already host it
	my ($already) = &get_managed_server_by("xen", $name);
	if ($already) {
		&$second_print(&text('find_xenalready', $already->{'host'}));
		next;
		}
	$already = $alreadyip{$address};
	if ($already) {
		&$second_print(&text('find_xenalready2', $already->{'host'},
				     $address));
		next;
		}

	# Skip if not running, if requested
	if ($running && !$runningmap{$name}) {
		&$second_print(&text('find_notrunning', $name));
		next;
		}

	# Create the server object
	my $revhost = gethostbyaddr(inet_aton($address), AF_INET);
	my $server = { %$tmplserver,
			  'id' => &server_id(),
			  'desc' => "Automatically added Xen instance $name",
			  'type' => 'unknown',
			  'manager' => 'xen',
			  'xen' => $name,
			  'xen_on' => $vhost->{'id'},
			  'xen_conf' => $file,
			  'host' => $revhost || $address,
			};
	&complete_managed_server($server);
	&set_xen_filesystem($server, $conf);

	# Get status one more time, and save the server object
	&save_managed_server($server);
	&check_server_status($server);
	if ($server->{'type'} eq 'unknown' || !$server->{'type'}) {
		my $guess = &guess_os_type($server);
		if ($guess) {
			$server->{'type'} = $guess;
			&check_server_status($server, 0, 1);
			}
		}
	&save_managed_server($server);
	&$second_print(&text('find_xendone', $server->{'host'},
			     &describe_status($server, 1, 1)));
	push(@rv, $server);
	}

return @rv;
}

# Returns 2 if some Xen system can have it's network speed set, but needs
# a reboot, or 1 if it can be set live
sub type_xen_can_network_rate
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $xenver = &get_xen_software_version($vhost);
if ($xenver >= 4) {
	return 1;
	}
return 2;
}

# type_xen_get_network_rate(&server)
# Returns the maximum network speed (in bits per second) for a Xen instance.
# Returns 0 if not set.
sub type_xen_get_network_rate
{
my ($server) = @_;
my $conf = &get_xen_config($server, 1);
return undef if (!ref($conf));
my $vifs = &xen_find_value("vif", $conf);
if ($vifs->[0] =~ /rate=(\d+)([KMGT])([Bb])\/s/) {
	my $r = $1;
	$r *= ($2 eq "K" ? 1024 :
	       $2 eq "M" ? 1024*1024 :
	       $2 eq "G" ? 1024*1024*1024 :
	       $2 eq "T" ? 1024*1024*1024*1024 : 1);
	$r *= 8 if ($3 eq "B");
	return $r;
	}
return 0;
}

# type_xen_save_network_rate(&server, rate)
# Updates the network rate for a Xen instance. Returns undef on success or an
# error message on failure.
sub type_xen_save_network_rate
{
my ($server, $rate) = @_;
my $ratebits = $rate;
my $vhost = &type_xen_parent($server);

# Update the .cfg file
my $conf = &get_xen_config($server, 0);
my $vifs = &xen_find_value("vif", $conf);
my $ratestr;
if ($rate) {
	my $units;
	if ($rate % (1024*1024) == 0) {
		$rate /= 1024*1024;
		$units = "M";
		}
	else {
		$rate = int($rate/1024);
		$units = "K";
		}
	$ratestr = "${rate}${units}b/s";
	foreach my $vif (@$vifs) {
		if ($vif !~ s/rate=(\d+)([KMGT])([Bb])\/s/rate=$ratestr/) {
			$vif .= "," if ($vif);
			$vif .= "rate=$ratestr";
			}
		}
	}
else {
	foreach my $vif (@$vifs) {
		$vif =~ s/[ ,]*rate=(\S+)//;
		}
	$ratestr = undef;
	}
&xen_save_directive($conf, "vif", $vifs);
my $err = &put_xen_config($server, $conf);
return $err if ($err);

# Apply to the active network interface
# XXX for HVM too?
my $ifaces = &type_xen_list_interfaces($server);
ref($ifaces) || return "Failed to list network interfaces : $ifaces";
foreach my $iface (sort { $b->{'devidx'} <=> $a->{'devidx'} } @$ifaces) {
	next if ($iface->{'virtual'} ne '' || $iface->{'devidx'} eq '');
	$iface->{'rate'} = $ratestr;
	my $err = &re_attach_xen_interface($vhost, $server, $iface, $iface);
	}

$server->{'xen_network_rate'} = $ratebits;
return undef;
}

# can_host_xen(&server)
# Returns 1 if some server is of a type that we can do Xen hosting on
sub can_host_xen
{
my ($server) = @_;
return $server->{'manager'} eq 'this' || $server->{'manager'} eq 'real' ||
       $server->{'manager'} eq 'kvm';
}

# check_xen_host_software(&server)
# Checks if some system has the Xen software installed, and the kernel
# patch needed to use it.
sub check_xen_host_software
{
my ($server) = @_;

# Check if linux
my ($out, $ex, $err) = &execute_command_on_server($server, "uname");
return $text{'xen_elinux'} if ($out !~ /linux/i);

# Check commands
&has_command_on_server($server, "parted") ||
	return &text('xen_ewhich', "<tt>parted</tt>");
&has_command_on_server($server, "xm") ||
    &has_command_on_server($server, "xl") ||
	return &text('xen_ewhich', "<tt>xm/xl</tt>");

# Check kernel
my ($out, $ex, $err) = &execute_command_on_server($server,
				"test -d /proc/xen");
return $text{'xen_ekernel'} if ($ex);

# Check for virbr0 interface, which breaks networking!
if (&get_xen_software_version($server) < 4.4) {
	my ($out, $ex, $err) = &execute_command_on_server($server, "ifconfig -a");
	if ($out =~ /(virbr\d+)/) {
		return &text('xen_evibr', "<tt>$1</tt>",
			"<tt>/etc/libvirt/qemu/networks/default.xml</tt>");
		}
	}

# Make sure at least one bridge exists
my @bridges = &list_host_bridges($server);
if (!@bridges) {
	return $text{'xen_ebridges'};
	}

# Check for VirtIO disk or networking
# XXX

return undef;
}

# list_xen_instances(&server, [no-cache])
# Returns a array ref of all Xen instances on some real server, or
# an error message
sub list_xen_instances
{
my ($server, $nocache) = @_;
if ($list_xen_instances_cache{$server->{'id'}} && !$nocache) {
	# Use cached value
	return $list_xen_instances_cache{$server->{'id'}};
	}
my ($out, $ex, $err) = &run_xen_api_command($server, "xm list");
return $out if ($ex);
return $err if ($err);
my @rv;
foreach my $l (split(/\r?\n/, $out)) {
	my @w = split(/\s+/, $l);
	if (@w < 6) {
		# Missing domain ID and state, meaning it is down in Xen 4
		next;
		}
	my ($name, $id, $mem, $cpus, $state, $time) = @w;
	if ($id ne "ID" && defined($id) && $id != 0) {
		my ($s) = grep { $_ ne "-" } split(//, $state);
		my $zombie = 0;
		if ($name =~ s/^Zombie-//) {
			$zombie = 1;
			}
		push(@rv, { 'id' => $id,
			    'instance' => $name,
			    'zombie' => $zombie,
			    'mem' => $mem*1024*1024,
			    'cpus' => $cpus,
			    'state' => $s,
			    'time' => $time });
		}
	}
$list_xen_instances_cache{$server->{'id'}} = \@rv;
return \@rv;
}

# parse_xen_config(file)
# Parses a Xen config file (actually a list of python variables!) into an
# array ref.
sub parse_xen_config
{
my ($file) = @_;
open(CONF, $file);
my @lines = <CONF>;
close(CONF);
return &parse_xen_config_lines(\@lines, $file);
}

# parse_xen_config_lines(&lines, [file])
# Parses Xen config text into an array ref
sub parse_xen_config_lines
{
my ($lines, $file) = @_;
my @rv;
for(my $lnum=0; $lnum<@$lines; $lnum++) {
	my $l = $lines->[$lnum];
	$l =~ s/\r|\n//g;
	$l =~ s/^#.*//g;	# strip whole-line comments
	$l =~ s/\s#.*//g;	# strip end-line comments
	if ($l =~ /^\s*(\S+)\s*=\s*'([^']*)'/ ||
	    $l =~ /^\s*(\S+)\s*=\s*"([^"]*)"/ ||
	    $l =~ /^\s*(\S+)\s*=\s*(\d+)/) {
		# Single-value variable
		push(@rv, { 'name' => $1,
			    'value' => $2,
			    'type' => 0,
			    'line' => $lnum,
			    'eline' => $lnum,
			    'file' => $file });
		}
	elsif ($l =~ /^\s*(\S+)\s*=\s*\[(.*)\]/) {
		# List of values
		push(@rv, { 'name' => $1,
			    'rawvalue' => $2,
			    'values' => [ &split_xen_array($2) ],
			    'type' => 1,
			    'line' => $lnum,
			    'eline' => $lnum,
			    'file' => $file });
		}
	elsif ($l =~ /^\s*(\S+)\s*=\s*\[([^\]]*)$/) {
		# List, but split over several lines!
		my $dir = { 'name' => $1,
			       'rawvalue' => $2,
			       'values' => [ &split_xen_array($2) ],
			       'type' => 1,
			       'line' => $lnum,
			       'eline' => $lnum,
			       'file' => $file };
		push(@rv, $dir);
		$lnum++;
		# Read list until ending ]
		while($lnum < @$lines) {
			$l = $lines->[$lnum];
			$l =~ s/\r|\n//g;
			$l =~ s/^#.*//g;	# strip whole-line comments
			$l =~ s/\s#.*//g;	# strip end-line comments
			if ($l =~ /^(\s*)(.*)\]\s*$/) {
				# End of list, perhaps with some value
				if ($2) {
					$rawvalue .= $1.$2;
					push(@{$dir->{'values'}},
					     &split_xen_array($2));
					}
				last;
				}
			elsif ($l =~ /\S/) {
				# Some value(s)
				$rawvalue .= $_;
				push(@{$dir->{'values'}},
				     &split_xen_array($_));
				}
			$lnum++;
			}
		$dir->{'eline'} = $lnum;
		}
	elsif ($l =~ /\S/) {
		# Unknown line?
		print STDERR "Unknown Xen config line $_\n";
		}
	}
return \@rv;
}

# xen_find_value(name, &conf)
# Returns the value or values with some name
sub xen_find_value
{
my ($name, $conf) = @_;
my @rv = map { $_->{'values'} || $_->{'value'} } &xen_find(@_);
return wantarray ? @rv : $rv[0];
}

# xen_find(name, &conf)
# Returns the config object or objects with some name
sub xen_find
{
my ($name, $conf) = @_;
my @rv = grep { $_->{'name'} eq $name } @$conf;
return wantarray ? @rv : $rv[0];
}

# split_xen_array(string)
# Converts a Python array string like 'foo', 6, 'bar' into an Perl array
sub split_xen_array
{
my ($str) = @_;
$str =~ s/^\s*//g;
$str =~ s/\s*$//g;
my @rv;
while(1) {
	if ($str =~ /^\s*'([^']*)',?(.*)/ ||
	    $str =~ /^\s*"([^"]*)",?(.*)/ ||
	    $str =~ /^\s*(\d+),?(.*)/) {
		push(@rv, $1);
		$str = $2;
		}
	else {
		if ($str =~ /\S/) {
			print STDERR "Unknown array element $str\n";
			}
		last;
		}
	}
return @rv;
}

# xen_save_directive(&conf, name, [value])
# Updates a value in a Xen config file. No need to call flush_file_lines
sub xen_save_directive
{
my ($conf, $name, $value) = @_;
my $old = &xen_find($name, $conf);
my $file = $old ? $old->{'file'} : $conf->[0]->{'file'};
my $oldlen = $old ? $old->{'eline'}-$old->{'line'}+1 : undef;
my $lref = &read_file_lines($file);
my $newline;
if (defined($value)) {
	# Work out new config file line
	if (ref($value)) {
		# A list of strings
		$newline = "$name = [".join(",", map { "'$_'" } @$value)."]";
		}
	elsif ($value =~ /^\d+$/ || $value =~ /^\d*\.\d+$/) {
		# A number, no quoting needed
		$newline = "$name = $value";
		}
	elsif ($value =~ /"/) {
		# Need to quote "
		$newline = "$name = '$value'";
		}
	else {
		# Just some string
		$newline = "$name = \"$value\"";
		}
	}
if ($old && defined($value)) {
	# Update a line
	splice(@$lref, $old->{'line'}, $oldlen, $newline);
	$old->{'value'} = $value;
	$old->{'type'} = ref($value) ? 1 : 0;
	$old->{'eline'} = $old->{'line'};
	if ($oldlen != 1) {
		# Need to re-number later lines down
		foreach my $c (@$conf) {
			$c->{'line'} -= $oldlen-1
				if ($c->{'line'} > $old->{'eline'});
			}
		}
	}
elsif ($old && !defined($value)) {
	# Remove a line
	splice(@$lref, $old->{'line'}, $oldlen);
	my $idx = &indexof($old, @$conf);
	splice(@$conf, $idx, 1) if ($idx >= 0);
	foreach my $c (@$conf) {
		$c->{'line'} -= $oldlen if ($c->{'line'} > $old->{'eline'});
		}
	}
elsif (!$old && defined($value)) {
	# Append a line
	push(@$conf, { 'name' => $name,
		       'value' => $value,
		       'type' => ref($value) ? 1 : 0,
		       'line' => scalar(@$lref),
		       'file' => $file });
	push(@$lref, $newline);
	}
&flush_file_lines($file);
}

# get_xen_config(&server, [read-only])
# Returns the Xen config structure for some server, or an error message
sub get_xen_config
{
my ($server, $ro) = @_;
my $vhost = &type_xen_parent($server);
return "Failed to get host system" if (!$vhost);
if ($get_xen_config_cache{$server->{'id'}} && $ro) {
	# Use cached copy
	return $get_xen_config_cache{$server->{'id'}};
	}
my $conf;
if ($ro && &can_ssh_status($vhost)) {
	# Can just cat file
	my ($out, $ex, $err) = &execute_command_on_server(
		$vhost, "cat ".quotemeta($server->{'xen_conf'}));
	return $err if ($err);
	return $out if ($ex);
	my @lines = split(/\r?\n/, $out);
	$conf = &parse_xen_config_lines(\@lines);
	}
else {
	# Make local copy for editing
	my $cfile = &transname();
	my $err = &get_file_from_server($vhost, $server->{'xen_conf'}, $cfile);
	return $err if ($err);
	$conf = &parse_xen_config($cfile);
	&unlink_file($cfile) if ($ro);
	}
return "Failed to parse config file $server->{'xen_conf'}" if (!$conf);
return "Config file $server->{'xen_conf'} is empty" if (!@$conf);
$get_xen_config_cache{$server->{'id'}} = $conf;
return $conf;
}

# put_xen_config(&server, &conf)
# Uploads a Xen configuration (which must be returned by get_xen_config) to the 
# specified server's host. Returns undef if OK, or an error message if not.
sub put_xen_config
{
my ($server, $conf) = @_;
my $vhost = &type_xen_parent($server);
return "Failed to get host system" if (!$vhost);
my $cfile = $conf->[0]->{'file'};
my $err = &put_file_to_server($vhost, $cfile, $server->{'xen_conf'});
$get_xen_config_cache{$server->{'id'}} = $conf;
return $err;
}

# split_xen_vif_ips(vif-info, [field])
# Given a string like "ip=193.9.101.222,bridge=xenbr0" , return the IP
# addresses as a list
sub split_xen_vif_ips
{
my ($vifinfo, $field) = @_;
$field ||= "ip";
my %info = map { split(/=/, $_) } split(/\s*,\s*/, $vifinfo);
return split(/\s+/, $info{$field});
}

# join_xen_vif_ips(vif-info, &values, [field])
# Given a string like "ip=193.9.101.222,bridge=xenbr0" and a new list of IPs,
# update the IPs in the string and return it
sub join_xen_vif_ips
{
my ($vifinfo, $ips, $field) = @_;
$field ||= "ip";
my @info = split(/,/, $vifinfo);
my $found = 0;
foreach my $i (@info) {
	my ($n, $v) = split(/=/, $i);
	if ($n eq $field) {
		$i = $n."=".join(" ", @$ips);
		$found = 1;
		last;
		}
	}
if (!$found) {
	push(@info, $n."=".join(" ", @$ips));
	}
return join(",", @info);
}

# xen_instance_chooser_button(instance-field, host-field)
# Returns HTML for a popup window for selecting a running xen instance
sub xen_instance_chooser_button
{
my ($ifield, $hfield) = @_;
return "<input type=button onClick='ifield = form.$ifield; chooser = window.open(\"xen_chooser.cgi?host=\"+escape(form.$hfield.value), \"chooser\", \"toolbar=no,menubar=no,scrollbars=yes,width=600,height=300\"); chooser.ifield = ifield; window.ifield = ifield' value=\"...\">\n";
}

sub list_xen_on_actions
{
return ("reboot", "shutdown", "crash", "poweroff");
}

sub list_xen_on_options
{
return ('destroy', 'restart', 'preserve', 'rename-restart');
}

# get_xen_software_version(&server, [include-extra-version])
# Returns the Xen version installed on some host system, like 3.2. Or returns
# undef if it could not be worked out.
sub get_xen_software_version
{
my ($server, $incextra) = @_;
my ($out, $ex, $err) = &run_xen_api_command($server, "xm info", $short_timeout);
if (!$ex && !$err) {
	my $vmajor = $out =~ /xen_major:?\s*:\s*([0-9\.]+)/i ? $1 : undef;
	my $vminor = $out =~ /xen_minor:?\s*:\s*([0-9\.]+)/i ? $1 : undef;
	my $vextra = $out =~ /xen_extra:?\s*:\s*([0-9\.]+)/i ? $1 : undef;
	if (defined($vmajor) && defined($vminor)) {
		my $rv = "$vmajor.$vminor";
		if (defined($vextra) && $incextra) {
			$rv .= $vextra;
			}
		return $rv;
		}
	}
if ($server->{'xen_version'}) {
	# Use last cached version
	return $server->{'xen_version'};
	}
return undef;
}

# get_xen_instance_id(&server)
# Returns the Xen ID number for a system
sub get_xen_instance_id
{
my ($server) = @_; 
my $vhost = &type_xen_parent($server);
my $instances = &list_xen_instances($vhost);
return undef if (!ref($instances));
my ($i) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
return undef if (!$i);
return $i->{'id'};
}

# get_active_vnc_port(&server)
# Returns the currently assigned dynamic VNC port
sub get_active_vnc_port
{
my ($server) = @_;
my $vhost = &type_xen_parent($server);
my $id = &get_xen_instance_id($server);
return undef if (!$id);
my ($out, $ex, $err) = &run_xen_api_command($vhost,
	"xenstore-read /local/domain/$id/console/vnc-port");
if (!$ex && !$err && $out =~ /(\d+)/) {
	return $1;
	}
return undef;
}

# setup_xen_vnc_console(&server)
# Add directives to a Xen instance's .conf file for VNC console access. Returns
# undef on success, or an error message on failure.
sub setup_xen_vnc_console
{
my ($server) = @_;
my $conf = &get_xen_config($server, 0);
return $conf if (!ref($conf));
my $vhost = &type_xen_parent($server);
my $ip = &get_server_ip($vhost);
if ($config{'vnc_passmode'} == 0) {
	$server->{'xen_vnc_passwd'} = $server->{'ssh_pass'};
	}
$server->{'xen_vnc_passwd'} ||= &generate_random_password();
$server->{'xen_vnc_passwd'} =~ s/,//g;
$server->{'xen_vnc_unused'} = 1;
&xen_save_directive($conf, "vnc", 1);
&xen_save_directive($conf, "vnclisten", "0.0.0.0");
&xen_save_directive($conf, "vncunused", 1);
&xen_save_directive($conf, "vncpasswd", $server->{'xen_vnc_passwd'});
if (!$server->{'xen_hvm'}) {
	&xen_save_directive($conf, "vfb",
		[ "type=vnc,vncunused=1,vncpasswd=$server->{'xen_vnc_passwd'},".
		  "vnclisten=0.0.0.0" ]);
	}
return &put_xen_config($server, $conf);
}

sub type_xen_can_backup
{
return 1;
}

# type_xen_backup_system(&server, dest, shutdown, owner-name)
# Backup a single Xen instance to the given destination, and return a backup
# status hash ref.
sub type_xen_backup_system
{
my ($server, $deststr, $shutdown, $owner) = @_;
return &images_backup_system($server, $deststr, $shutdown, $owner);
}

# type_xen_restore_system(&server, dest, shutdown)
# Restore single Xen instance from the given destination, and return a backup
# status hash ref.
sub type_xen_restore_system
{
my ($server, $deststr) = @_;
delete($server->{'xen_console'});	# May be incorrect on new host
return &image_restore_system($server, $deststr);
}

# type_xen_backup_extension(&server)
# Always returns .gz, as even multi-disk systems are backed up as separate files
sub type_xen_backup_extension
{
my ($server) = @_;
return "gz";
}

# type_xen_open_shell(&server)
# Runs SSH to connect to a remote system. Returns an error message
# (or undef if OK), and a file handle.
sub type_xen_open_shell
{
my ($server) = @_;
my ($out, $ex, $err, $status, $fh, $fpid) =
	&execute_command_on_server($server, &get_bourne_shell($server), 0, 1);
return $err ? ($err, undef) : $ex ? ($out, undef) : (undef, $fh);
}

# type_xen_get_dns_name(&server)
# Returns a system's DNS name and zone name
sub type_xen_get_dns_name
{
my $vhost = &get_parent_server($server);
return ( $server->{'xen_dns'}, $vhost->{'xen_zone'} );
}

# type_xen_image_formats()
# Any of these can be converted to a Xen partition
sub type_xen_image_formats
{
return ( "", "disk", "qcow2", "qcow", "vmdk", "cd" );
}

# type_xen_can_convert_image(&image)
# Xen whole-disk images can be converted to KVM
sub type_xen_can_convert_image
{
my ($img) = @_;
if ($img->{'format'} eq 'disk' || $img->{'format'} eq 'cd') {
	return ( { 'manager' => 'kvm', },
		 { 'manager' => 'citrix',
		   'citrix_hvm' => 1, },
	       );
	}
return ( );
}

# type_xen_can_clone(&server)
# Any Xen instance can be cloned, if the host system is up
sub type_xen_can_clone
{
my ($server) = @_;
return $server->{'status'} eq 'nohost' ? 0 :
       $server->{'xen_filesystem_vg'} ? 2 : 1;
}

# type_xen_can_clone_remote(&server)
# Xen instances can be cloned to a another host system.
sub type_xen_can_clone_remote
{
return 1;
}

# type_xen_check_clone(&oldserver, &newserver, new-hostname, &ips, snapshot)
# 		       [&new-host], [&ips-v6])
# Checks if cloning of some system will work, before shutting it down. Returns
# undef on success or an error message on failure.
sub type_xen_check_clone
{
my ($oldserver, $newserver, $newhost, $ips, $snapshot, $dest, $ips6) = @_;
my $vhost = &type_xen_parent($newserver);
$dest ||= $vhost;

# Make sure hostname is sensible
if ($dest->{'xen_zone'}) {
	return $text{'clone_ehost2'} if ($newhost =~ /\./);
	}

# Check for IPs if needed
if (!$dest->{'xen_start'}) {
	$ips && scalar(@$ips) || return &text('clone_eips', $dest->{'host'});
	}
if ($ips) {
	my @oldips = split(/\s+/, $oldserver->{'xen_ip'});
	scalar(@oldips) == scalar(@$ips) ||
		return &text('clone_eips2', scalar(@oldips), scalar(@$ips));
	}

# Check for IPv6 addresses
if (!$dest->{'xen_pool6'} && $oldserver->{'xen_ip6'}) {
        $ips6 && scalar(@$ips6) || return &text('clone_eips6', $dest->{'host'});
        }
if ($ips6) {
        my @oldips6 = split(/\s+/, $oldserver->{'xen_ip6'});
        scalar(@oldips6) == scalar(@$ips6) ||
                return &text('clone_eips62', scalar(@oldips), scalar(@$ips));
        }

# Make sure all disks are on LVM if snapshotting
if ($snapshot) {
	my $disks = &type_xen_list_disks($newserver, 0);
	foreach my $d (@$disks) {
		next if ($d->{'device'} && !$d->{'lv'});
		$d->{'lv'} || return &text('clone_elvm', $d->{'real'});
		}
	}

# If cloning to a new host, make sure it uses the same iSCSI server
if ($vhost->{'id'} ne $dest->{'id'}) {
        my $disks = &type_xen_list_disks($newserver, 0);
        foreach my $d (@$disks) {
                if ($d->{'iscsi_ip'}) {
                        $d->{'iscsi_server'} eq '' &&
                                return $text{'clone_eicsiserver'};
                        my $old_iscsiserver = &get_managed_server(
                                $d->{'iscsi_server'});
                        if ($server->{'xen_force_iscsi'} ne '') {
                                my $new_iscsiserver = &get_managed_server(
                                                        $d->{'iscsi_server'});
                                my $err = &check_iscsi_client($dest);
                                $err && return &text('smove_eiscsidest', $err);
                                }
                        else {
                                $dest->{'xen_iscsi'} eq '' &&
                                        return $text{'clone_eicsidest'};
                                my $new_iscsiserver = &get_managed_server(
                                                        $dest->{'xen_iscsi'});
                                $old_iscsiserver->{'id'} eq
                                    $new_iscsiserver->{'id'} ||
                                        return &text('clone_eiscsidiff',
                                            $old_iscsiserver->{'host'},
                                            $new_iscsiserver->{'host'});
                                }
                        }
                }
        }

# Prevent snapshotting when cloning remotely
if ($snapshot && $dest->{'id'} ne $vhost->{'id'}) {
	return $text{'clone_esnapremote'};
	}

# If cloning a system on LVM, make sure the dest also has LVM
if ($server->{'xen_filesystem_vg'} && !$dest->{'xen_vg'} &&
    $dest->{'id'} ne $vhost->{'id'}) {
	return $text{'clone_elvmremote'};
	}

return undef;
}

# type_xen_clone_system(&oldserver, &newserver, new-hostname, &ips, snapshot,
# 			[&new-host], [&ips-v6])
# Creates a clone of some Xen instance, updating the newserver hash.
# Returns an error message and a flag indicating if the failure was total or
# only partial.
sub type_xen_clone_system
{
my ($oldserver, $newserver, $newhost, $ips, $snapshot, $dest, $ips6) = @_;
my $vhost = &type_xen_parent($newserver);
$dest ||= $vhost;
delete($list_xen_instances_cache{$vhost->{'id'}});
delete($list_xen_instances_cache{$dest->{'id'}});
my $old_xen_dir = $oldserver->{'directory'} || $vhost->{'xen_dir'};
my $new_xen_dir = $newserver->{'directory'} || $dest->{'xen_dir'};

# Generate the new full hostname
$newserver->{'xen'} = $newhost;
if ($dest->{'xen_zone'}) {
	$newserver->{'host'} = $newhost.".".$dest->{'xen_zone'};
	}
else {
	$newserver->{'host'} = $newhost;
	}
my $clash = &get_managed_server_by("host", $newserver->{'host'});
if ($clash) {
	return (&text('clone_eclash', $newserver->{'host'}), 1);
	}

# Allocate new IP addresses
if ($ips) {
	# Use supplied list, keep old netmasks and gateways
	$newserver->{'xen_ip'} = join(" ", @$ips);
	}
else {
	# Allocate new IPs
	&$first_print($text{'clone_allocating'});
	my $err = &allocate_xen_ip_addresses($newserver, $dest);
	return ($err, 1) if ($err);
	}
my @ips = split(/\s+/, $newserver->{'xen_ip'});
my @cidrs = split(/\s+/, $newserver->{'xen_cidr'});
my @netmasks = map { &net::prefix_to_mask($_) } @cidrs;
my $defgw = &get_server_default_gateway($dest);
my @gws = split(/\s+/, $newserver->{'xen_gw'});
my @setgws = grep { $_ ne "*" } @gws;
if (!@setgws && $defgw) {
	@gws = ( $defgw );
	}
$newserver->{'ip_addresses'} = join(" ", @ips);
&complete_managed_server($newserver);
if (!$ips) {
	&$second_print(&text('clone_allocated', join(" ", @ips)));
	}

# Allocate new IPv6 addresses
if ($ips6) {
	# Use supplied list, keep old netmasks
	$newserver->{'xen_ip6'} = join(" ", @$ips6);
        }
elsif ($oldserver->{'xen_ip6'}) {
        # Need to allocate new ones
        &$first_print($text{'clone_allocating6'});
        my $err = &allocate_xen_ip6_addresses($newserver, $dest);
        return ($err, 1) if ($err);
        }
my @ips6 = split(/\s+/, $newserver->{'xen_ip6'});
my @netmasks6 = split(/\s+/, $newserver->{'xen_netmask6'});
$newserver->{'ip_addresses6'} = join(" ", @ips6);
&complete_managed_server($newserver);
if (!$ips6 && @ips6) {
        &$second_print(&text('clone_allocated', join(" ", @ips6)));
        }

# Copy the Xen .cfg file
my $cfgfile = "$new_xen_dir/$newserver->{'xen'}.cfg";
&$first_print(&text('clone_xencfg', "<tt>$cfgfile</tt>"));
my $err = &transfer_file_between_servers($vhost, $dest,
					 $oldserver->{'xen_conf'},
				 	 $cfgfile);
if ($err) {
	return (&text('clone_exencfg', $err), 1);
	}
my $conf = &get_xen_config($oldserver, 0);	# Will be saved on new host
&xen_save_directive($conf, "name", $newserver->{'xen'});
&xen_save_directive($conf, "address", $ips[0]);

&$second_print($text{'create_done'});

# Clear iSCSI username and password, so that a new one is generated
delete($newserver->{'iscsi_user'});
delete($newserver->{'iscsi_pass'});
delete($newserver->{'iscsi_disk_count'});

# Copy and fix each of the disks in the .cfg file
&$first_print(&text('clone_copydisks'));
my $disks = &type_xen_list_disks($newserver, 1);
if (!ref($disks)) {
	$disks = &type_xen_list_disks($newserver, 0);
	}
if (!ref($disks)) {
	return (&text('clone_ecopydisks', $disks), 1);
	}
&$indent_print();
my @disklist;
my $oldxen = $oldserver->{'xen'};
my $newxen = $newserver->{'xen'};
foreach my $d (@$disks) {
	&$first_print(&text('clone_copydisk', $d->{'real'}));
	if ($d->{'device'} && !$d->{'lv'} && $d->{'iscsi_server'} eq '') {
		# A real device, which shouldn't be touched
		my $media = $d->{'media'} && $d->{'media'} ne 'disk' ?
				':'.$d->{'media'} : '';
		push(@disklist, $d->{'suffix'}.":".$d->{'real'}.",".
				$d->{'virtdev'}.$media.",".$d->{'mode'});
		&$second_print(&text('smove_devdisk'));
		next;
		}
	my $err;
	my $snapped = 0;
	if ($d->{'lv'}) {
		# Pick a new LV name
		my $newlv = $d->{'lv'};
		if ($newlv =~ /\Q$oldxen\E/) {
			$newlv =~ s/\Q$oldxen\E/$newxen/;
			}
		else {
			$newlv .= "-".$newxen;
			}
		if ($dest->{'xen_vg'} ne $d->{'vg'}) {
			$d->{'vg'} = $dest->{'xen_vg'};
			}
		$d->{'newreal'} = "/dev/".$d->{'vg'}."/".$newlv;

		# Create the new LV
		$last_remote_error = undef;
		my $lv = { 'name' => $newlv,
			   'size' => int($d->{'size'}/1024),
			   'perm' => 'rw',
			   'alloc' => 'n',
			   'vg' => $d->{'vg'} };
		if ($snapshot) {
			$lv->{'is_snap'} = 1;
			$lv->{'snapof'} = $d->{'lv'};
			$snapped = 1;
			}
		&remote_finished();
		&remote_foreign_require($dest, "lvm", "lvm-lib.pl");
		$err = &remote_foreign_call($dest, "lvm",
					  "create_logical_volume", $lv);
		$err ||= $last_remote_error;

		if (!$err && !$d->{'swap'} && !$snapshot) {
			# Copy the data using dd
			$err = &transfer_file_between_servers(
			    $vhost, $dest,
			    $d->{'real'}, $d->{'newreal'}, 1, 1,
			    int($d->{'size'}/(1024*1024))*1024*1024);
			}
		elsif (!$err && !$d->{'swap'} && $snapshot &&
		       &type_xen_can_fsck($oldserver, $d)) {
			# Fsck the disk after snapshotting
			my $newd = { %$d };
			$newd->{'real'} = $d->{'newreal'};
			$newd->{'lv'} = $newlv;
			my ($ok, $out) = &type_xen_fsck_disk(
						$oldserver, $newd, 0);
			}
		}
	elsif ($d->{'iscsi_server'} ne '') {
		# Cloning to a new iSCSI disk (with new target name)
                my $newd = { %$d };
                delete($newd->{'iscsi_target'});
		$newserver->{'xen_on'} = $dest->{'id'}; # So disk is mounted on
							# new host system
		$err = &create_iscsi_disk($newserver, $newd);
		if (!$err) {
                        # Copy the data using DD, on the iSCSI server
                        $d->{'newreal'} = $newd->{'real'};
			$err = &clone_iscsi_disk($oldserver, $newserver,
						 $d, $newd);
                        }
		}
	else {
		# Pick a new filename
		$d->{'newreal'} = $d->{'real'};
		if ($d->{'newreal'} =~ /\Q$oldxen\E/) {
			$d->{'newreal'} =~ s/\Q$oldxen\E/$newxen/;
			}
		else {
			$d->{'newreal'} .= "-".$newxen;
			}
		if ($d->{'real'} !~ /^\Q$new_xen_dir\E\//) {
			$d->{'newreal'} =~ s/^.*[\\\/]//;
			$d->{'newreal'} = "$new_xen_dir/$d->{'newreal'}";
			}
		# Copy the file
		$err = &transfer_file_between_servers($vhost, $dest,
                                $d->{'real'}, $d->{'newreal'}, 0, 1);
		}

	if ($err) {
		# Failed .. but may be skippable
		&$second_print(&text('clone_ecopydisk', $err));
		if ($newserver->{'xen_filesystem'} eq $d->{'real'} ||
		    $newserver->{'xen_swap'} eq $d->{'real'}) {
			return (&text('clone_ecopydiskfatal', $err), 1);
			}
		}
	else {
		# Update list of Xen disk options for saving to .cfg file
		if (($d->{'suffix'} eq 'file' || $d->{'suffix'} eq 'tap:aio') &&
		    $dest->{'id'} ne $vhost->{'id'}) {
			# Use file-based prefix for new host
			$d->{'suffix'} = $dest->{'xen_filetype'} || 'file';
			}
		push(@disklist, $d->{'suffix'}.":".$d->{'newreal'}.",".
				$d->{'virtdev'}.",".$d->{'mode'});
		if ($newserver->{'xen_filesystem'} eq $d->{'real'}) {
			$newserver->{'xen_filesystem'} = $d->{'newreal'};
			$newserver->{'xen_filesystem_iscsi_target'} =
                                $d->{'iscsi_target'};
			}
		if ($newserver->{'xen_swapfile'} eq $d->{'real'}) {
			$newserver->{'xen_swapfile'} = $d->{'newreal'};
			$newserver->{'xen_swapfile_iscsi_target'} =
                                $d->{'iscsi_target'};
			}
		&$second_print(&text($snapped ? 'clone_snapdiskdone'
					      : 'clone_copydiskdone',
				     $d->{'newreal'}));
		}
	}
&$outdent_print();
&xen_save_directive($conf, "disk", \@disklist);
$newserver->{'xen_on'} = $dest->{'id'};
$newserver->{'xen_conf'} = $cfgfile;
delete($newserver->{'xen_console'});
my $err = &put_xen_config($newserver, $conf);
&$second_print($text{'create_done'});

# For Xen 4.2, need to register the config file
my $xenver = &get_xen_software_version($dest);
if ($xenver >= 4.2 && $vhost->{'xen_api_type'} ne 'xl') {
	my ($out, $ex, $err) = &run_xen_api_command($dest,
		"xm new $cfgfile");
	$err = $out if ($ex && !$err);
	return ("Failed to register Xen instance : $err", 1) if ($err);
	}

# Register in DNS
if ($dest->{'xen_zone'}) {
	my $dnsname = $newserver->{'xen'}.".".
		      $dest->{'xen_zone'}.".";
	if (&add_bind8_record($dnsname, $ips[0],
			      $dest->{'xen_zone'})) {
		$newserver->{'xen_dns'} = $dnsname;
		if (@ips6) {
                        &add_bind8_record($dnsname, $ips6[0],
                                          $dest->{'xen_zone'});
                        }
		}
	else {
		delete($newserver->{'xen_dns'});
		}
	}

# Mount the filesystem
&$first_print($text{'create_xenmount'});
delete($server->{'xen_filesystem_offset'});	# May be invalidated
my $err = &mount_filesystem_if_needed($newserver);
my $mp = $newserver->{'xen_mount'};
if ($err || !$mp) {
	$err ||= "No mount point found!";
	&$second_print(&text('create_exenmount', $err));
	return ($err, 0);
	}
&$second_print(&text('create_xenmounted', "<tt>$mp</tt>"));

# Delete any virtual network interfaces if possible, and re-save all real
# interfaces to fix the vif line in the .cfg file
my $fixed_macs;
if (&type_xen_can_interfaces($newserver)) {
	my $ifaces = &type_xen_list_interfaces($newserver);
	my %hostbridges = map { $_, 1 } &list_host_bridges($dest);
	if (ref($ifaces)) {
		foreach my $i (@$ifaces) {
			if ($i->{'virtual'} ne '' ||
			    $i->{'fullname'} =~ /^eth(\d+)$/ &&
			    $1 >= scalar(@ips)) {
				# No IP is allocated for this interface ..
				# remove it
				&type_xen_delete_interface($newserver, $i);
				}
			elsif ($i->{'virtual'} eq '' &&
			       $i->{'fullname'} =~ /^eth(\d+)$/) {
				# Set the new correct IP and MAC address for
				# this interface
				my $oldi = { %$i };
				$i->{'address'} = $ips[$1];
				$i->{'ether'} = &generate_mac_address(
							$newserver, $1);
				if ($i->{'bridge'} &&
				    $vhost->{'id'} ne $dest->{'id'} &&
				    !$hostbridges{$i->{'bridge'}}) {
					# Bridge doesn't exist on new host
					delete($i->{'bridge'});
					}
				&type_xen_modify_interface(
					$newserver, $i, $oldi);
				}
			}
		$fixed_macs = 1;
		}
	}
if (!$fixed_macs) {
	# Cloudmin doesn't know how to manage interfaces, so instead fix mac=
	# directives
	my $conf = &get_xen_config($newserver, 0);
	my $vifs = &xen_find_value("vif", $conf);
	for(my $i=0; $i<@$vifs; $i++) {
		if ($vifs->[$i] =~ /mac=[0-9a-f:]+/i) {
			my $mac = &generate_mac_address($newserver, $i);
			$vifs->[$i] =~ s/mac=[0-9A-Fa-f:]+/mac=$mac/;
			}
		}
	&xen_save_directive($conf, "vif", $vifs);
	&put_xen_config($newserver, $conf);
	}

# Fix other config files that refer to the hostname or IP
&$first_print($text{'create_xenfix'});
my $fullhost = $dest->{'xen_zone'} ? 
		$newserver->{'xen'}.".".$dest->{'xen_zone'} :
		$newserver->{'xen'};
my $err = &fix_hostname_config_files($newserver, $ips[0],
					$newserver->{'host'}, $fullhost, $dest,
					$mp, $dest->{'xen_resolv'});
if ($err) {
	&$second_print(&text('create_exenfix', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Setup networking config files to use the IP.
&$first_print($text{'create_xennet'});
my $err = &fix_network_config_files($newserver, $ips[0], $netmasks[0],
				    $gws[0], $dest, $mp);
if ($err) {
	&$second_print(&text('create_exennet', $err));
	}       
else {
	&$second_print($text{'create_done'});
	}

# Add network configs for extra IPs
if (@ips > 1) {
	&$first_print($text{'create_xennet2'});
	my $err = &fix_additional_network_interface_files($newserver,
		[ @ips[1..$#ips] ], [ @netmasks[1..$#netmasks] ],
		[ @gws[1..$#gws] ], $dest, $mp);
	if ($err) {
		&$second_print(&text('create_exennet2', $err));
		}       
	else {
		&$second_print($text{'create_done'});
		}
	}

# Apply IPv6 addresses to interfaces
if (@ips6) {
        &$first_print($text{'create_xenipv6'});
        my $err = &apply_ipv6_addresses($newserver, \@ips6, \@netmasks6);
        if ($err) {
                &$second_print(&text('create_exenipv6', $err));
                }
        else {
                &$second_print($text{'create_done'});
                }
        }

# Un-mount
&$first_print($text{'create_xenumount'});
&umount_filesystem_if_needed($newserver);
&$second_print($text{'create_done'});

# Add DHCP entries
if ($config{'dhcp_add'}) {
	# Get MAC addresses
	my $fakeifaces = &type_xen_list_interfaces($newserver, 1);
	if (ref($fakeifaces)) {
		$fakeifaces->[0]->{'address'} ||= $ips[0];
		&add_all_iface_dhcpd($newserver, $fakeifaces);
		}
	}

# Fix up other config entries using the old name
foreach my $f ('xen_filesystem_lv', 'xen_swapfile_lv',
	       'xen_console', 'login_user', 'data_files') {
	$newserver->{$f} =~ s/^\Q$old_xen_dir\//$new_xen_dir\//;
	$newserver->{$f} =~ s/\Q$oldxen\E/$newxen/g;
	}

return (undef, 0);
}

# type_xen_can_failover([&server, [&new-dest]])
# Return 1 if this virtual system can be failed over to a new host
sub type_xen_can_failover
{
my ($server, $newvhost) = @_;
if (!$server) {
	# Failovers are supported for Xen, in theory
	return 1;
	}
if ($server && $newvhost) {
	# Make sure disks exist on target
	return 0 if (!$server->{'data_files'} &&
		     !$server->{'checked_data_files'});
	foreach my $f (split(/\s+/, $server->{'data_files'})) {
		my @st = &stat_file_on_server($newvhost, $f);
		return 0 if (@st <= 1);
		}
	return 1;
	}
return 1;
}

# type_xen_get_failover_files(&server, save-as)
# Fetches from the host of some Xen instance the config files needed to
# perform a failover.
sub type_xen_get_failover_files
{
my ($server, $savefile) = @_;
my $vhost = &type_xen_parent($server);
return &get_file_from_server($vhost, $server->{'xen_conf'}, $savefile);
}

# type_xen_put_failover_files(&server, save-from)
# Uploads to the new host of some Xen instance the config files needed to
# start it up after a failover.
sub type_xen_put_failover_files
{
my ($server, $savefile) = @_;
my $vhost = &type_xen_parent($server);
return &put_file_to_server($vhost, $savefile, $server->{'xen_conf'});
}

# type_xen_get_data_files(&server)
# Returns a list of files/directories that this virtual system uses for
# storage. Used when failing over to verify that storage is shared.
sub type_xen_get_data_files
{
my ($server) = @_;
my $disks = &type_xen_list_disks($server, 0);
return ( ) if (!ref($disks));
return map { $_->{'real'} } grep { $_->{'iscsi_server'} eq '' } @$disks;
}

# type_xen_failover_system(&server, &dest)
# Move a Xen system to a new host, in the case where the old host is down.
# Returns undef on success, or an error message on failure.
sub type_xen_failover_system
{
my ($server, $dest) = @_;
my $vhost = &get_parent_server($server);

# Check for name clash on target
my $instances = &list_xen_instances($dest);
ref($instances) || return $instances;
my ($clash) = grep { $_->{'instance'} eq $server->{'xen'} } @$instances;
if ($clash) {
	return &text('smove_exenclash', $server->{'xen'});
	}

# Write out the Xen config file, if missing
&$first_print($text{'fail_xenconf'});
$server->{'xen_on'} = $dest->{'id'};
my $ffiles = $failover_files_dir."/".$server->{'id'};
my $err = &type_xen_put_failover_files($server, $ffiles);
if ($err) {
	$server->{'xen_on'} = $vhost->{'id'};
	return &text('fail_efiles', $err);
	}
else {
	# Fix VNC IP if needed
	my $conf = &get_xen_config($server, 0);
	my $vnclisten = &xen_find_value("vnclisten", $conf);
	if ($vnclisten && $vnclisten ne "0.0.0.0") {
		my $ip = &get_server_ip($newvhost);
		&xen_save_directive($conf, "vnclisten",
			$ip || &to_ipaddress($newvhost->{'host'}));
		}
	&put_xen_config($server, $conf);

	&$second_print(&text('fail_files', $server->{'xen_conf'}));
	}

return undef;
}

# type_xen_list_boot_modes(&server)
# Returns an array ref of available boot modes for some system, or an error
# message. Each entry is a hash ref with keys :
# id - Unique ID for this mode
# desc - Human-readable description
sub type_xen_list_boot_modes
{
my ($server) = @_;
if ($server->{'xen_hvm'}) {
	return undef;
	}
my $vhost = &type_xen_parent($server);
my $kdesc;
if ($vhost) {
	my $info = &get_server_collected($vhost);
	if ($info && $info->{'kernel'}) {
		$kdesc = " (".$info->{'kernel'}->{'os'}." ".
			      $info->{'kernel'}->{'version'}.")";
		}
	}
return [ { 'id' => 'host',
	   'desc' => 'Host system kernel'.$kdesc },
	 { 'id' => 'pygrub',
	   'desc' => 'Xen instance kernel with PyGrub' },
	 { 'id' => 'pvgrub',
	   'desc' => 'Xen instance kernel with Pv-Grub' },
       ];
}

# type_xen_get_boot_mode(&server)
# Returns a status code (1 or 0) and either the boot mode ID or an error message
sub type_xen_get_boot_mode
{
my ($server) = @_;
if ($server->{'xen_hvm'}) {
	return (0, "Boot mode cannot be set for HVM Xen instances");
	}
my $conf = &get_xen_config($server, 1);
return (0, $conf) if (!ref($conf));
my $kernel = &xen_find_value("kernel", $conf);
my $bootloader = &xen_find_value("bootloader", $conf);
if ($bootloader =~ /pygrub/) {
	return (1, 'pygrub');
	}
elsif ($kernel =~ /pv-grub/) {
	return (1, 'pvgrub');
	}
else {
	return (1, 'host');
	}
}

# type_xen_set_boot_mode(&server, mode)
# Sets the boot mode based on some ID. Returns undef on success, or an error
# message on failure.
sub type_xen_set_boot_mode
{
my ($server, $mode) = @_;
my $cfg = &get_xen_config($server, 0);
return $cfg if (!ref($cfg));
my $vhost = &type_xen_parent($server);
my $xenver = &get_xen_software_version($vhost);
if ($xenver >= 4 && $server->{'status'} ne 'down') {
	return "Under Xen 4 the boot mode of a running instance ".
	       "cannot be changed.";
	}

# Find GRUB config file
my $grubfile;
my $err = &mount_filesystem_if_needed($server);
return $err if ($err);
foreach my $f ("/boot/grub/grub.conf", "/boot/grub/menu.lst") {
	if (&stat_file_on_server($server, $f)) {
		$grubfile = $f;
		}
	}

# Copy kernel modules if needed
if ($mode eq "host") {
	my ($uout, $ex, $err) = &execute_command_on_server($vhost, "uname -r");
	if ($uout && !$ex && !$err) {
		$uout =~ s/\r|\n//g;
		my $moddir = "/lib/modules/$uout";
		my @st = &stat_file_on_server($server, $moddir);
		my $mp = $server->{'xen_mount'};
		if (!@st && $mp) {
			# Copy under the mount point
			&execute_command_on_server($vhost,
				"cp -r ".quotemeta($moddir)." ".
				 	 quotemeta($mp.$moddir));
			}
		elsif (!@st && !$mp) {
			# Transfer direct
			&transfer_file_between_servers($vhost, $server,
						       $moddir, $moddir);
			}
		}
	}
&umount_filesystem_if_needed($server);

if ($mode eq "host") {
	# Setup kernel on host
	my ($kok, $kfile, $ifile) = &setup_xen_host_kernel($vhost);
	$kok || return $kfile;
	&xen_save_directive($cfg, "kernel", $kfile);
	&xen_save_directive($cfg, "ramdisk", $ifile);
	&xen_save_directive($cfg, "root", "/dev/sda1 ro");
	&xen_save_directive($cfg, "extra");
	&xen_save_directive($cfg, "bootloader");
	delete($server->{'xen_bootloader'});
	$server->{'xen_pygrub'} = 0;
	}
elsif ($mode eq "pvgrub") {
	# Find pvgrub executable
	$grubfile || return "The system does not have its own kernel or GRUB ".
			    "configuration file";
	my $img = &get_server_image($server->{'image'});
	my $arch = $img->{'arch'} eq 'x86_64' ? '64' : '32';
	my $pvgrub = "/usr/lib/xen/boot/pv-grub-x86_${arch}.gz";
	if (!&stat_file_on_server($vhost, $pvgrub)) {
		return "Pv-Grub command $pvgrub was not found on the ".
		       "host system";
		}

	# Configure Xen to use it
	&xen_save_directive($cfg, "kernel", $pvgrub);
	&xen_save_directive($cfg, "extra", "(hd0)".$grubfile);
	&xen_save_directive($cfg, "root");
	&xen_save_directive($cfg, "ramdisk");
	&xen_save_directive($cfg, "bootloader");
	$server->{'xen_bootloader'} = $pvgrub;
	$server->{'xen_pygrub'} = 1;
	}
elsif ($mode eq "pygrub") {
	# Find pygrub executable
	$grubfile || return "The system does not have its own kernel or GRUB ".
			    "configuration file";
	my $pygrub = &has_command_on_server($vhost, "pygrub");
	my $xenver = &get_xen_software_version($vhost, 0);
	$pygrub ||= &has_command_on_server($vhost,
                                           "/usr/lib/xen-$xenver/bin/pygrub");
	my $xenver2 = &get_xen_software_version($vhost, 1);
	$pygrub ||= &has_command_on_server($vhost,
                                           "/usr/lib/xen-$xenver2/bin/pygrub");
	$pygrub || return "PvGrub command was not found on the host system";

	# Configure Xen to use it
	&xen_save_directive($cfg, "bootloader", $pygrub);
	&xen_save_directive($cfg, "root");
	&xen_save_directive($cfg, "ramdisk");
	&xen_save_directive($cfg, "kernel");
	&xen_save_directive($cfg, "extra");
	$server->{'xen_bootloader'} = $pygrub;
	$server->{'xen_pygrub'} = 1;
	}
else {
	return "Unknown boot mode $mode";
	}

# Re-save Xen config file
my $err = &put_xen_config($server, $cfg);
return $err if ($err);

# For Xen 4, need to re-generate the config
if ($xenver >= 4 && $vhost->{'xen_api_type'} ne 'xl') {
	&run_xen_api_command($vhost, "xm delete $server->{'xen'}");
	&run_xen_api_command($vhost, "xm new $server->{'xen_conf'}");
	}

return undef;
}

# type_xen_list_boot_devices(&server)
# Returns an array ref of available boot devices for some system, or an error
# message. Each entry is a hash ref with keys :
# id - Unique ID for this mode
# desc - Human-readable description
sub type_xen_list_boot_devices
{
my ($server) = @_;
if (!$server->{'xen_hvm'}) {
	return undef;
	}
return [ { 'id' => 'c',
	   'desc' => 'Hard disk' },
	 { 'id' => 'd',
	   'desc' => 'CD or DVD drive' },
	 { 'id' => 'a',
	   'desc' => 'Floppy drive' },
	 { 'id' => 'n',
	   'desc' => 'Network' } ];
}

# type_xen_get_boot_device(&server)
# Returns a status code (1 or 0) and either the boot dev IDs or an error message
sub type_xen_get_boot_device
{
my ($server) = @_;
if (!$server->{'xen_hvm'}) {
	return (0, "Boot device can only be set for HVM Xen instances");
	}
my $conf = &get_xen_config($server, 1);
ref($conf) || return (0, $conf);
my $val = &xen_find_value("boot", $conf);
$val ||= "cda";
return (1, [ split(//, $val) ], 1);
}

# type_xen_set_boot_device(&server, &modes)
# Sets the boot device based on some ID. Returns undef on success, or an error
# message on failure.
sub type_xen_set_boot_device
{
my ($server, $devices) = @_;
my $conf = &get_xen_config($server, 0);
&xen_save_directive($conf, "boot", join("", @$devices));
return &put_xen_config($server, $conf);
}

# type_xen_can_graphical_password(&server)
# Returns 2, indicating that the VNC password can be set with a reboot
sub type_xen_can_graphical_password
{
my ($server) = @_;
if ($server->{'xen_vnc_display'} || $server->{'xen_vnc_unused'}) {
	return 2;
	}
return 0;
}

# type_xen_get_graphical_password(&server)
# Returns the VNC password
sub type_xen_get_graphical_password
{
my ($server) = @_;
return $server->{'xen_vnc_passwd'};
}

# type_xen_set_graphical_password(&server, pass)
# Updates the VNC password to use at next boot
sub type_xen_set_graphical_password
{
my ($server, $pass) = @_;
$pass =~ /,/ && return $text{'pass_evcomma'};
my $conf = &get_xen_config($server);
return $conf if (!ref($conf));
&xen_save_directive($conf, "vncpasswd", $pass);
my $err = &put_xen_config($server, $conf);
return $err if ($err);
if (!$server->{'xen_hvm'}) {
	&xen_save_directive($conf, "vfb",
		[ "type=vnc,vncunused=1,vncpasswd=$pass,vnclisten=0.0.0.0" ]);
	}
$server->{'xen_vnc_passwd'} = $pass;
return undef;
}

# type_xen_get_graphical_port(&server)
# Returns the VNC port number
sub type_xen_get_graphical_port
{
my ($server) = @_;
my $activeport = &get_active_vnc_port($server);
return $activeport ? $activeport :
       $server->{'xen_vnc_display'} ? 5900 + $server->{'xen_vnc_display'}
				    : undef;
}

# find_xen_hvmloader(&vhost)
# Returns the path to the hvmloader command on some system
sub find_xen_hvmloader
{
my ($vhost) = @_;
my $xenver = &get_xen_software_version($vhost, 0);
my $xenver2 = &get_xen_software_version($vhost, 1);
my @hvmpaths = ( "/usr/lib/xen/boot/hvmloader",
		 "/usr/lib/xen-$xenver/boot/hvmloader",
		 "/usr/lib/xen-$xenver2/boot/hvmloader",
		 "/usr/lib64/xen/boot/hvmloader",
                 "/usr/lib64/xen-$xenver/boot/hvmloader",
                 "/usr/lib64/xen-$xenver2/boot/hvmloader",
	       );
foreach my $p (@hvmpaths) {
	my @st = &stat_file_on_server($vhost, $p);
	if (@st && $st[7]) {
		return $p;
		}
	}
return undef;
}

# find_xen_qemu_dm(&vhost)
# Returns the path to the qemu-dm command on some host
sub find_xen_qemu_dm
{
my ($vhost) = @_;
my $xenver = &get_xen_software_version($vhost, 0);
my $xenver2 = &get_xen_software_version($vhost, 1);
my $info = &get_server_collected($vhost);
my $arch = $info->{'kernel'}->{'arch'} =~ /64/ ? "x86_64" : "i386";
my @dmpaths = ( "/usr/lib64/xen/bin/qemu-dm",
		"/usr/lib/xen/bin/qemu-dm",
		"/usr/lib64/xen-$xenver/bin/qemu-dm",
		"/usr/lib/xen-$xenver/bin/qemu-dm",
		"/usr/lib64/xen-$xenver2/bin/qemu-dm",
		"/usr/lib/xen-$xenver2/bin/qemu-dm",
		"/usr/bin/qemu-system-$arch" );
foreach my $p (@dmpaths) {
	my @st = &stat_file_on_server($vhost, $p);
	if (@st && $st[7]) {
		return $p;
		}
	}
return undef;
}

# find_xen_grub_bootloader(&vhost, &image)
# Returns the path to the pv-grub or pygrub bootloader on some host
sub find_xen_grub_bootloader
{
my ($vhost, $img) = @_;

# Use the forced bootloader
if ($config{'xen_bootloader'}) {
	return $config{'xen_bootloader'};
	}

# First look for Pv-Grub
my $arch = $img->{'arch'} eq 'x86_64' ? '64' : '32';
my $pvgrub = "/usr/lib/xen/boot/pv-grub-x86_${arch}.gz";
if (&stat_file_on_server($vhost, $pvgrub)) {
	return $pvgrub;
	}

# Fall back to checking for PyGrub
my $pygrub = &has_command_on_server($vhost, "pygrub");
if (!$pygrub) {
	# Try debian-style location
	my $xenver = &get_xen_software_version($vhost);
	$pygrub = &has_command_on_server($vhost,
			"/usr/lib/xen-$xenver/bin/pygrub");
	}
return $pygrub;
}

# compute_xen_disk_prefix(&disk, &vhost)
# Returns the prefix for a disk, like phy or file
sub compute_xen_disk_prefix
{
my ($disk, $vhost) = @_;
return $disk->{'real'} =~ /^\/dev\// ? "phy" :
       $disk->{'media'} eq "cdrom" ? "file" :
       $vhost->{'xen_filetype'} ? $vhost->{'xen_filetype'} : "file";
}

# wait_for_xen_block_detach(&vhost, xen-id, block-id)
# Wait for up to 10 seconds for a VM disk to detach
sub wait_for_xen_block_detach
{
my ($vhost, $xenid, $blockid) = @_;
for(my $i=0; $i<10; $i++) {
	my ($out, $ex, $err) = &run_xen_api_command($vhost,
		"xm block-list $xenid");
	if (!$ex && !$err) {
		my $found = 0;
		foreach my $l (split(/\r?\n/, $out)) {
			if ($l =~ /^(\d+)\s/ && $1 == $blockid) {
				$found = 1;
				}
			}
		return 1 if (!$found);
		}
	sleep(1);
	}
return 0;
}

# re_attach_xen_interface(&vhost, &server, &old-iface, &new-iface)
# Detach and re-attach a Xen interface with new settings. Returns an error
# message on failure.
sub re_attach_xen_interface
{
my ($vhost, $server, $oldiface, $iface) = @_;
my ($out, $ex, $err) = &run_xen_api_command($vhost,
    "xm network-detach $server->{'xen'} $oldiface->{'devidx'}");
if ($ex || $err) {
	return "Failed to detach network interface : ".
	       ($err || $out);
	}
sleep(1);	# Wait for removal to take effect
my ($out, $ex, $err) = &run_xen_api_command($vhost,
	"xm network-attach $server->{'xen'}".
	" mac=$iface->{'ether'}".
	($iface->{'bridge'} ?
		" bridge=$iface->{'bridge'}" : "").
	($iface->{'rate'} ?
		" rate=$iface->{'rate'}" : "").
	" ip=$iface->{'address'}");
if ($ex || $err) {
	return "Failed to connect network interface : ".
	       ($err || $out);
	}
return undef;
}

# check_xen_hvm_support(&vhost)
# Returns 1 if some host system supports HVM, 0 if not, or -1 if unknown
sub check_xen_hvm_support
{
my ($vhost) = @_;
my $fallback = -1;

# Check CPU from kernel
my ($out, $ex, $err) = &execute_command_on_server($vhost, "cat /proc/cpuinfo");
foreach my $l (split(/\r?\n/, $out)) {
	if ($l =~ /flags\s*:\s*(.*)/) {
		my @flags = split(/\s+/, $1);
		return 1 if(&indexof("vmx", @flags) >= 0 ||
			    &indexof("svm", @flags) >= 0);
		$fallback = 0;
		}
	}

# Check XM command capabilities
($out, $ex, $err) = &run_xen_api_command($vhost, "xm info ; xl info");
if ($out =~ /virt_caps\s*:.*hvm/) {
	return 1;
	}

return $fallback;
}

# type_xen_open_vnc_port(&server)
# Opens a VNC port on the master system which is connected to the VNC console
# on the VM. Returns a port number and password on success, or 0 and an error
# message on failure.
sub type_xen_open_vnc_port
{
my ($server) = @_;

my $vhost = &type_xen_parent($server);
my $activeport = &get_active_vnc_port($server);
if (!$server->{'xen_vnc_display'} && !$server->{'xen_vnc_unused'}) {
	# Not setup yet .. need to add perhaps
	my $conf = &get_xen_config($server, 1);
	my $vnclisten = &xen_find_value("vnclisten", $conf);
	my $vnc = &xen_find_value("vnc", $conf);
	my $vncunused = &xen_find_value("vncunused", $conf);
	my $vncdisplay = &xen_find_value("vncdisplay", $conf);
	my $vncpasswd = &xen_find_value("vncpasswd", $conf);
	my $listenok = &to_ipaddress($vnclisten) eq
		         &to_ipaddress($vhost->{'host'}) ||
		       $vnclisten eq "0.0.0.0";
	if ($listenok && ($vnc == 1 || $vnc eq '') && $vncdisplay) {
		# Already setup on a static port .. just record it
		$server->{'xen_vnc_display'} = $vncdisplay;
		$server->{'xen_vnc_passwd'} = $vncpasswd;
		&save_managed_server($server);
		}
	elsif ($listenok && $vnc && $vncunused && $activeport) {
		# Already setup on a dynamic port
		$server->{'xen_vnc_passwd'} = $vncpasswd;
		$server->{'xen_vnc_unused'} = 1;
		&save_managed_server($server);
		}
	else {
		# Setup needed .. probably have to reboot
		my $err = &setup_xen_vnc_console($server);
		if ($err) {
			return (0, $err);
			}
		else {
			return (0, $text{'vnc_needreboot'});
			}
		}
	}

# Check that the VNC port is really open on the Xen host, and connect to it
my $port = $activeport || 5900 + $server->{'xen_vnc_display'};
my $err = undef;
&open_socket($vhost->{'ip'}, $port, VNCSOCK, \$err);
if ($err) {
	return (0, &text('vnc_nolisten', $port, $vhost->{'host'}));
	}

my $masterport;
if ($vhost->{'id'} == 0) {
	# Special case - this box is the Xen host, so no forwarding needed
	$masterport = $port;
	close(VNCSOCK);
	}
else {
	# Pick a free port on the Cloudmin master, 5900+
	my $err;
	($masterport, $err) = &forward_vnc_port(5900, VNCSOCK);
        return (0, &text('vnc_eforward', $err)) if ($err);
	}
return ($masterport, $server->{'xen_vnc_passwd'});
}

# type_xen_supports_websockets(&server)
# Can this system accept a websockets VNC connection?
sub type_xen_supports_websockets
{
return 0;	# Never supported on Xen
}

# run_xen_api_command(&vhost, command, timeout)
# Runs a command on a vhost, and returns the same results as
# execute_command_on_server. May use "xl" if "xm" isn't installed.
sub run_xen_api_command
{
my ($vhost, $cmd, $timeout) = @_;
my $api = $vhost->{'xen_api_type'} || 'xm';
if ($cmd =~ /^(xm|xl)\s+(.*)/) {
	my $rest = $2;
	if ($api eq 'xl' && $1 eq 'xm') {
		# Make changes to new Xen API command format

		# New command has been effectively replaced by create
		$rest =~ s/^new/create/;

		if ($rest =~ /^block-attach/) {
			# Block-attach takes a relative path now, like xvda2
			# instead of /dev/xvda2
			$rest =~ s/\s\/dev\/(\S+)/ $1/g;
			}
		}
	$cmd = $api." ".$rest;
	}
return &execute_command_on_server($vhost, $cmd, $timeout);
}

# install_xen_host_software(&vhost)
# Performs all installation and configuration needed to setup a Xen host,
# printing stuff along the way. Returns 0 on failure, 1 on success, or 2 if
# success but a reboot is needed.
sub install_xen_host_software
{
my ($vhost) = @_;

# Install Webmin if missing
if (!&can_webmin_status($vhost)) {
	&$first_print($text{'xeninstall_webmin'});
	if ($vhost->{'status'} eq 'downwebmin' ||
	    $vhost->{'status'} eq 'nowebminlogin') {
		&$second_print($text{'xeninstall_ewebmindown'});
		return 0;
		}
	my ($src, $type, $err) = &get_install_webmin_source($vhost);
	if (!$src) {
		&$second_print($text{'xeninstall_ewebminsrc'});
		return 0;
		}
	my ($out, $err) = &install_webmin_server($vhost, $src, $type, 1);
	if ($err) {
		&$second_print(&text('xeninstall_ewebmin', $err));
		return 0;
		}
	&check_server_status($vhost);
	&save_managed_server($vhost);
	if (!$vhost->{'webminroot'}) {
		&$second_print($text{'xeninstall_ewebminroot'});
		return 0;
		}
	&$second_print($text{'create_done'});
	}

# Check status again, to ensure Webmin is accessible
&$first_print($text{'xeninstall_status'});
if (!&can_webmin_status($vhost)) {
	&$second_print(&text('xeninstall_estatus',&describe_status($vhost, 1)));
	return 0;
	}
&remote_foreign_require($vhost, "fdisk");
my @disks = &remote_foreign_call($vhost, "fdisk", "list_disks_partitions");
my $virtio = 0;
foreach my $d (@disks) {
	if ($d->{'device'} =~ /^\/dev\/vd/) {
		$virtio++;
		}
	}
if ($virtio) {
	&$second_print($text{'xeninstall_evirtio'});
	return 0;
	}
&$second_print($text{'create_done'});

my $xenbr = 0;
if (&redhat_server_type($vhost)) {
	# Install vital packages on Redhat
	&$first_print($text{'xeninstall_pkgs'});
	my ($ok, $err) = &install_package_on_server(
				$vhost, "wget lsof bzip2");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}
	my ($fok, $ferr) = &install_package_on_server($vhost, "fdisk");
	my ($pok, $perr) = &install_package_on_server($vhost, "parted");
	if (!$fok && !$pok) {
		&$second_print(&text('xeninstall_epkgs', $ferr."\n".$perr));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}

	# On Redhat and CentOS, enable the Xen kernel repo
	&$first_print($text{'xeninstall_kernelrepo'});
	my ($ok, $err) = &install_package_on_server($vhost, "centos-release-xen");
	if (!$ok) {
		&$second_print(&text('xeninstall_ekernelrepo', $err));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}

	# Install Xen packages
	&$first_print($text{'xeninstall_xen'});
	my ($ok, $err) = &install_package_on_server($vhost, "xen kernel");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}

	# Disable virbrN interfaces
	&$first_print($text{'xeninstall_virbr'});
	my $ifaces = &list_active_server_interfaces($vhost);
	my $virbr = 0;
	if (ref($ifaces)) {
		foreach my $iface (@$ifaces) {
			if ($iface->{'name'} =~ /^virbr/) {
				&delete_active_server_interface($vhost, $iface);
				$virbr++;
				}
			elsif ($iface->{'name'} =~ /^xenbr/) {
				$xenbr++;
				}
			}
		}
	my $virbr_file = "/etc/libvirt/qemu/networks/default.xml";
	my @st = &stat_file_on_server($vhost, $virbr_file);
	if (@st > 1) {
		&rename_file_on_server($vhost, $virbr_file,
					       $virbr_file.".disabled");
		&put_file_data_to_server($vhost, $virbr_file, "");
		}
	if ($virbr) {
		&$second_print(&text('xeninstall_virbrdone', $virbr));
		}
	else {
		&$second_print($text{'xeninstall_virbrnone'});
		}

	}
elsif (&debian_server_type($vhost)) {
	# Install vital packages on Debian
	&$first_print($text{'xeninstall_pkgs'});
	my ($ok, $err) = &install_package_on_server(
			$vhost, "wget", "lsof", "bzip2", "parted");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}

	# Install Xen packages
	&$first_print($text{'xeninstall_xen'});
	my ($ok, $err) = &install_package_on_server($vhost, "xen-tools", "bridge-utils");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}

	# Install more Xen packages with unpredictable names
	my $info = &get_server_collected($vhost);
	my $arch = $info->{'kernel'}->{'arch'} =~ /64/ ? "amd64" : "i386";
	($ok, $err) = &try_install_packages_on_server(
		$vhost, "ubuntu-xen-server", "xen-linux-system",
			"xen-system-$arch");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	($ok, $err) = &try_install_packages_on_server(
		$vhost, "xen-utils-3.2", "xen-utils-4.0", "xen-utils-4.6",
			"xen-utils");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	($ok, $err) = &try_install_packages_on_server(
		$vhost, "xen-hypervisor-3.2-1-$arch", "xen-hypervisor-3.2",
			"xen-hypervisor-4.0-$arch", "xen-hypervisor-4.6-$arch",
			"xen-hypervisor");
	if (!$ok) {
		&$second_print(&text('xeninstall_epkgs', $err));
		return 0;
		}
	&$second_print($text{'create_done'});

	# Configure Xen kernel to boot first
	&$first_print($text{'xeninstall_xengrub'});
	my ($out, $ex, $err) = &execute_command_on_server($vhost,
		"dpkg-divert --divert /etc/grub.d/08_linux_xen --rename /etc/grub.d/20_linux_xen && update-grub");
	if ($ex || $err) {
		&$second_print(&text('xeninstall_exengrub', $err || $out));
		return 0;
		}
	&$second_print($text{'create_done'});

	# Find Xen network bridges
	my $ifaces = &list_active_server_interfaces($vhost);
	if (ref($ifaces)) {
		foreach my $iface (@$ifaces) {
			if ($iface->{'name'} =~ /^xenbr/) {
				$xenbr++;
				}
			}
		}
	}
else {
	# Unknown OS type
	&$first_print(&text('xeninstall_etype',
			    &describe_os($vhost->{'type'})));
	return 0;
	}

# Enable Xen bridge (same for Redhat and Debian-based systems)
&$first_print(&text('xeninstall_xenbr', 'xenbr0'));
if ($xenbr) {
	&$second_print($text{'xeninstall_xenbrgot'});
	}
else {
	# Copy bridge creation script
	my $iface = &first_ethernet_iface($vhost);
	my $eth = $iface ? $iface->{'name'} : 'eth0';
	my $src = $module_root_directory."/create-net-bridge.pl";
	my $dst = $vhost->{'webminroot'}."/net/create-net-bridge.pl";
	my $err = &put_file_to_server($vhost, $src, $dst);
	if ($err) {
		&$second_print(&text('xeninstall_exenbrcopy', $err));
		return 0;
		}
	my $pp = &get_server_perl_path($server);
	my $cmd = $pp." ".$dst." --interface $eth --bridge xenbr0";
	my ($out, $ex, $err) = &execute_command_on_server($vhost, $cmd);
	if ($ex == 1 && !$err) {
		&$second_print($text{'xeninstall_xenbrreboot'});
		return 2;	# Need reboot to activate
		}
	elsif ($ex || $err) {
		&$second_print(&text('xeninstall_exenbr',$err || $out));
		return 0;
		}
	else {
		&$second_print($text{'create_done'});
		}
	}

# Running a Xen kernel? If not, return 2
my @st = &stat_file_on_server($vhost, "/proc/xen");
return @st > 1 ? 1 : 2;
}

# type_xen_validate(&server)
# Check if the Xen config file exists
sub type_xen_validate
{
my ($server) = @_;
my @rv;
my $conf = &get_xen_config($server, 1);
if (!ref($conf)) {
	push(@rv, &text('validate_exenconf', $conf));
	}
return @rv;
}

# type_xen_image_fields(&image)
# Returns a field for entering image-specific Xen options
sub type_xen_image_fields
{
my ($img) = @_;
return &ui_table_row($text{'eimage_xencfg'},
	&ui_textarea("xen_cfg", join("\n", split(/\t+/, $img->{'xen_cfg'})),
		     5, 60));
}

# type_xen_image_parse(&image, &in)
# Update an image object with input from type_xen_image_fields
sub type_xen_image_parse
{
my ($img, $in) = @_;
my @lines = grep { /\S/ } split(/\r?\n/, $in->{'xen_cfg'});
foreach my $l (@lines) {
	$l =~ /^\S+\s*=\s*\S/ || &error(&text('eimage_exencfg', "<tt>$l</tt>"));
	}
$img->{'xen_cfg'} = join("\t", @lines);
}

1;

